package com.kanglai.push.service.Impl;

import com.blankj.utilcode.util.EncodeUtils;
import com.blankj.utilcode.util.EncryptUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.entity.KiccUser;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.di.component.DaggerMapperComponent;
import com.kanglai.push.entity.SsoUser;
import com.kanglai.push.entity.TokenEnhancer;
import com.kanglai.push.entity.User;
import com.kanglai.push.mapper.LoginMapper;
import com.kanglai.push.service.LoginService;
import com.dolphin.core.constant.SecurityConstant;
import com.dolphin.core.http.api.ResultResponse;

import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;

import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import javax.inject.Inject;

import io.reactivex.Observable;

/**
 *<p>
 * 登录服务实现
 *</p>
 *
 * @Author: entfrm开发团队-王翔
 * @Date: 2022/6/28
 */
public class LoginServiceImpl implements LoginService {

    @Inject
    LoginMapper loginMapper;

    public LoginServiceImpl() {
        DaggerMapperComponent
                .builder()
                .appComponent(((AppApplication)Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public Observable<TokenEnhancer> login(KiccUser user,String identityProvider, String tenantId) {
        // 处理ASE加密
        byte[] passwordSecret = EncryptUtils.encryptAES(
                user.getPassword().getBytes(Charset.forName("UTF-8")),
                new SecretKeySpec(SecurityConstant.GATEWAY_ASE_ENCODE_SECRET.getBytes(), "AES").getEncoded(),
                "AES/CFB/NoPadding",
                new IvParameterSpec(SecurityConstant.GATEWAY_ASE_ENCODE_SECRET.getBytes()).getIV());
        Map<String, String> params = new HashMap(0);
        params.put("grant_type", "password");
        params.put("scope", "server");
        params.put("identityProvider",identityProvider);
        params.put("tenantId", tenantId);
        params.put("username", user.getUsername());
        params.put("password", EncodeUtils.base64Encode2String(passwordSecret));
        for (String var:params.values()) {
            System.err.println(var);
        }
        return loginMapper.login(params);
    }

    @Override
    public Observable<ResultResponse> logout() {
        return loginMapper.logout();
    }

    @Override
    public Observable<ResultResponse<User>> getUserInfo() {
        return loginMapper.getUserInfo();
    }

    @Override
    public Observable<ResultResponse> sendSmsCode(String phone) {
        return loginMapper.sendSmsCode(phone);
    }

    @Override
    public Observable<ResultResponse<User>> getPhoneRegister(Map<String, Object> fields) {
        return loginMapper.getPhoneRegister(fields);
    }

    @Override
    public Observable<ResultResponse> updateSsoUser(SsoUser ssoUser) {
        return loginMapper.updateSsoUser(ssoUser);
    }

}
