package com.kanglai.push.ui.activity;

import android.os.Bundle;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.blankj.utilcode.util.CollectionUtils;
import com.dolphin.core.base.BaseActivity;
import com.dolphin.core.widget.DefaultItemDecoration;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.databinding.ActivityAuditListBinding;
import com.kanglai.push.entity.LinkEntity;
import com.kanglai.push.ui.adapter.SearchRecyclerAdapter;
import com.kanglai.push.ui.vm.AuditListViewModel;
import com.kongzue.dialogx.DialogX;
import com.kongzue.dialogx.dialogs.MessageDialog;
import com.kongzue.dialogx.interfaces.OnDialogButtonClickListener;

/**
 * 好友审核列表
 * @Author: liusixiang007
 * @since: 2023/4/12
 */
public class AuditListActivity extends BaseActivity<ActivityAuditListBinding, AuditListViewModel> implements SearchRecyclerAdapter.EventListener{

    public RecyclerView mRecyclerView;
    public SearchRecyclerAdapter mAdapter;
    public EditText ed_search;
    public TextView tv_cancel;
    public LinearLayout empty_img;

    @Override
    public int setContentView(Bundle savedInstanceState) {
        return R.layout.activity_audit_list;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        super.mViewModel.mActivity = this;

        empty_img = findViewById(R.id.default_empty);

        mRecyclerView = findViewById(R.id.recycler_view);
        final SearchRecyclerAdapter searchRecyclerAdapter = new SearchRecyclerAdapter(CollectionUtils.newArrayList());
        searchRecyclerAdapter.setEventListener((SearchRecyclerAdapter.EventListener) this);
        mAdapter = searchRecyclerAdapter;
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(this);
        mRecyclerView.setLayoutManager(mLayoutManager);
        mRecyclerView.addItemDecoration(new DefaultItemDecoration(ContextCompat.getColor(this, R.color.common_divider_color)));
        mRecyclerView.setAdapter(mAdapter);
        mViewModel.loadHttp();

        ed_search = findViewById(R.id.find_edit_search);
        ed_search.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                if (!TextUtils.isEmpty(charSequence)) mAdapter.empty();
            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (TextUtils.isEmpty(editable)) mViewModel.loadAuditData();
                else mViewModel.filtrateList(editable);
            }
        });
        tv_cancel = findViewById(R.id.find_btn_cancel);
        tv_cancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mAdapter.empty();
                mViewModel.loadAuditData();
            }
        });
    }

    @Override
    public void onItemViewClicked(LinkEntity item) {
        // 审批
        MessageDialog.build()
                .setTheme(DialogX.THEME.LIGHT)
                .setTitle("关注审核")
                .setMessage(item.getName() + " ( " + item.getNikeName() + " ) 想要成为你的粉丝")
                .setOkButton("通 过", new OnDialogButtonClickListener<MessageDialog>() {
                    @Override
                    public boolean onClick(MessageDialog dialog, View v) {
                        mViewModel.auditRequest(item.getId(), "2");
                        return false;
                    }
                })
                .setCancelButton("驳 回", new OnDialogButtonClickListener<MessageDialog>() {
                    @Override
                    public boolean onClick(MessageDialog dialog, View v) {
                        mViewModel.auditRequest(item.getId(), "-1");
                        return false;
                    }
                })
                .show();
    }
}
