package com.kanglai.push.ui.activity;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.blankj.utilcode.util.CollectionUtils;
import com.bumptech.glide.Glide;
import com.dolphin.core.base.BaseActivity;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.databinding.ActivityBlackListBinding;
import com.kanglai.push.entity.PushBlacklist;
import com.kanglai.push.ui.adapter.PushBlackListRecyclerAdapter;
import com.kanglai.push.ui.vm.BlackListViewModel;
import com.kongzue.dialogx.DialogX;
import com.kongzue.dialogx.dialogs.FullScreenDialog;
import com.kongzue.dialogx.dialogs.MessageDialog;
import com.kongzue.dialogx.interfaces.OnBindView;
import com.kongzue.dialogx.interfaces.OnDialogButtonClickListener;
import com.scwang.smart.refresh.layout.api.RefreshLayout;

/**
 * 黑名单列表
 * @Author: liusixiang007
 * @since: 2023/6/28
 */
public class BlackListActivity extends BaseActivity<ActivityBlackListBinding, BlackListViewModel> implements PushBlackListRecyclerAdapter.EventListener{

    public PushBlackListRecyclerAdapter mAdapter;
    public RecyclerView mRecyclerView;
    public RefreshLayout mRefreshLayout;


    @Override
    public int setContentView(Bundle savedInstanceState) {
        return R.layout.activity_black_list;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        super.mViewModel.mActivity = this;

        mRefreshLayout = findViewById(R.id.refreshLayout_black_list);
        mRecyclerView = findViewById(R.id.recycler_view_black_list);

        final PushBlackListRecyclerAdapter pushBlackListRecyclerAdapter = new PushBlackListRecyclerAdapter(CollectionUtils.newArrayList());
        pushBlackListRecyclerAdapter.setEventListener(this);
        mAdapter = pushBlackListRecyclerAdapter;
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(this);
        mRecyclerView.setLayoutManager(mLayoutManager);
        mRecyclerView.setAdapter(mAdapter);

        mRefreshLayout.setOnRefreshListener(mViewModel::refresh);   // 刷新
        mRefreshLayout.setOnLoadMoreListener(mViewModel::loadMore); // 加载更多
        mRefreshLayout.autoRefresh();   // 自动刷新
    }

    @Override
    public void onItemOnClick(PushBlacklist item) {     /** 点击查看详情*/
        FullScreenDialog.show(new OnBindView<FullScreenDialog>(R.layout.item_full_dialog_fans) {
            @Override
            public void onBind(FullScreenDialog dialog, View v) {
                TextView name = v.findViewById(R.id.fans_name);
                name.setText(item.getNickName() + "【"+ item.getUserName() +"】");
                TextView dept = v.findViewById(R.id.fans_tv_01);
                dept.setText("所属部门：" + item.getDeptName());
                TextView phone = v.findViewById(R.id.fans_tv_02);
                phone.setText("联系电话：" + item.getPhone());
                ImageView avatar = v.findViewById(R.id.fans_avatar);
                Glide.with(getApplication())
                        .load(item.getAvatar())
                        .skipMemoryCache(true)
                        .centerCrop()
                        .override(300, 300)
                        .placeholder(R.drawable.icon_img_loading)    // 加载图片
                        .error(R.drawable.icon_links_avatar)    // 加载错误
                        .into(avatar);
                Button button02 = v.findViewById(R.id.fans_btn_02);
                button02.setVisibility(View.GONE);
                Button button01 = v.findViewById(R.id.fans_btn_01);
                button01.setText("移 除");
                button01.setBackgroundResource(R.drawable.icon_btn_red_bg);
                button01.setOnClickListener(view -> againConfirm(item, dialog));
            }
        });
    }

    /** 再次确认是否移除黑名单*/
    private void againConfirm(PushBlacklist item, FullScreenDialog fullScreenDialog) {
        MessageDialog.build()
                .setTheme(DialogX.THEME.LIGHT)
                .setTitle("警告")
                .setMessage("确认移除”"+item.getUserName()+"“？")
                .setOkButton("确认", new OnDialogButtonClickListener<MessageDialog>() {
                    @Override
                    public boolean onClick(MessageDialog dialog, View v) {
                        if (!TextUtils.isEmpty(item.getId())){
                            mViewModel.removePushBlack(item, fullScreenDialog);
                        }else {
                            ToastUtil.showBottomWarn("用户数据异常");
                            mRefreshLayout.autoRefresh();   // 刷新界面数据
                            fullScreenDialog.dismiss();
                        }
                        dialog.dismiss();
                        return false;
                    }
                })
                .setCancelButton("取 消", new OnDialogButtonClickListener<MessageDialog>() {
                    @Override
                    public boolean onClick(MessageDialog dialog, View v) {
                        dialog.dismiss();
                        return false;
                    }
                })
                .show();
    }
}
