package com.kanglai.push.ui.activity;

import android.os.Bundle;
import android.text.Editable;
import android.text.InputFilter;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.blankj.utilcode.util.CollectionUtils;
import com.bumptech.glide.Glide;
import com.dolphin.core.base.BaseActivity;
import com.dolphin.core.widget.DefaultItemDecoration;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.databinding.ActivityEnterpriseListBinding;
import com.kanglai.push.entity.LinkEntity;
import com.kanglai.push.ui.adapter.SearchRecyclerAdapter;
import com.kanglai.push.ui.vm.EnterpriseListViewModel;
import com.kanglai.push.util.ViewFilterUtil;
import com.kongzue.dialogx.dialogs.FullScreenDialog;
import com.kongzue.dialogx.interfaces.OnBindView;

/**
 * 企业列表
 * @Author: liusixiang007
 * @since: 2023/2/24
 */
public class EnterpriseListActivity extends BaseActivity<ActivityEnterpriseListBinding, EnterpriseListViewModel> implements SearchRecyclerAdapter.EventListener{

    public SearchRecyclerAdapter mAdapter;

    public RecyclerView mRecyclerView;

    public EditText ed_search; // 搜索框

    public TextView tv_cancel;  // 取消按钮
    public LinearLayout empty_img;

    @Override
    public int setContentView(Bundle savedInstanceState) {
        return R.layout.activity_enterprise_list;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        super.mViewModel.mActivity = this;

        empty_img = findViewById(R.id.default_empty);
        mRecyclerView = findViewById(R.id.recycler_view);
        final SearchRecyclerAdapter searchRecyclerAdapter = new SearchRecyclerAdapter(CollectionUtils.newArrayList());
        searchRecyclerAdapter.setEventListener((SearchRecyclerAdapter.EventListener) this);
        mAdapter = searchRecyclerAdapter;
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(this);
        mRecyclerView.setLayoutManager(mLayoutManager);
        mRecyclerView.addItemDecoration(new DefaultItemDecoration(ContextCompat.getColor(this, R.color.common_divider_color)));
        mRecyclerView.setAdapter(mAdapter);

        mViewModel.loadEnterpriseData(); // 加载企业列表数据

        /** 顶部搜索框 */
        ed_search = findViewById(R.id.find_edit_search);
        ed_search.setFilters(new InputFilter[]{ViewFilterUtil.getNoSpecialCharactersFilter()});  // 去空格、换行符等特殊符号
        ed_search.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {}

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                if (!TextUtils.isEmpty(charSequence)) {
                    mAdapter.empty();
                }
            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (TextUtils.isEmpty(editable)) mViewModel.loadAllData();
                else mViewModel.filtrateList(editable);
            }
        });

        tv_cancel = findViewById(R.id.find_btn_cancel);
        tv_cancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) { // 取消按钮
                mAdapter.empty();
                mViewModel.loadAllData();
            }
        });
    }

    @Override
    public void onItemViewClicked(LinkEntity linkEntity) {
        // 关注企业
        FullScreenDialog.show(new OnBindView<FullScreenDialog>(R.layout.item_full_dialog_fans) {
            @Override
            public void onBind(FullScreenDialog dialog, View v) {
                TextView name = v.findViewById(R.id.fans_name);
                name.setText(linkEntity.getName());
                TextView dept = v.findViewById(R.id.fans_tv_01);
                dept.setVisibility(View.GONE);
                TextView phone = v.findViewById(R.id.fans_tv_02);
                phone.setText("地址："+linkEntity.getDetailedAddress());
                ImageView avatar = v.findViewById(R.id.fans_avatar);
                // 头像格式校验
                String avater = linkEntity.getAvatar();
//                if (!Patterns.WEB_URL.matcher(avater).matches()) avater = BuildConfig.HOST_URL + linkEntity.getAvatar().substring(1);

                // todo 企业列表接口暂未提供企业列表头像 - 企业列表功能 - 供客服审核企业信息(同意 / 驳回)
                Glide.with(getApplication())
                        .load("")
                        .skipMemoryCache(true)
                        .centerCrop()
                        .override(300, 300)
                        .placeholder(R.drawable.icon_links_avatar)    // 加载图片
                        .error(R.drawable.icon_links_avatar)    // 加载错误
                        .into(avatar);
                Button button01 = v.findViewById(R.id.fans_btn_01);
                button01.setText("消 息 记 录");
                button01.setOnClickListener(view -> msgHistory(linkEntity));
                Button button02 = v.findViewById(R.id.fans_btn_02);
                button02.setVisibility(View.GONE);
            }

        });
    }

    /** 消息记录 */
    private void msgHistory(LinkEntity item) {
        // 企业列表 单列点击事件
        /**
         * Bundle bundle = new Bundle();
         * bundle.putParcelable("GOING_TO_INSTANT", new LocalPushChatMsg("", item.getId(), item.getAvatar(), item.getName(), "2", "1"));    // 默认为企业用户
         * startActivity(InstantActivity.class, bundle);
         *
         */
    }
}
