package com.kanglai.push.ui.activity;

import android.content.Context;
import android.os.Bundle;
import android.text.Editable;
import android.text.InputFilter;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.Patterns;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.CollectionUtils;
import com.blankj.utilcode.util.StringUtils;
import com.bumptech.glide.Glide;
import com.dolphin.core.BuildConfig;
import com.dolphin.core.base.BaseActivity;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.databinding.ActivityFindUsersBinding;
import com.kanglai.push.entity.ConcernFan;
import com.kanglai.push.entity.LinkEntity;
import com.kanglai.push.entity.User;
import com.kanglai.push.ui.adapter.SearchRecyclerAdapter;
import com.kanglai.push.ui.vm.FindUsersViewModel;
import com.kanglai.push.util.ViewFilterUtil;
import com.kongzue.dialogx.dialogs.FullScreenDialog;
import com.kongzue.dialogx.interfaces.OnBindView;

/**
 * 查找各类用户
 * @Author: liusixiang007
 * @since: 2023/2/20
 */
public class FindUsersActivity extends BaseActivity<ActivityFindUsersBinding, FindUsersViewModel> implements SearchRecyclerAdapter.EventListener{

    public EditText ed_search;
    public TextView tv_cancel;
    public TextView tv_name;
    public LinearLayout ly_recommend;   // 搜索推荐内容
    public LinearLayout ly_result;

    /** 最终搜索结果列表 */
    public RecyclerView mRecyclerView;
    public SearchRecyclerAdapter mAdapter;

    public RelativeLayout user_search_btn;  // 用户名点击搜索按钮
    public TextView tv_val_user;

    public RelativeLayout firm_search_btn;  // 公司点击搜索按钮
    public TextView tv_val_firm;
    public FullScreenDialog mDialog;

    public static String ADD_LINK_BUNDLE = "ADD_LINK_BUNDLE";
    @Override
    public int setContentView(Bundle savedInstanceState) {
        return R.layout.activity_find_users;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        super.mViewModel.mActivity = this;
        mViewModel.loadUserList();
        mViewModel.loadFirmList();

        ed_search = findViewById(R.id.find_edit_search);
        ed_search.setFilters(new InputFilter[]{ViewFilterUtil.getNoSpecialCharactersFilter()});
        tv_cancel = findViewById(R.id.find_btn_cancel);
        tv_name = findViewById(R.id.user_name);
        ly_recommend = findViewById(R.id.recommend_layout);
        ly_result = findViewById(R.id.result_layout);

        user_search_btn = findViewById(R.id.rl_btn_user);   // 触发用户搜索
        tv_val_user = findViewById(R.id.tv_val_user);
        firm_search_btn = findViewById(R.id.rl_btn_firm);   // 触发公司搜索
        tv_val_firm = findViewById(R.id.tv_val_firm);

        User user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, User.CREATOR);
        if (user != null) tv_name.setText("当前登录账号：" + user.getNickName());
        else tv_name.setText("");

        mRecyclerView = findViewById(R.id.recycler_view);
        // 结果列表初始化
        final SearchRecyclerAdapter searchRecyclerAdapter = new SearchRecyclerAdapter(CollectionUtils.newArrayList());
        searchRecyclerAdapter.setEventListener((SearchRecyclerAdapter.EventListener) this);
        mAdapter = searchRecyclerAdapter;
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(this);
        mRecyclerView.setLayoutManager(mLayoutManager);
        mRecyclerView.setAdapter(mAdapter);

        /** 文字搜索框修改事件 */
        ed_search.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                // 修改前
            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                if (TextUtils.isEmpty(charSequence)) {
                    // 默认状态|输入框为空时 用户、企业搜索推荐 当前登录账号 输入推荐
                    tv_cancel.setVisibility(View.GONE);
                    ly_recommend.setVisibility(View.VISIBLE);
                    ly_result.setVisibility(View.GONE);
                    user_search_btn.setVisibility(View.GONE);
                    firm_search_btn.setVisibility(View.GONE);
                }else { // 非空
                    tv_cancel.setVisibility(View.VISIBLE);  // 显示取消按钮
                    ly_recommend.setVisibility(View.GONE);  // 隐藏推荐
                    ly_result.setVisibility(View.VISIBLE);   // 显示结果
                    user_search_btn.setVisibility(View.VISIBLE);
                    firm_search_btn.setVisibility(View.VISIBLE);
                }
                mAdapter.empty();   // 每次修改输入框都需要将搜索结果清空
            }

            @Override
            public void afterTextChanged(Editable editable) {   // 输入框文字修改
                // 修改之后
                if (!TextUtils.isEmpty(editable)) {
                    tv_val_user.setText(editable);
                    tv_val_firm.setText(editable);
                }
            }
        });
        tv_cancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                // 隐藏取消按钮
                tv_cancel.setVisibility(View.GONE);
                ed_search.setText("");
            }
        });

        /** 用户搜索点击事件 */
        user_search_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                /** 用户点击搜索 */
                mViewModel.filterUser(ed_search.getText());
                user_search_btn.setVisibility(View.GONE);
                firm_search_btn.setVisibility(View.GONE);
            }
        });
        firm_search_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                /** 企业点击搜索 */
                mViewModel.filterEnterprise(ed_search.getText());
                user_search_btn.setVisibility(View.GONE);
                firm_search_btn.setVisibility(View.GONE);
            }
        });

        mDialog = FullScreenDialog.build();

    }

    /** 通过全屏对话框 来展示目标用户数据 */
    @Override
    public void onItemViewClicked(LinkEntity item) {
        hideKeyboard();
        mDialog.show(new OnBindView<FullScreenDialog>(R.layout.item_full_dialog_fans) {
            @Override
            public void onBind(FullScreenDialog dialog, View v) {
                // 加载头像
                ImageView img = v.findViewById(R.id.fans_avatar);
                String avatar = "";
                if (!StringUtils.isEmpty(item.getAvatar())){
                    avatar = item.getAvatar();
                    if (!Patterns.WEB_URL.matcher(avatar).matches()) avatar = BuildConfig.HOST_URL + item.getAvatar().substring(1);
                }
                Glide.with(getApplication())
                        .load(avatar)
                        .skipMemoryCache(true)
                        .centerCrop()
                        .override(300, 300)
                        .placeholder(R.drawable.icon_img_loading)    // 加载图片
                        .error(R.drawable.icon_links_avatar)    // 加载错误
                        .into(img);
                TextView name = v.findViewById(R.id.fans_name);
                TextView text01 = v.findViewById(R.id.fans_tv_01);
                TextView text02 = v.findViewById(R.id.fans_tv_02);
                Button button01 = v.findViewById(R.id.fans_btn_01);
                Button button02 = v.findViewById(R.id.fans_btn_02);
                button02.setVisibility(View.GONE);
                if (item.isFirm()) {    // 企业
                    name.setText(item.getName());
                    text01.setText(item.getDetailedAddress());
                    text02.setVisibility(View.GONE);
                    button01.setVisibility(View.GONE);
                }else { // 用户
                    name.setText(item.getNikeName()+"【"+ item.getName() +"】");
                    text01.setText("所属部门："+item.getDeptName());
                    text02.setText("联系电话："+item.getPhone());
                    button01.setText("关 注");
                    button01.setOnClickListener(view -> mViewModel.attentionUser(item.getId(), dialog));
                    determineCFType(item, v);
                }
            }
        });
    }

    /**
     * 判断用户是否已关注
     * @param item
     * @param v
     */
    private void determineCFType(LinkEntity item, View v) {
        Button button01 = v.findViewById(R.id.fans_btn_01);
        for (ConcernFan fans :  mViewModel.fansList){
            if (item.getId().equals(fans.getId()) && "2".equals(fans.getCfStatus())) {
                button01.setEnabled(false);
                button01.setText("已 关 注");
                button01.setBackgroundResource(R.drawable.icon_btn_gray_bg);
            }
        }
    }

    /**隐藏键盘 */
    public void hideKeyboard() {
        InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
        View currentFocusView = getCurrentFocus();
        if (currentFocusView != null) imm.hideSoftInputFromWindow(currentFocusView.getWindowToken(), 0);
        else imm.hideSoftInputFromWindow(getWindow().getDecorView().getWindowToken(), 0);
    }
}
