package com.kanglai.push.ui.activity;

import android.os.Bundle;
import android.text.TextUtils;
import android.util.Patterns;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.blankj.utilcode.util.CollectionUtils;
import com.bumptech.glide.Glide;
import com.dolphin.core.BuildConfig;
import com.dolphin.core.base.BaseActivity;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.databinding.ActivityFocusDetailBinding;
import com.kanglai.push.entity.ConcernFans;
import com.kanglai.push.entity.PushType;
import com.kanglai.push.ui.adapter.PushTypeRecyclerAdapter;
import com.kanglai.push.ui.vm.FocusDetailViewModel;
import com.kongzue.dialogx.DialogX;
import com.kongzue.dialogx.dialogs.FullScreenDialog;
import com.kongzue.dialogx.dialogs.MessageDialog;
import com.kongzue.dialogx.interfaces.OnBindView;
import com.kongzue.dialogx.interfaces.OnDialogButtonClickListener;
import com.suke.widget.SwitchButton;

/**
 * 关注用户列表
 *
 * 含用户信息消息记录、取关、推送列表
 * @Author: liusixiang007
 * @since: 2023/6/29
 */
public class FocusDetailActivity extends BaseActivity<ActivityFocusDetailBinding, FocusDetailViewModel>  implements PushTypeRecyclerAdapter.EventListener{

    public Button cancel_focus;     // 取关按钮
    public PushTypeRecyclerAdapter mAdapter;
    public RecyclerView mRecyclerView;
    public TextView listTit;

    @Override
    public int setContentView(Bundle savedInstanceState) {
        return R.layout.activity_focus_detail;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        super.mViewModel.mActivity = this;

        final PushTypeRecyclerAdapter pushTypeRecyclerAdapter = new PushTypeRecyclerAdapter(CollectionUtils.newArrayList());
        pushTypeRecyclerAdapter.setEventListener(this);
        mAdapter = pushTypeRecyclerAdapter;
        mRecyclerView = findViewById(R.id.focus_detail_recycler_view);
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(this);
        mRecyclerView.setLayoutManager(mLayoutManager);
        mRecyclerView.setAdapter(mAdapter);

        ConcernFans item = getIntent().getParcelableExtra("FOCUS_FENS");      // 接收跳转过来的值
        if (item instanceof ConcernFans) {
            mViewModel.followFans = item;
            renderViews(item);
            mViewModel.loadPushType();
        } else {
            finish();
            ToastUtil.showBottomWarn("用户数据异常请重试");
        }
    }

    private void renderViews(ConcernFans item) {    // 渲染接口控件数据
        TextView name = findViewById(R.id.tv_focus_name);
        name.setText(item.getUserName());
        TextView phone = findViewById(R.id.tv_focus_phone);
        phone.setText("电话:"+item.getPhone());
        TextView dept = findViewById(R.id.tv_focus_dept_name);
        dept.setText("所属部门:"+item.getDeptName());
        ImageView man = findViewById(R.id.iv_sex_0);
        ImageView woman = findViewById(R.id.iv_sex_1);
        if (!TextUtils.isEmpty(item.getSex()) && "0".equals(item.getSex())) man.setVisibility(View.VISIBLE);
        else  if (!TextUtils.isEmpty(item.getSex()) && "1".equals(item.getSex())) woman.setVisibility(View.VISIBLE);
        ImageView avatar = findViewById(R.id.iv_focus_avatar);
        if (!TextUtils.isEmpty(item.getAvatar()) && !Patterns.WEB_URL.matcher(item.getAvatar()).matches()) {
            item.setAvatar(BuildConfig.HOST_URL + item.getAvatar().substring(1));
        }
        Glide.with(getApplication())
                .load(item.getAvatar())
                .skipMemoryCache(true)
                .centerCrop()
                .override(400, 400)
                .placeholder(R.drawable.icon_img_loading)    // 加载图片
                .error(R.drawable.icon_links_avatar)    // 加载错误
                .into(avatar);
        listTit = findViewById(R.id.tv_tit);    // 标题
    }

    /** 取关确认*/
    private void verifyTit(ConcernFans item) {
        MessageDialog.build()
                .setTheme(DialogX.THEME.LIGHT)
                .setTitle("警告")
                .setMessage("确认取消对”"+item.getUserName()+"“的关注，同时清除与该用户的聊天记录")
                .setOkButton("确认", new OnDialogButtonClickListener<MessageDialog>() {
                    @Override
                    public boolean onClick(MessageDialog dialog, View v) {
//                        mViewModel.cancelFocus(item);
                        return false;
                    }
                })
                .setCancelButton("取 消", new OnDialogButtonClickListener<MessageDialog>() {
                    @Override
                    public boolean onClick(MessageDialog dialog, View v) {
                        dialog.dismiss();
                        return false;
                    }
                })
                .show();
    }

    @Override
    public void onItemViewClicked(PushType pushType) {
        // 查看单列设置
        FullScreenDialog.show(new OnBindView<FullScreenDialog>(R.layout.item_full_dialog_push) {
            @Override
            public void onBind(FullScreenDialog dialog, View v) {
                TextView tit = v.findViewById(R.id.dialog_tit);
                tit.setText(pushType.getName());
                EditText input = v.findViewById(R.id.edit_type_name);   //  类型名称
                input.setVisibility(View.GONE);     // 隐藏输入框
                SwitchButton switch01 = v.findViewById(R.id.switch_button_01);  // 声音
                switch01.setChecked("1".equals(pushType.getPlaySound()) ? true : false);
                switch01.setEnabled(false);
                SwitchButton switch02 = v.findViewById(R.id.switch_button_02);  // 震动
                switch02.setChecked("1".equals(pushType.getPlayVibrate()) ? true : false);
                switch02.setEnabled(false);
                SwitchButton switch03 = v.findViewById(R.id.switch_button_03);  // 闪光
                switch03.setChecked("1".equals(pushType.getPlayLights()) ? true : false);
                switch03.setEnabled(false);
                SwitchButton switch04 = v.findViewById(R.id.switch_button_04);  // 语音转文字
                switch04.setChecked("1".equals(pushType.getPlayToText())?true:false);
                switch04.setEnabled(false);
                Button editor = v.findViewById(R.id.dialog_type_button);
                editor.setVisibility(View.GONE);    // 隐藏提交按钮
            }
        });
    }

    @Override
    public void onSwitchButtonCheck(PushType pushType, SwitchButton view) {
        // 修改任意一项
        MessageDialog.show("警 告","确认要启用【"+pushType.getName()+"】为默认类型?", "确 定", "取 消")
                .setOkButton(new OnDialogButtonClickListener<MessageDialog>() {
                    @Override
                    public boolean onClick(MessageDialog dialog, View v) {
                        mViewModel.setDefaultType(pushType, dialog);
                        return true;
                    }
                })
                .setCancelButton(new OnDialogButtonClickListener<MessageDialog>() {
                    @Override
                    public boolean onClick(MessageDialog dialog, View v) {
                        view.setChecked(false);     // 取消后恢复初始状态
                        return false;
                    }
                })
                .show();
    }
}
