package com.kanglai.push.ui.activity;

import android.os.Bundle;
import android.text.Editable;
import android.text.InputFilter;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.core.content.ContextCompat;
import androidx.databinding.library.baseAdapters.BR;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.blankj.utilcode.util.CollectionUtils;
import com.dolphin.core.base.BaseActivity;
import com.dolphin.core.widget.DefaultItemDecoration;
import com.kanglai.push.R;
import com.kanglai.push.databinding.ActivityFocusListBinding;
import com.kanglai.push.entity.ConcernFan;
import com.kanglai.push.entity.ConcernFans;
import com.kanglai.push.ui.adapter.MyFocusRecycleAdapter;
import com.kanglai.push.ui.vm.FocusListViewModel;
import com.kanglai.push.util.ViewFilterUtil;

/**
 * 已关注的用户列表
 * @Author: liusixiang007
 * @since: 2023/4/11
 */
public class FocusListActivity extends BaseActivity<ActivityFocusListBinding, FocusListViewModel> implements MyFocusRecycleAdapter.EventListener{

    public MyFocusRecycleAdapter mAdapter;
    public RecyclerView mRecyclerView;
    public EditText ed_search; // 搜索框
    public TextView tv_cancel;  // 取消按钮
    public LinearLayout empty_img;

    @Override
    public int setContentView(Bundle savedInstanceState) {
        return R.layout.activity_focus_list;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        super.mViewModel.mActivity = this;

        empty_img = findViewById(R.id.default_empty);
        mRecyclerView = findViewById(R.id.recycler_view);
        final MyFocusRecycleAdapter myFocusRecycleAdapter = new MyFocusRecycleAdapter(CollectionUtils.newArrayList());
        myFocusRecycleAdapter.setEventListener(this);
        mAdapter = myFocusRecycleAdapter;
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(this);
        mRecyclerView.setLayoutManager(mLayoutManager);
        mRecyclerView.addItemDecoration(new DefaultItemDecoration(ContextCompat.getColor(this, R.color.common_divider_color)));
        mRecyclerView.setAdapter(mAdapter);

        /** 顶部搜索框 */
        ed_search = findViewById(R.id.find_edit_search);
        ed_search.setFilters(new InputFilter[]{ViewFilterUtil.getNoSpecialCharactersFilter()});  // 去空格、换行符等特殊符号
        ed_search.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {}

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                if (!TextUtils.isEmpty(charSequence)) {
                    mAdapter.empty();
                }
            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (TextUtils.isEmpty(editable)) mViewModel.refreshView();
                else mViewModel.filtrateList(editable);
            }
        });

        tv_cancel = findViewById(R.id.find_btn_cancel);
        tv_cancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mAdapter.empty();
                mViewModel.refreshView();
            }
        });
    }


    @Override
    public void onItemViewClicked(ConcernFan fans) {
        /** 进入关注用户详情页 */
        Bundle bundle = new Bundle();
        ConcernFans item = new ConcernFans(fans.getId(), fans.getAvatar(), fans.getUserName(), fans.getPhone(), fans.getSex(), fans.getUserType(), fans.getDeptName() ,fans.getConcernUserId());
        item.setFanUserId(fans.getFanUserId());
        bundle.putParcelable("FOCUS_FENS", item);
        startActivity(FocusDetailActivity.class, bundle);
    }

    @Override
    protected void onResume() {
        super.onResume();
        mViewModel.loadFocusList();  // 每次当当前界面置于前台都 加载并且刷新一次数据
    }
}
