package com.kanglai.push.ui.activity;

import android.content.Context;
import android.content.Intent;
import android.icu.text.SimpleDateFormat;
import android.icu.util.Calendar;
import android.os.Bundle;
import android.text.InputFilter;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.GridLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.dolphin.core.base.BaseActivity;
import com.dolphin.core.bus.RxBus;
import com.dolphin.core.bus.RxSubscriptions;
import com.dolphin.core.util.RxUtil;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.constant.CommonConstant;
import com.kanglai.push.databinding.ActivityInstantBinding;
import com.kanglai.push.entity.LocalPushChatMsg;
import com.kanglai.push.entity.PushChatMessage;
import com.kanglai.push.ui.adapter.ScrollBoundaryDeciderAdapter;
import com.kanglai.push.ui.adapter.SoloChatRoomAdapter;
import com.kanglai.push.ui.vm.InstantViewModel;
import com.kanglai.push.util.ViewFilterUtil;
import com.scwang.smart.refresh.layout.api.RefreshLayout;

import java.util.Locale;
import java.util.Objects;

import io.reactivex.disposables.Disposable;

/**
 * 即时通讯界面
 * @Author: liusixiang007
 * @since: 2023/5/17
 */
public class InstantActivity extends BaseActivity<ActivityInstantBinding, InstantViewModel>{

    public SoloChatRoomAdapter mAdapter;
    public RefreshLayout refreshLayout;
    public RecyclerView mRecyclerView;
    public EditText input_box;
    public ImageView more_line;
    public ImageView more_fill;
    public GridLayout bottom_grid;
    public LinearLayout layout_footer;
    public Boolean isShowBottom = false;
    private Disposable mSubscription;

    @Override
    public int setContentView(Bundle savedInstanceState) {
        return R.layout.activity_instant;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        super.mViewModel.mActivity = this;
        mViewModel.localPushChatMsg = getIntent().getParcelableExtra(CommonConstant.INSTANT_LAUNCHER_RESULT_KEY);
        mAdapter = new SoloChatRoomAdapter(mViewModel.localPushChatMsg);
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(this);
        mRecyclerView = findViewById(R.id.instant_recycler);
        mRecyclerView.setLayoutManager(mLayoutManager);
        mRecyclerView.setAdapter(mAdapter);
        refreshLayout = findViewById(R.id.instant_smart_refresh);
        refreshLayout.setEnableRefresh(false);
        refreshLayout.setEnableAutoLoadMore(false);
        refreshLayout.setEnableNestedScroll(false);
        refreshLayout.setEnableScrollContentWhenLoaded(true);
        refreshLayout.setScrollBoundaryDecider(new ScrollBoundaryDeciderAdapter() {
            @Override
            public boolean canLoadMore(View content) {
                return super.canRefresh(content);
            }
        });
        refreshLayout.setOnLoadMoreListener(mViewModel::loadMore);
        input_box = findViewById(R.id.instant_input_box);
        input_box.setFilters(new InputFilter[]{ViewFilterUtil.getNoSpecialCharactersFilter()});
        more_line = findViewById(R.id.iv_more_line);
        more_fill = findViewById(R.id.iv_more_fill);
        more_line.setOnClickListener(view -> changBottomType());
        more_fill.setOnClickListener(view -> changBottomType());
        bottom_grid = findViewById(R.id.instant_grid);
        layout_footer = findViewById(R.id.layout_footer);
        mViewModel.setTitleText(mViewModel.localPushChatMsg.getUserName());
        if ("2".equals(mViewModel.localPushChatMsg.getUserType()) || "5".equals(mViewModel.localPushChatMsg.getUserType()) || "0".equals(mViewModel.localPushChatMsg.getUserType())){
            layout_footer.setVisibility(View.INVISIBLE);
        }
        mViewModel.initData(refreshLayout);
        mSubscription = RxBus.getInstance().toObservable(com.dolphin.umeng.entity.PushChatMessage.class)
        .compose(RxUtil.schedulersTransformer())
        .compose(RxUtil.exceptionTransformer())
        .subscribe(msg -> {
            com.dolphin.umeng.entity.PushChatMessage pushChatMessage = (com.dolphin.umeng.entity.PushChatMessage) msg;
            PushChatMessage chatMessage = new PushChatMessage();
            chatMessage.setUserId(pushChatMessage.getFanUserId());
            chatMessage.setText(pushChatMessage.getText());
            Calendar calendar = Calendar.getInstance();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault());
            String currentTime = sdf.format(calendar.getTime());
            chatMessage.setCreateTime(currentTime);
            mAdapter.append(chatMessage);
            mRecyclerView.scrollToPosition(mAdapter.getItemCount() - 1);
            setActivitieResult();
        });
        RxSubscriptions.add(mSubscription);
    }


    public void setActivitieResult() {
        if (!mAdapter.mItemList.isEmpty()) {
            setResult(RESULT_OK, new Intent().putExtra(CommonConstant.INSTANT_LAUNCHER_RESULT_KEY, new LocalPushChatMsg()
                    .setId(mViewModel.localPushChatMsg.getId())
                    .setUserName(mViewModel.localPushChatMsg.getUserName())
                    .setUserType(mViewModel.localPushChatMsg.getUserType())
                    .setAvatar(mViewModel.localPushChatMsg.getAvatar())
                    .setSendTime(mAdapter.mItemList.get(mAdapter.getItemCount() - 1).getCreateTime())
                    .setLastContent(mAdapter.mItemList.get(mAdapter.getItemCount() - 1).getText())
            ));
        }
    }

    /** 底部功能模块显示开关 */
    private void changBottomType(){
        isShowBottom = !isShowBottom;
        if (isShowBottom) {
            // 展示
            hideKeyboard();
            more_fill.setVisibility(View.VISIBLE);
            more_line.setVisibility(View.GONE);
            bottom_grid.setVisibility(View.VISIBLE);
        }else {
            // 隐藏
            more_fill.setVisibility(View.GONE);
            more_line.setVisibility(View.VISIBLE);
            bottom_grid.setVisibility(View.GONE);
        }
    }

    public void hideKeyboard() {
        InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
        View currentFocusView = getCurrentFocus();
        imm.hideSoftInputFromWindow(Objects.requireNonNullElseGet(currentFocusView, () -> getWindow().getDecorView()).getWindowToken(), 0);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        RxSubscriptions.remove(mSubscription);
    }
}
