package com.kanglai.push.ui.activity;

import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Patterns;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.dolphin.core.BuildConfig;
import com.dolphin.core.base.BaseActivity;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.constant.CommonConstant;
import com.kanglai.push.databinding.ActivityLinkDetailBinding;
import com.kanglai.push.entity.ConcernFan;
import com.kanglai.push.ui.vm.LinkDetailViewModel;

/**
 * 粉丝碎片页详情
 * @Author: liusixiang007
 * @since: 2023/2/15
 */
public class LinkDetailActivity extends BaseActivity<ActivityLinkDetailBinding, LinkDetailViewModel>{

    private String link_Id;     // 粉丝id
    private int link_type;      // 用户类型
    private ImageView avatar;
    private TextView link_name;
    private TextView link_phone;
    private TextView dept;
    private TextView remark;
    private ImageView sex_man;
    private ImageView sex_woman;
    private LinearLayout focus_btn;
    private LinearLayout send_msg_btn;


    @Override
    public int setContentView(Bundle savedInstanceState) {
        return R.layout.activity_link_detail;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        super.mViewModel.mActivity = this;

        avatar = findViewById(R.id.link_detail_avatar);
        link_name = findViewById(R.id.link_detail_name);
        link_phone = findViewById(R.id.link_detail_phone);
        dept = findViewById(R.id.link_detail_dept);
        remark = findViewById(R.id.link_detail_remark);
        sex_man = findViewById(R.id.gender_man);
        sex_woman = findViewById(R.id.gender_woman);
        focus_btn = findViewById(R.id.focus_btn);
        send_msg_btn = findViewById(R.id.action_bar_send_msg);
        // 加载用户数据
        Intent intent = getIntent();
        link_Id = intent.getStringExtra(CommonConstant.USER_LINK_ID);
        link_type = intent.getIntExtra(CommonConstant.USER_LINK_TYPE, 0);
        if (!TextUtils.isEmpty(link_Id)) mViewModel.getUserMsgById(link_Id);
        loadingView(link_type);
    }

    /**
     * 加载视图
     * @param type
     * 1-粉丝 2-关注 0-陌生人
     */
    private void loadingView(int type) {
        switch (type){
            case 0: // 陌生人
                focus_btn.setVisibility(View.VISIBLE);
                send_msg_btn.setVisibility(View.GONE);
                break;
            case 1: // 粉丝 隐藏关注按钮 显示取关|发送消息
                focus_btn.setVisibility(View.GONE);
                send_msg_btn.setVisibility(View.VISIBLE);
                break;
            case 2: // 我的关注
                break;
        }
    }


    /** 加载粉丝 */
    public void loadUserMsg(ConcernFan fans) {
        if (!TextUtils.isEmpty(fans.getAvatar()) && !Patterns.WEB_URL.matcher(fans.getAvatar()).matches()) {
            fans.setAvatar(BuildConfig.HOST_URL + fans.getAvatar().substring(1));
        }
        Glide.with(this)
                .load(fans.getAvatar())
                .skipMemoryCache(true)
                .centerCrop()
                .override(300, 300)
                .placeholder(R.drawable.icon_img_loading)    // 加载图片
                .error(R.drawable.icon_links_avatar)    // 加载错误
                .into(avatar);
        link_name.setText(fans.getNickName() + " (" + fans.getUserName() + ") ");
        if (!TextUtils.isEmpty(fans.getPhone())) link_phone.setText("电话："+fans.getPhone());
        else link_phone.setText("电话：暂无");
        if (!TextUtils.isEmpty(fans.getDeptName())) dept.setText("归属机构："+fans.getDeptName());
        else dept.setText("归属机构：未认证");
        if (!TextUtils.isEmpty(fans.getRemarks())) remark.setText("备注："+fans.getRemarks());
        else remark.setText("备注：暂无");
        // 性别 0-男 1-女
        if (fans.getSex() == "0") sex_woman.setVisibility(View.GONE);
        else sex_man.setVisibility(View.GONE);

    }
}
