package com.kanglai.push.ui.activity;

import android.Manifest;
import android.app.AlertDialog;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.text.InputFilter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import androidx.core.app.NotificationManagerCompat;

import com.blankj.utilcode.util.StringUtils;
import com.dolphin.core.base.AppManager;
import com.dolphin.core.base.BaseActivity;
import com.dolphin.core.constant.AppConstant;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.databinding.ActivityLoginBinding;
import com.kanglai.push.ui.vm.LoginViewModel;
import com.kanglai.push.util.ViewFilterUtil;
import com.tencent.mmkv.MMKV;
import com.umeng.socialize.UMShareAPI;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 *<p>
 * 登录活动
 * 提供模板规范代码参考,请尽量保证编写代码风格跟模板规范代码一致
 * 采用 rxJava 可观测编写
 * Copyright © 2020-2022 <a href="http://www.entfrm.com/">entfrm</a> All rights reserved.
 *</p>
 *
 * @Author: entfrm开发团队-王翔
 * @since: 2022/10/14
 */
public class LoginActivity extends BaseActivity<ActivityLoginBinding, LoginViewModel> {

    /** 请求权限检查 */
    private boolean requestPermissionNeedCheck = true;

    public TextView mTextView; //  验证码倒计时

    public Button loginBtn; // 登录按钮

    /** 软件申请动态权限,必须设置不设置不让进软件 */
    private final String[] needPermissions = {
        Manifest.permission.WRITE_EXTERNAL_STORAGE,
        Manifest.permission.READ_EXTERNAL_STORAGE,
        Manifest.permission.ACCESS_FINE_LOCATION,
        Manifest.permission.ACCESS_BACKGROUND_LOCATION,
        Manifest.permission.READ_PHONE_STATE
    };
    
    @Override
    public int setContentView(Bundle savedInstanceState) {
        return R.layout.activity_login;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        super.mViewModel.mActivity = this;
        if(!StringUtils.isTrimEmpty(MMKV.defaultMMKV().getString(AppConstant.ACCESS_TOKEN_NAME,null))){
            startActivity(TabBarActivity.class);
            finish();
        }
        EditText input01 = findViewById(R.id.et_phone);
        EditText input02 = findViewById(R.id.et_captcha);
        input01.setFilters(new InputFilter[]{ViewFilterUtil.getNoSpecialCharactersFilter()});
        input02.setFilters(new InputFilter[]{ViewFilterUtil.getNoSpecialCharactersFilter()});
        input01.setFilters(new InputFilter[]{new InputFilter.LengthFilter(11)});
        input02.setFilters(new InputFilter[]{new InputFilter.LengthFilter(6)});
        mTextView = findViewById(R.id.tv_sendCMS);
        loginBtn = findViewById(R.id.login_btn);
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (requestPermissionNeedCheck) checkPermissions(needPermissions);
    }

    /** 检查软件所需要的权限,没有则弹出软件权限申请框申请 */
    private void checkPermissions(String... permissions) {
        List<String> needRequestPermissionList = findDeniedPermissions(permissions);
        if (needRequestPermissionList.size() > 0) {
            requestPermissions(needRequestPermissionList.toArray(new String[needRequestPermissionList.size()]), AppConstant.PERMISSION_REQUEST_CODE);
        }
    }

    /** 查找被用户拒绝以及需要动态申请的权限 */
    private List<String> findDeniedPermissions(String[] permissions) {
        List<String> needRequestPermissionList = new ArrayList();
        Arrays.stream(permissions).forEach(item -> {
            if (checkSelfPermission(item) != PackageManager.PERMISSION_GRANTED || shouldShowRequestPermissionRationale(item)) needRequestPermissionList.add(item);
        });
        return needRequestPermissionList;
    }

    @Override
    // https://www.cnblogs.com/ganchuanpu/p/6798682.html
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] paramArrayOfInt) {
        super.onRequestPermissionsResult(requestCode, permissions, paramArrayOfInt);
        if (requestCode == AppConstant.PERMISSION_REQUEST_CODE) {
//            if (Arrays.stream(paramArrayOfInt).anyMatch(item -> item != PackageManager.PERMISSION_GRANTED)) {
//                requestPermissionNeedCheck = false;
//                PermissionUtil.openApplicationSettings(this, "当前应用缺少必要权限。\n请点击设置-权限-打开所需权限!");
//            }
            if (!NotificationManagerCompat.from(getApplicationContext()).areNotificationsEnabled()) {
                new AlertDialog.Builder(this)
                    .setTitle("授权设备权限")
                    .setMessage(getString(com.dolphin.core.R.string.app_name) + " 想要获取此设备的通知权限?")
                    .setNegativeButton("取消", (dialog, which) -> AppManager.getAppManager().finishAllActivity())
                    .setPositiveButton("前往设置", (dialog, which) -> {
                        dialog.dismiss();
                        Intent intent = new Intent();
                        intent.setAction("android.settings.APP_NOTIFICATION_SETTINGS");
                        intent.putExtra("android.provider.extra.APP_PACKAGE", getPackageName());
                        startActivity(intent);
                    })
                    .setCancelable(false)
                    .show();
            }
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        UMShareAPI.get(this).onActivityResult(requestCode, resultCode, data);
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
        System.exit(0);
    }

}
