package com.kanglai.push.ui.activity;

import android.os.Bundle;
import android.text.Editable;
import android.text.InputFilter;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.databinding.library.baseAdapters.BR;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.blankj.utilcode.util.CollectionUtils;
import com.dolphin.core.base.BaseActivity;
import com.kanglai.push.R;
import com.kanglai.push.databinding.ActivityPushTypeBinding;
import com.kanglai.push.entity.PushType;
import com.kanglai.push.ui.adapter.PushTypeRecyclerAdapter;
import com.kanglai.push.ui.vm.PushTypeViewModel;
import com.kanglai.push.util.ViewFilterUtil;
import com.kongzue.dialogx.dialogs.FullScreenDialog;
import com.kongzue.dialogx.dialogs.MessageDialog;
import com.kongzue.dialogx.interfaces.OnBindView;
import com.kongzue.dialogx.interfaces.OnDialogButtonClickListener;
import com.suke.widget.SwitchButton;

/**
 * 推送设置页（含推送列表）
 * @Author: liusixiang007
 * @since: 2023/6/19
 */
public class PushTypeActivity extends BaseActivity<ActivityPushTypeBinding, PushTypeViewModel> implements PushTypeRecyclerAdapter.EventListener {

    public PushTypeRecyclerAdapter mAdapter;
    public RecyclerView mRecyclerView;
    @Override
    public int setContentView(Bundle savedInstanceState) {
        return R.layout.activity_push_type;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        super.mViewModel.mActivity = this;
        /** 展示顶部右侧更多操作*/
        ImageView rightLogo = findViewById(R.id.iv_right_icon);
        rightLogo.setImageDrawable(getResources().getDrawable(R.drawable.icon_tool_bar_add));
        /** 加载消息类型列表数据*/
        mViewModel.loadPushTypeList();

        final PushTypeRecyclerAdapter pushTypeRecyclerAdapter = new PushTypeRecyclerAdapter(CollectionUtils.newArrayList());
        pushTypeRecyclerAdapter.setEventListener(this);
        mAdapter = pushTypeRecyclerAdapter;
        mRecyclerView = findViewById(R.id.recycler_view);
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(this);
        mRecyclerView.setLayoutManager(mLayoutManager);
        mRecyclerView.setAdapter(mAdapter);
    }

    /**
     * 列表单项详情以及修改
     * @param pushType
     */
    @Override
    public void onItemViewClicked(PushType pushType) {
        /** 单行点击事件-查看*/;
        FullScreenDialog.show(new OnBindView<FullScreenDialog>(R.layout.item_full_dialog_push) {
            @Override
            public void onBind(FullScreenDialog dialog, View v) {
                TextView tit = v.findViewById(R.id.dialog_tit);
                tit.setText("修改推送类型");
                EditText input = v.findViewById(R.id.edit_type_name);   //  类型名称
                input.setFilters(new InputFilter[]{ViewFilterUtil.getNoSpecialCharactersFilter()});  // 去空格、换行符等特殊符号
                input.setText(pushType.getName());
                input.addTextChangedListener(new TextWatcher() {
                    @Override
                    public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

                    }

                    @Override
                    public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

                    }

                    @Override
                    public void afterTextChanged(Editable editable) {
                        pushType.setName(String.valueOf(editable).trim());
                    }
                });
                SwitchButton switch01 = v.findViewById(R.id.switch_button_01);  // 声音
                switch01.setChecked("1".equals(pushType.getPlaySound()) ? true : false);
                switch01.setOnCheckedChangeListener(new SwitchButton.OnCheckedChangeListener() {
                    @Override
                    public void onCheckedChanged(SwitchButton view, boolean isChecked) {
                        pushType.setPlaySound((isChecked ? "1" : "0"));
                    }
                });
                SwitchButton switch02 = v.findViewById(R.id.switch_button_02);  // 震动
                switch02.setChecked("1".equals(pushType.getPlayVibrate()) ? true : false);
                switch02.setOnCheckedChangeListener(new SwitchButton.OnCheckedChangeListener() {
                    @Override
                    public void onCheckedChanged(SwitchButton view, boolean isChecked) {
                        pushType.setPlayVibrate((isChecked ? "1" : "0"));
                    }
                });
                SwitchButton switch03 = v.findViewById(R.id.switch_button_03);  // 闪光
                switch03.setChecked("1".equals(pushType.getPlayLights()) ? true : false);
                switch03.setOnCheckedChangeListener(new SwitchButton.OnCheckedChangeListener() {
                    @Override
                    public void onCheckedChanged(SwitchButton view, boolean isChecked) {
                        pushType.setPlayLights((isChecked ? "1" : "0"));
                    }
                });
                SwitchButton switch04 = v.findViewById(R.id.switch_button_04);  // 语音转文字
                switch04.setChecked("1".equals(pushType.getPlayToText())?true:false);
                switch04.setOnCheckedChangeListener(new SwitchButton.OnCheckedChangeListener() {
                    @Override
                    public void onCheckedChanged(SwitchButton view, boolean isChecked) {
                        pushType.setPlayToText((isChecked ? "1" : "0"));
                    }
                });
                Button editor = v.findViewById(R.id.dialog_type_button);
                editor.setText("修 改");
                if (!TextUtils.isEmpty(pushType.getName())) {
                    editor.setOnClickListener(view -> mViewModel.updateType(pushType));
                }
            }
        });
    }

    /**
     * 推送类型列表开关修改事件
     * @param pushType
     * @param view
     */
    @Override
    public void onSwitchButtonCheck(PushType pushType, SwitchButton view) {
        MessageDialog.show("警 告","确认要启用【"+pushType.getName()+"】为默认类型?", "确 定", "取 消")
                .setOkButton(new OnDialogButtonClickListener<MessageDialog>() {
                    @Override
                    public boolean onClick(MessageDialog dialog, View v) {
                        mViewModel.setDefaultType(pushType, dialog);
                        return true;
                    }
                })
                .setCancelButton(new OnDialogButtonClickListener<MessageDialog>() {
                    @Override
                    public boolean onClick(MessageDialog dialog, View v) {
                        view.setChecked(false);     // 取消后恢复初始状态
                        return false;
                    }
                })
                .show();
    }
}
