package com.kanglai.push.ui.activity;

import android.os.Bundle;

import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;

import com.dolphin.core.base.BaseActivity;
import com.dolphin.core.base.BaseViewModel;
import com.dolphin.core.constant.AppConstant;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.databinding.ActivityTabBarBinding;
import com.kanglai.push.ui.fragment.LinkFragment;
import com.kanglai.push.ui.fragment.MessageFragment;
import com.kanglai.push.ui.fragment.UserFragment;
import com.kanglai.push.ui.fragment.WorkbenchFragment;

import java.util.ArrayList;
import java.util.List;

import me.majiajie.pagerbottomtabstrip.NavigationController;
import me.majiajie.pagerbottomtabstrip.PageNavigationView;
import me.majiajie.pagerbottomtabstrip.item.BaseTabItem;
import me.majiajie.pagerbottomtabstrip.item.NormalItemView;
import me.majiajie.pagerbottomtabstrip.listener.OnTabItemSelectedListener;

/**
 *<p>
 * 底部标签栏活动
 *</p>
 *
 * @Author: entfrm开发团队-王翔
 * @since: 2022/11/21
 */
public class TabBarActivity extends BaseActivity<ActivityTabBarBinding, BaseViewModel> {

    private List<Fragment> mFragments;
    private PageNavigationView pageNavigationView;

    @Override
    public int setContentView(Bundle savedInstanceState) {
        return R.layout.activity_tab_bar;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        pageNavigationView = findViewById(R.id.pager_bottom_tab);
        initFragment();
    }

    private void initFragment() {
        mFragments = new ArrayList<>();
        mFragments.add(new MessageFragment());
        mFragments.add(new LinkFragment());
        mFragments.add(new WorkbenchFragment());
        mFragments.add(new UserFragment());
        Integer defaultPoint = getIntent().getIntExtra(AppConstant.TAB_BAR_DEFAULT_INDEX, 0);
        commitAllowingStateLoss(defaultPoint);
        initBottomTab(defaultPoint);
    }

    private void initBottomTab(int position) {
        // todo 优化完善底部logo
        NavigationController navigationController = pageNavigationView.custom()
                .addItem(normalItem(R.drawable.icon_tab_01_line, R.drawable.icon_tab_01_fill,"消息"))
                .addItem(normalItem(R.drawable.icon_tab_02_line, R.drawable.icon_tab_02_fill, "粉丝"))
                .addItem(normalItem(R.drawable.icon_tab_03_line, R.drawable.icon_tab_03_fill, "工作台"))
                .addItem(normalItem(R.drawable.icon_tab_04_line, R.drawable.icon_tab_04_fill, "我的"))
                .build();
        // 设置默认底部按钮选中
        navigationController.setSelect(position);
        // 底部按钮的点击事件监听
        navigationController.addTabItemSelectedListener(new OnTabItemSelectedListener() {
            @Override
            public void onSelected(int index, int old) {
                commitAllowingStateLoss(index);
            }

            @Override
            public void onRepeat(int index) {
            }
        });
    }

    private BaseTabItem normalItem(int drawable, int checkedDrawable, String text) {
        NormalItemView normalItemView = new NormalItemView(this);
        normalItemView.initialize(drawable, checkedDrawable, text);
        normalItemView.setBackgroundColor(ContextCompat.getColor(this, R.color.white));
        normalItemView.setTextDefaultColor(ContextCompat.getColor(this, R.color.black));
        normalItemView.setTextCheckedColor(ContextCompat.getColor(this, R.color.common_app_them));
        return normalItemView;
    }

    private void commitAllowingStateLoss(int position) {
        hideAllFragment();
        FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
        Fragment currentFragment = getSupportFragmentManager().findFragmentByTag(position + "");
        if (currentFragment != null) {  // 重复切换
            transaction.show(currentFragment);
            if (position == 1 || position == 0) currentFragment.onResume();  // 重复打开消息页、粉丝页强制重复刷新
        } else { // 初次加载
            currentFragment = mFragments.get(position);
            transaction.add(R.id.frameLayout, currentFragment, position + "");
        }
        transaction.commitAllowingStateLoss();
    }

    /** 隐藏所有Fragment */
    private void hideAllFragment() {
        FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
        for (int i = 0; i < mFragments.size(); i++) {
            Fragment currentFragment = getSupportFragmentManager().findFragmentByTag(i + "");
            if (currentFragment != null) {
                transaction.hide(currentFragment);
            }
        }
        transaction.commitAllowingStateLoss();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        finish();
    }
}
