package com.kanglai.push.ui.adapter;

import android.content.Context;
import android.text.TextUtils;
import android.util.Patterns;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.dolphin.core.BuildConfig;
import com.kanglai.push.R;
import com.kanglai.push.entity.ConcernFan;


import me.yokeyword.indexablerv.IndexableAdapter;


/**
 * 联系人碎片页适配器
 * @Author: liusixiang007
 * @since: 2023/2/15
 */
public class LinkAdapter extends IndexableAdapter<ConcernFan> {

    private LayoutInflater mInflater;

    /** 初始化 */
    public LinkAdapter(Context context){
        mInflater = LayoutInflater.from(context);
    }
    @Override
    public RecyclerView.ViewHolder onCreateTitleViewHolder(ViewGroup parent) {
        // 联系人标题页
        View view = mInflater.inflate(R.layout.item_link_tit, parent, false);
        return new IndexVH(view);
    }

    @Override
    public RecyclerView.ViewHolder onCreateContentViewHolder(ViewGroup parent) {
        View view = mInflater.inflate(R.layout.item_link, parent, false);
        return new ContentVH(view);
    }

    @Override
    public void onBindTitleViewHolder(RecyclerView.ViewHolder holder, String index) {
        IndexVH vh = (IndexVH) holder;
        /** 赋值 - 单个 */
        vh.tv_index.setText(index);
    }

    @Override
    public void onBindContentViewHolder(final RecyclerView.ViewHolder holder, ConcernFan entity) {
        ContentVH vh = (ContentVH) holder;
        vh.linkName.setText(entity.getNickName() + "(" + entity.getUserName() + ")");
        vh.tv_Mobile.setText("电话:"+entity.getPhone());
        String avatar = entity.getAvatar();
        if (!TextUtils.isEmpty(entity.getAvatar())) {
            if (!Patterns.WEB_URL.matcher(avatar).matches()) {
                avatar = BuildConfig.HOST_URL + avatar.substring(1);
            }
            // 加载图片
            Glide.with(vh.itemView.getContext())
                    .load(avatar)
                    .skipMemoryCache(true)
                    .centerCrop()
                    .override(100, 100)
                    .placeholder(R.drawable.icon_img_loading)    // 加载图片
                    .error(R.drawable.icon_links_avatar)    // 加载错误
                    .into(vh.link_avatar);
        }
    }



    private class IndexVH extends RecyclerView.ViewHolder {
        TextView tv_index;

        public IndexVH(View itemView) {
            super(itemView);
            tv_index = (TextView) itemView.findViewById(R.id.tv_index);
        }
    }

    private class ContentVH extends RecyclerView.ViewHolder {
        TextView linkName;
        TextView tv_Mobile;
        ImageView link_avatar;

        public ContentVH(View itemView) {
            super(itemView);
            linkName = (TextView) itemView.findViewById(R.id.tv_name);
            tv_Mobile = (TextView) itemView.findViewById(R.id.tv_mobile);
            link_avatar = (ImageView) itemView.findViewById(R.id.img_avatar);
        }
    }
}
