package com.kanglai.push.ui.adapter;

import android.graphics.Color;
import android.text.TextUtils;
import android.util.Patterns;
import android.view.View;

import com.blankj.utilcode.util.ObjectUtils;
import com.bumptech.glide.Glide;
import com.dolphin.core.BuildConfig;
import com.kanglai.push.R;
import com.kanglai.push.entity.LocalPushChatMsg;
import com.kanglai.push.util.TimeFormatUtil;

import java.util.ArrayList;
import java.util.Collection;

/**
 *<p>
 * 消息回收列表数据适配器
 *</p>
 *
 * @Author: wangxiang4
 * @since: 2023/2/2
 */
public class MessageRecyclerAdapter extends NotificationListRecyclerAdapter {

    private ArrayList<LocalPushChatMsg> mItemList;

    private EventListener mEventListener;

    public MessageRecyclerAdapter(ArrayList<LocalPushChatMsg> mItemList){
        this.mItemList = mItemList;
    }

    public interface EventListener {
        void onItemViewClicked(LocalPushChatMsg localPushChatMsg);
    }

    @Override
    public int getItemCount() {
        return mItemList.size();
    }

    @Override
    public void onBindViewHolder(final ViewHolder viewHolder, final int position) {
        super.onBindViewHolder(viewHolder, position);
        final LocalPushChatMsg item = mItemList.get(position);
        viewHolder.layoutContent.setOnClickListener(view -> onItemViewClick(item));
        viewHolder.titleLabel01.setText(item.getUserName());      // 用户名赋值
        viewHolder.titleLabel03.setText(item.getLastContent());
        if (TextUtils.isEmpty(item.getSendTime())) {
            viewHolder.titleLabel02.setText(TimeFormatUtil.formatTimeUI(item.getSendTime())); // 时间
        }
        viewHolder.titleLabel03.setTextColor(Color.GRAY);
        viewHolder.leftBadge.setVisibility(View.GONE);      // 隐藏徽章

        String avatar = item.getAvatar();
        if (!TextUtils.isEmpty(item.getAvatar())) {
            if (!Patterns.WEB_URL.matcher(avatar).matches()) {
                avatar = BuildConfig.HOST_URL + avatar.substring(1);
            }
            // 加载图片
            Glide.with(viewHolder.itemView.getContext())
                    .load(avatar)
                    .skipMemoryCache(true)
                    .centerCrop()
                    .override(100, 100)
                    .placeholder(R.drawable.icon_img_loading)    // 加载图片
                    .error(R.drawable.icon_links_avatar)    // 加载错误
                    .into(viewHolder.leftAvatar);
        }
    }

    private void onItemViewClick(LocalPushChatMsg entity) {
        if (mEventListener != null && ObjectUtils.isNotEmpty(mItemList)) {
            mEventListener.onItemViewClicked(entity);
        }
    }

    public EventListener getEventListener(){
        return mEventListener;
    }

    public MessageRecyclerAdapter setEventListener(EventListener eventListener) {
        mEventListener = eventListener;
        return this;
    }

    public MessageRecyclerAdapter refresh(Collection<LocalPushChatMsg> collection) {
        mItemList.clear();
        mItemList.addAll(collection);
        notifyDataSetChanged();
        mLastPosition = -1;
        return this;
    }

    public MessageRecyclerAdapter loadMore(Collection<LocalPushChatMsg> collection) {
        mItemList.addAll(collection);
        notifyDataSetChanged();
        return this;
    }

    public MessageRecyclerAdapter insert(Collection<LocalPushChatMsg> collection) {
        mItemList.addAll(0, collection);
        notifyItemRangeInserted(0, collection.size());
        return this;
    }

}
