package com.kanglai.push.ui.adapter;

import android.text.TextUtils;
import android.util.Patterns;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.dolphin.core.BuildConfig;
import com.h6ah4i.android.widget.advrecyclerview.swipeable.SwipeableItemAdapter;
import com.h6ah4i.android.widget.advrecyclerview.swipeable.SwipeableItemConstants;
import com.h6ah4i.android.widget.advrecyclerview.swipeable.action.SwipeResultAction;
import com.h6ah4i.android.widget.advrecyclerview.swipeable.action.SwipeResultActionDefault;
import com.h6ah4i.android.widget.advrecyclerview.swipeable.action.SwipeResultActionMoveToSwipedDirection;
import com.h6ah4i.android.widget.advrecyclerview.utils.AbstractSwipeableItemViewHolder;
import com.h6ah4i.android.widget.advrecyclerview.utils.RecyclerViewAdapterUtils;
import com.kanglai.push.R;
import com.kanglai.push.entity.LocalPushChatMsg;

import java.util.Collection;
import java.util.List;

/**
 * 消息页滑动回收视图适配器
 * @Author: liusixiang007
 * @since: 2023/7/13
 */
public class MessageSwipeableRecyclerAdapter extends RecyclerView.Adapter<MessageSwipeableRecyclerAdapter.ViewHolder> implements SwipeableItemAdapter<MessageSwipeableRecyclerAdapter.ViewHolder> {

    public List<LocalPushChatMsg> mItemList;
    private EventListener mEventListener;
    public interface EventListener{
        /** 滑动固定项后回调 */
        void onItemPinned(int position);

        /** 滑动容器触摸 */
        void onItemViewTouch(View v, MotionEvent event);

        /** 滑动容器点击 */
        void onItemViewClicked(View v, LocalPushChatMsg item);

        /** 将当前这项目设置为已读 */
        void setToRemove(LocalPushChatMsg item);

        /** 删除当前项 */
        void setToRead(int position);
    }
    public static class ViewHolder extends AbstractSwipeableItemViewHolder{
        public LinearLayout behindLayout;   // 隐藏的按钮
        public TextView hideBtn1;
        public TextView hideBtn2;
        public TextView hideBtn3;
        public LinearLayout container;    // 未隐藏item
        public ImageView image;     // 头像
        public ImageView badge;     // 徽章
        public TextView userName;
        public TextView textContent;
        public TextView days;

        public ViewHolder(View v) {
            super(v);
            behindLayout = v.findViewById(R.id.message_layout_behind);
            //hideBtn2 = v.findViewById(R.id.hide_btn_02);
            hideBtn3 = v.findViewById(R.id.hide_btn_03);
            container = v.findViewById(R.id.message_layout_container);
            image = v.findViewById(R.id.message_left_image);
            //badge = v.findViewById(R.id.message_left_badge);
            userName = v.findViewById(R.id.title_label01);
            days = v.findViewById(R.id.title_label02);
            textContent = v.findViewById(R.id.title_label03);
        }

        @Override
        public View getSwipeableContainerView() {
            return container;
        }
    }
    public MessageSwipeableRecyclerAdapter(List<LocalPushChatMsg> mItemList) {
        this.mItemList = mItemList;
        setHasStableIds(true);
    }
    private void onSwipeableViewContainerTouch(View v, MotionEvent event) {
        if (mEventListener != null) {
            mEventListener.onItemViewTouch(RecyclerViewAdapterUtils.getParentViewHolderItemView(v), event);
        }
    }
    private void onSwipeableViewContainerClick(View v, LocalPushChatMsg item) {
        if (mEventListener != null) {
            mEventListener.onItemViewClicked(RecyclerViewAdapterUtils.getParentViewHolderItemView(v),item);
        }
    }
//    private void onBtnSwipeable1Click(View v) {
//        if (mEventListener != null) {
//            mEventListener.onBtnSwipeable1(RecyclerViewAdapterUtils.getParentViewHolderItemView(v));
//        }
//    }
    private void setToRead(int position) {
        if (mEventListener != null) {
            mEventListener.setToRead(position);
            //mItemList.get(position).setIsRead("0");
            mItemList.get(position).setPinned(false);
            notifyItemChanged(position);
        }
    }
    private void setToRemove(LocalPushChatMsg item) {
        if (mEventListener != null) {
            mEventListener.setToRemove(item);
        }
    }

    @Override
    public long getItemId(int position) {
        return mItemList.get(position).getId().hashCode();
    }

    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        final LayoutInflater inflater = LayoutInflater.from(parent.getContext());
        final View v = inflater.inflate(R.layout.item_swipeable_message, parent, false);
        return new ViewHolder(v);
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        final LocalPushChatMsg item = mItemList.get(position);
        holder.setMaxLeftSwipeAmount(-0.22f);
        holder.setMaxRightSwipeAmount(0);
        holder.setSwipeItemHorizontalSlideAmount(item.getPinned() ? -0.22f : 0);

        /** 单项赋值 */
        holder.userName.setText(item.getUserName());
        holder.days.setText(item.getSendTime());
        holder.textContent.setText(item.getLastContent());
        String avatarUrl = item.getAvatar();
        //if (!TextUtils.isEmpty(item.getIsRead())&&"1".equals(item.getIsRead())) holder.badge.setVisibility(View.VISIBLE);
        /** 文字非空 以及 非url */
        if (!TextUtils.isEmpty(item.getAvatar()) && !Patterns.WEB_URL.matcher(avatarUrl).matches()) avatarUrl = BuildConfig.HOST_URL + avatarUrl.substring(1);
        if (holder.image.getDrawable() == null) {
            Glide.with(holder.itemView.getContext())
                    .load(avatarUrl)
                    .centerCrop()
                    .override(100, 100)
                    .placeholder(R.drawable.icon_img_loading)
                    .error(R.drawable.icon_links_avatar)
                    .into(holder.image);
        }
        holder.container.setOnTouchListener((view, event) -> {
            onSwipeableViewContainerTouch(view, event);
            return false;
        });
        holder.container.setOnClickListener(view -> onSwipeableViewContainerClick(view, item));
        //holder.hideBtn1.setOnClickListener(view -> onBtnSwipeable1Click(view));
        //holder.hideBtn2.setOnClickListener(view -> setToRead(position));
        holder.hideBtn3.setOnClickListener(view -> setToRemove(item));
    }

    @Override
    public int getItemCount() {
        return mItemList.size();
    }

    @Override
    public int onGetSwipeReactionType(@NonNull ViewHolder holder, int position, int x, int y) {
        final int tx = (int) (holder.getSwipeableContainerView().getTranslationX() + 0.22f);
        final int ty = (int) (holder.getSwipeableContainerView().getTranslationY() + 0.22f);
        final int left = holder.getSwipeableContainerView().getLeft() + tx;
        final int right = holder.getSwipeableContainerView().getRight() + tx;
        final int top = holder.getSwipeableContainerView().getTop() + ty;
        final int bottom = holder.getSwipeableContainerView().getBottom() + ty;
        // 获取滑动状态,SwipeableItem内部通过此变量来进行控制
        if ((x >= left) && (x <= right) && (y >= top) && (y <= bottom)) {
            return SwipeableItemConstants.REACTION_CAN_SWIPE_BOTH_H;
        } else {
            return SwipeableItemConstants.REACTION_CAN_NOT_SWIPE_BOTH_H;
        }
    }

    @Override
    public void onSwipeItemStarted(@NonNull ViewHolder holder, int position) {
        notifyDataSetChanged();
    }

    @Override
    public void onSetSwipeBackground(@NonNull ViewHolder holder, int position, int type) {
        if (type == SwipeableItemConstants.DRAWABLE_SWIPE_NEUTRAL_BACKGROUND) {
            holder.behindLayout.setVisibility(View.GONE);
        }else {
            holder.behindLayout.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public SwipeResultAction onSwipeItem(@NonNull ViewHolder holder, int position, int result) {
        switch (result) {
            case SwipeableItemConstants.RESULT_SWIPED_LEFT:
                return new SwipeLeftResultAction(this, position);
            case SwipeableItemConstants.RESULT_SWIPED_RIGHT:
            case SwipeableItemConstants.RESULT_CANCELED:
            default:
                if (position != RecyclerView.NO_POSITION) {
                    return new UnpinResultAction(this, position);
                } else {
                    return null;
                }
        }
    }

    public List<LocalPushChatMsg> getData(){
        return mItemList;
    }

    public EventListener getEventListener() {
        return mEventListener;
    }

    public MessageSwipeableRecyclerAdapter setEventListener(EventListener eventListener) {
        mEventListener = eventListener;
        return this;
    }

    public MessageSwipeableRecyclerAdapter refresh(Collection<LocalPushChatMsg> collection) {
        mItemList.clear();
        mItemList.addAll(collection);
        notifyDataSetChanged();
        return this;
    }


    public MessageSwipeableRecyclerAdapter loadMore(Collection<LocalPushChatMsg> collection) {
        mItemList.addAll(collection);
        notifyDataSetChanged();
        return this;
    }

    public MessageSwipeableRecyclerAdapter insert(Collection<LocalPushChatMsg> collection) {
        mItemList.addAll(0, collection);
        notifyItemRangeInserted(0, collection.size());
        return this;
    }


    /**左滑*/
    private static class SwipeLeftResultAction extends SwipeResultActionMoveToSwipedDirection {
        private MessageSwipeableRecyclerAdapter mMessageSwipeableRecyclerAdapter;
        private final int mPosition;
        private boolean mSetPinned;
        SwipeLeftResultAction(MessageSwipeableRecyclerAdapter messageSwipeableRecyclerAdapter, int position) {
            mMessageSwipeableRecyclerAdapter = messageSwipeableRecyclerAdapter;
            mPosition = position;
        }

        @Override
        protected void onPerformAction() {
            super.onPerformAction();
            LocalPushChatMsg item = mMessageSwipeableRecyclerAdapter.mItemList.get(mPosition);
            // 检测是否已经处于滑动完毕
            if (!item.getPinned()) {
                item.setPinned(true);
                mMessageSwipeableRecyclerAdapter.notifyItemChanged(mPosition);
                mSetPinned = true;
            }
        }

        @Override
        protected void onSlideAnimationEnd() {
            super.onSlideAnimationEnd();
            if (mSetPinned && mMessageSwipeableRecyclerAdapter.mEventListener != null){
                mMessageSwipeableRecyclerAdapter.mEventListener.onItemPinned(mPosition);
            }
        }

        @Override
        protected void onCleanUp() {
            super.onCleanUp();
            mMessageSwipeableRecyclerAdapter = null;
        }
    }

    /**关闭左滑*/
    private static class UnpinResultAction extends SwipeResultActionDefault {
        private MessageSwipeableRecyclerAdapter mMessageSwipeableRecyclerAdapter;
        private final int mPosition;
        UnpinResultAction(MessageSwipeableRecyclerAdapter mAdapter, int position) {
            mMessageSwipeableRecyclerAdapter = mAdapter;
            mPosition = position;
        }

        @Override
        protected void onPerformAction() {
            super.onPerformAction();
            LocalPushChatMsg item = mMessageSwipeableRecyclerAdapter.mItemList.get(mPosition);
            // 检测是否已经处于滑动完毕
            if (item.getPinned()) {
                item.setPinned(false);
                mMessageSwipeableRecyclerAdapter.notifyItemChanged(mPosition);
            }
        }

        @Override
        protected void onCleanUp() {
            super.onCleanUp();
            mMessageSwipeableRecyclerAdapter = null;
        }
    }


}
