package com.kanglai.push.ui.adapter;

import android.text.TextUtils;
import android.util.Patterns;
import android.view.View;

import com.blankj.utilcode.util.ObjectUtils;
import com.bumptech.glide.Glide;
import com.dolphin.core.BuildConfig;
import com.kanglai.push.R;
import com.kanglai.push.entity.ConcernFan;

import java.util.Collection;
import java.util.List;

/**
 * 我的关注(个人)回收视图适配器
 * @Author: liusixiang007
 * @since: 2023/6/13
 */
public class MyFocusRecycleAdapter extends DefaultRecyclerAdapter {

    private List<ConcernFan> mList;
    private EventListener mEventListener;

    public MyFocusRecycleAdapter(List<ConcernFan> mList){
        this.mList = mList;
    }
    /** 自定义列表点击事件*/
    public interface EventListener {
        void onItemViewClicked(ConcernFan fans);
    }

    @Override
    public int getItemCount() {
        return mList.size();
    }

    @Override
    public void onBindViewHolder(final ViewHolder viewHolder,final int position) {
        super.onBindViewHolder(viewHolder, position);
        final ConcernFan item = mList.get(position);
        viewHolder.layoutContent.setOnClickListener(view -> onItemViewClick(item));
        viewHolder.titleLabel.setText(item.getNickName()+ " 【"+ item.getUserName()+"】");
        viewHolder.disclosureImage.setVisibility(View.GONE);    // 隐藏返回箭头
        viewHolder.leftBadge.setVisibility(View.GONE);       // 隐藏徽章
        viewHolder.secondDetailLabel.setVisibility(View.GONE);      // 隐藏右侧文字部分
        String avater = item.getAvatar();
        if (!TextUtils.isEmpty(avater)) {
            if (!Patterns.WEB_URL.matcher(avater).matches()) avater = BuildConfig.HOST_URL + item.getAvatar().substring(1);
            // 加载图片
            Glide.with(viewHolder.itemView.getContext())
                    .load(avater)
                    .skipMemoryCache(true)
                    .centerCrop()
                    .override(100, 100)
                    .placeholder(R.drawable.icon_img_loading)    // 加载图片
                    .error(R.drawable.icon_links_avatar)    // 加载错误
                    .into(viewHolder.leftImage);
        }
    }

    private void onItemViewClick(ConcernFan item) {
        if (mEventListener != null && ObjectUtils.isNotEmpty(mList)) {
            mEventListener.onItemViewClicked(item);
        }
    }

    public MyFocusRecycleAdapter setEventListener(EventListener eventListener) {
        this.mEventListener = eventListener;
        return this;
    }

    /**
     * 刷新数据
     * @param collection
     * @return
     */
    public MyFocusRecycleAdapter refresh(Collection<ConcernFan> collection){
        mList.clear();
        mList.addAll(collection);
        notifyDataSetChanged();
        mLastPosition = -1;
        return this;
    }

    /**
     * 清空数据
     * @return
     */
    public MyFocusRecycleAdapter empty(){
        mList.clear();
        notifyDataSetChanged();
        mLastPosition = -1;
        return this;
    }
}
