package com.kanglai.push.ui.adapter;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.blankj.utilcode.util.ObjectUtils;
import com.kanglai.push.R;

/**
 * 通知列表回收列表数据适配器
 * @Author: liusixiang007
 * @since: 2023/5/24
 */
public class NotificationListRecyclerAdapter extends RecyclerView.Adapter<NotificationListRecyclerAdapter.ViewHolder> {

    /** 左侧头像 */
    protected Integer defaultImage;
    /** 头像左下角标签 */
    protected Integer defaultBadge;
    /** 隐藏纰漏 */
    protected Boolean hideDisclosure = false;
    /** 是否显示头像 */
    protected Boolean hideLeftImage= false;

    protected int mLastPosition = -1;

    protected boolean mOpenAnimationEnable = true;

    protected NotificationListRecyclerAdapter(){}

    protected NotificationListRecyclerAdapter(Integer defaultImage,
                                              Integer defaultBadge,
                                              Boolean hidesDisclosure,
                                              Boolean hideLeftImage){
        this.defaultImage = defaultImage;
        this.defaultBadge = defaultBadge;
        if (ObjectUtils.isNotEmpty(hidesDisclosure)) this.hideDisclosure = hidesDisclosure;
        if (ObjectUtils.isNotEmpty(hideLeftImage)) this.hideLeftImage = hideLeftImage;
    }

    @Override
    public int getItemCount() {
        return 0;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        final LayoutInflater inflater = LayoutInflater.from(parent.getContext());
        final View v = inflater.inflate(R.layout.item_notice_list, parent, false);
        return new ViewHolder(v);
    }

    @Override
    public void onBindViewHolder(final ViewHolder holder, int position) {
        if (ObjectUtils.isNotEmpty(defaultImage)) {
            holder.leftAvatar.setImageResource(defaultImage);
        }
        if (ObjectUtils.isNotEmpty(defaultBadge)) {
            holder.leftBadge.setImageResource(defaultBadge);
        }
        holder.layoutLeft.setVisibility(hideLeftImage ? View.GONE: View.VISIBLE);
    }

    @Override
    public void onViewAttachedToWindow(@NonNull ViewHolder holder) {
        super.onViewAttachedToWindow(holder);
        addAnimate(holder, holder.getLayoutPosition());
    }

    private void addAnimate(ViewHolder holder, int position) {
        // 添加动画
        if (mOpenAnimationEnable && mLastPosition < position) {
            holder.itemView.setAlpha(0);
            holder.itemView.animate().alpha(1).start();
            mLastPosition = position;
        }
    }

    protected static class ViewHolder extends RecyclerView.ViewHolder{

        public LinearLayout layoutContent;  // 名字+内容+时间
        public RelativeLayout layoutLeft;   // 头像+徽章
        public ImageView leftAvatar;    // 头像
        public ImageView leftBadge;     // 左下角徽章
        public TextView titleLabel01;     // 用户名
        public TextView titleLabel02;     // 时间
        public TextView titleLabel03;     // 消息内容


        public ViewHolder(@NonNull View v) {
            super(v);
            layoutContent = v.findViewById(R.id.notice_layout_content);
            layoutLeft = v.findViewById(R.id.notice_layout_left);
            leftAvatar = v.findViewById(R.id.notice_avatar);
            leftBadge = v.findViewById(R.id.notice_badge);
            titleLabel01 = v.findViewById(R.id.title_label01);
            titleLabel02 = v.findViewById(R.id.title_label02);
            titleLabel03 = v.findViewById(R.id.title_label03);
        }
    }

}
