package com.kanglai.push.ui.adapter;

import android.text.TextUtils;
import android.view.View;

import com.blankj.utilcode.util.ObjectUtils;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.kanglai.push.R;
import com.kanglai.push.entity.PushBlacklist;

import java.util.ArrayList;
import java.util.Collection;

/**
 * 黑名单列表回收视图适配器
 * @Author: liusixiang007
 * @since: 2023/6/28
 */
public class PushBlackListRecyclerAdapter extends DefaultRecyclerAdapter{

    private ArrayList<PushBlacklist> mItemList;

    private EventListener mEventListener;

    public PushBlackListRecyclerAdapter(ArrayList<PushBlacklist> mItemList){
        this.mItemList = mItemList;
    }

    public interface EventListener{
        /** 单项点击事件*/
        void onItemOnClick(PushBlacklist pushBlacklist);
    }

    @Override
    public int getItemCount() {
        return mItemList.size();
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        super.onBindViewHolder(holder, position);
        final PushBlacklist item = mItemList.get(position); // 获取单项
        holder.layoutContent.setOnClickListener(view -> onItemViewClick(item));
        holder.titleLabel.setText(item.getNickName() + " ("+item.getUserName()+")" );
        if (!TextUtils.isEmpty(item.getAvatar())){
            Glide.with(holder.itemView.getContext())
                    .load(item.getAvatar())
                    .centerCrop()
                    .placeholder(R.color.white)
                    .diskCacheStrategy(DiskCacheStrategy.ALL)
                    .into(holder.leftImage);
        }
        holder.leftBadge.setVisibility(View.GONE);  // 隐藏徽章
    }

    private void onItemViewClick(PushBlacklist item) {
        if (mEventListener != null && ObjectUtils.isNotEmpty(mItemList)) {
            mEventListener.onItemOnClick(item);
        }
    }

    public PushBlackListRecyclerAdapter setEventListener(EventListener eventListener) {
        mEventListener = eventListener;
        return this;
    }

    public PushBlackListRecyclerAdapter refresh(Collection<PushBlacklist> collection) {
        mItemList.clear();
        mItemList.addAll(collection);
        notifyDataSetChanged();
        mLastPosition = -1;
        return this;
    }

    public PushBlackListRecyclerAdapter loadMore(Collection<PushBlacklist> collection) {
        mItemList.addAll(collection);
        notifyDataSetChanged();
        return this;
    }

    public PushBlackListRecyclerAdapter insert(Collection<PushBlacklist> collection) {
        mItemList.addAll(0, collection);
        notifyItemRangeInserted(0, collection.size());
        return this;
    }

    /**
     * 清空数据
     * @return
     */
    public PushBlackListRecyclerAdapter empty(){
        mItemList.clear();
        notifyDataSetChanged();
        return this;
    }
}
