package com.kanglai.push.ui.adapter;

import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.blankj.utilcode.util.ObjectUtils;
import com.kanglai.push.R;
import com.kanglai.push.entity.PushType;
import com.suke.widget.SwitchButton;

import java.util.ArrayList;
import java.util.Collection;

/**
 * 推送类型回收列表数据适配器
 * @Author: liusixiang007
 * @since: 2023/6/19
 */
public class PushTypeRecyclerAdapter extends RecyclerView.Adapter<PushTypeRecyclerAdapter.ViewHolder> {

    public ArrayList<PushType> mItemList;

    public EventListener mEventListener;

    public PushTypeRecyclerAdapter(ArrayList<PushType> mItemList) {
        this.mItemList = mItemList;
    }

    public interface EventListener {
        void onItemViewClicked(PushType pushType);
        void onSwitchButtonCheck(PushType pushType,SwitchButton view);
    }

    @Override
    public int getItemCount() {
        return mItemList.size();
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        final LayoutInflater inflater = LayoutInflater.from(parent.getContext());
        final View v = inflater.inflate(R.layout.item_push_type, parent, false);
        return new ViewHolder(v);
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        final PushType item = mItemList.get(position);
        holder.names.setText(item.getName());
        holder.content.setOnClickListener(view -> onItemViewClick(item));
        /** 当被选中的某一项 设置禁用*/
        if ("1".equals(item.getDefaultType())) {
            holder.switchButton.setChecked(true);
            holder.switchButton.setEnabled(false);  // 为默认类型时 禁止点击
        }else{
            holder.switchButton.setEnabled(true);   // 去除禁用
            holder.switchButton.setChecked(false);  // 设置为false
        }
        holder.switchButton.setOnCheckedChangeListener(new SwitchButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(SwitchButton view, boolean isChecked) {
                if (isChecked) onSwitchButtonCheck(item, view);
            }
        });
    }

    /** 单行开关点击事件*/
    private void onSwitchButtonCheck(PushType item, SwitchButton view) {
        if (mEventListener != null && !TextUtils.isEmpty(item.getId())) {
            mEventListener.onSwitchButtonCheck(item, view);
        }
    }

    /** 单行点击事件*/
    private void onItemViewClick(PushType item) {
        if (mEventListener != null && ObjectUtils.isNotEmpty(mItemList)) {
            mEventListener.onItemViewClicked(item);
        }
    }

    public PushTypeRecyclerAdapter setEventListener(EventListener eventListener) {
        mEventListener = eventListener;
        return this;
    }

    public PushTypeRecyclerAdapter refresh(Collection<PushType> collection) {
        mItemList.clear();
        mItemList.addAll(collection);
        notifyDataSetChanged();
        return this;
    }

    public PushTypeRecyclerAdapter clear() {
        mItemList.clear();
        notifyDataSetChanged();
        return this;
    }

    protected static class ViewHolder extends RecyclerView.ViewHolder{

        public TextView names;  // 项目名
        public SwitchButton switchButton;       // 开关
        public LinearLayout content;    // 单行内容

        public ViewHolder(@NonNull View v) {
            super(v);
            names = v.findViewById(R.id.item_name);
            switchButton = v.findViewById(R.id.item_switch_button);
            content = v.findViewById(R.id.item_layout);
        }
    }
}
