package com.kanglai.push.ui.adapter;

import android.text.TextUtils;
import android.util.Patterns;
import android.view.View;

import com.blankj.utilcode.util.ObjectUtils;
import com.bumptech.glide.Glide;
import com.dolphin.core.BuildConfig;
import com.kanglai.push.R;
import com.kanglai.push.entity.LinkEntity;

import java.util.Collection;
import java.util.List;

/**
 * 自定义好友适配器
 * @Author: liusixiang007
 * @since: 2023/2/21
 */
public class SearchRecyclerAdapter extends DefaultRecyclerAdapter {

    private List<LinkEntity> mItemList;
    private EventListener mEventListener;

    public SearchRecyclerAdapter(List<LinkEntity> mItemList){
         this.mItemList = mItemList;
    }

    /** 自定义接口实现实现列表点击事件 */
    public interface EventListener {
        void onItemViewClicked(LinkEntity linkEntity);
    }

    @Override
    public int getItemCount() {
        return mItemList.size();
    }

    @Override
    public void onBindViewHolder(final ViewHolder viewHolder, final int position) {
        super.onBindViewHolder(viewHolder, position);
        final LinkEntity item = mItemList.get(position);

        viewHolder.layoutContent.setOnClickListener(view -> onItemViewClick(item));
        viewHolder.titleLabel.setText(item.getNikeName());
        viewHolder.disclosureImage.setVisibility(View.GONE);    // 隐藏返回箭头
        viewHolder.leftBadge.setVisibility(View.GONE);       // 隐藏徽章
        // 加载图片
        String avater = "";
        if (!TextUtils.isEmpty(avater)) {
            avater = item.getAvatar();
            if (!Patterns.WEB_URL.matcher(avater).matches()) avater = BuildConfig.HOST_URL + item.getAvatar().substring(1);
            // 加载图片
            Glide.with(viewHolder.itemView.getContext())
                    .load(avater)
                    .skipMemoryCache(true)
                    .centerCrop()
                    .override(100, 100)
                    .placeholder(R.drawable.icon_img_loading)    // 加载图片
                    .error(R.drawable.icon_links_avatar)    // 加载错误
                    .into(viewHolder.leftImage);
        }
    }

    private void onItemViewClick(LinkEntity item) {
        if (mEventListener != null && ObjectUtils.isNotEmpty(mItemList)) {
            mEventListener.onItemViewClicked(item);
        }
    }

    /** 设置状态监听 */
    public SearchRecyclerAdapter setEventListener(EventListener eventListener) {
            this.mEventListener = eventListener;
            return this;
    }

    public SearchRecyclerAdapter refresh(Collection<LinkEntity> collection) {
        mItemList.clear();
        mItemList.addAll(collection);
        notifyDataSetChanged();
        mLastPosition = -1;
        return this;
    }

    public SearchRecyclerAdapter empty() {
        mItemList.clear();
        notifyDataSetChanged();
        mLastPosition = -1;
        return this;
    }
}
