package com.kanglai.push.ui.adapter;

import android.view.View;
import android.widget.ImageView;

import androidx.annotation.NonNull;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.StringUtils;
import com.bumptech.glide.Glide;
import com.kanglai.push.R;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.entity.LocalPushChatMsg;
import com.kanglai.push.entity.PushChatMessage;
import com.kanglai.push.entity.User;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Locale;

public class SoloChatRoomAdapter extends ChatRoomListRecyclerAdapter{

    public ArrayList<PushChatMessage> mItemList = new ArrayList<>();

    public User user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, com.kanglai.push.entity.User.CREATOR, new User());   // 当前登录用户的数据
    public LocalPushChatMsg friendData;

    public SoloChatRoomAdapter(LocalPushChatMsg friendData) {
        this.friendData = friendData;
    }

    public SoloChatRoomAdapter(ArrayList<PushChatMessage> mItemList, LocalPushChatMsg friendData) {
        this.mItemList = mItemList;
        this.friendData = friendData;
    }

    @Override
    public int getItemCount() {
        return mItemList.size();
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        final PushChatMessage item  = mItemList.get(position);
        onItemBindingTime(holder, item, position);
        if (StringUtils.equals(user.getId(), item.getUserId())) {
            onItemBindingMine(holder, item);
        } else {
            onItemBindingOther(holder, item);
        }
    }

    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault());

    private void onItemBindingTime(ViewHolder holder, PushChatMessage model, int index) {
        PushChatMessage prev;
        Date lastTime = null;
        Date currentTime = null;
        if (index > 0) {
            prev = mItemList.get(index - 1);
            if (prev.getCreateTime() != null) {
                try {
                    lastTime = dateFormat.parse(prev.getCreateTime());
                    currentTime = dateFormat.parse(model.getCreateTime());
                } catch (ParseException ignored) {}
            }
        }
        if ((lastTime != null && currentTime!=null) && (currentTime.getTime() - lastTime.getTime() < 5 * 60 * 1000)) {
            holder.send_time.setVisibility(View.GONE);
        } else {
            holder.send_time.setVisibility(View.VISIBLE);
            holder.send_time.setText(model.getCreateTime());
        }
    }

    private void onItemBindingOther(ViewHolder holder, PushChatMessage item) {
        holder.right_layout.setVisibility(View.GONE);
        holder.left_layout.setVisibility(View.VISIBLE);
        loadAvatar(holder.left_avatar, friendData.getAvatar());
        holder.chatting_left_img.setVisibility(View.VISIBLE);
        holder.left_text.setText(item.getText());
    }

    private void onItemBindingMine(ViewHolder holder, PushChatMessage item) {
        holder.left_layout.setVisibility(View.GONE);
        holder.right_layout.setVisibility(View.VISIBLE);
        loadAvatar(holder.right_avatar, user.getAvatar());
        holder.chatting_right_img.setVisibility(View.VISIBLE);
        holder.right_text.setText(item.getText());
    }

    private void loadAvatar(ImageView avatar, String url) {
        Glide.with(avatar.getContext())
                .load(url)
                .skipMemoryCache(true)
                .centerCrop()
                .override(300, 300)
                .placeholder(R.drawable.icon_img_loading)    // 加载图片
                .error(R.drawable.icon_links_avatar)    // 加载错误
                .into(avatar);
    }


    public SoloChatRoomAdapter refresh(Collection<PushChatMessage> collection) {
        mItemList.clear();
        mItemList.addAll(collection);
        notifyDataSetChanged();
        mLastPosition = -1;
        return this;
    }

    public SoloChatRoomAdapter loadMore(Collection<PushChatMessage> collection) {
        mItemList.addAll(collection);
        notifyDataSetChanged();
        return this;
    }

    public SoloChatRoomAdapter insert(Collection<PushChatMessage> collection) {
        mItemList.addAll(0, collection);
        notifyItemRangeInserted(0, collection.size());
        return this;
    }

    public SoloChatRoomAdapter append(PushChatMessage pushChatMessage){
        mItemList.add(pushChatMessage);
        notifyItemRangeInserted(mItemList.size() - 1, 1);
        return this;
    }


}
