package com.kanglai.push.ui.fragment;

import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Filter;
import android.widget.ImageView;

import androidx.annotation.Nullable;
import androidx.appcompat.widget.SearchView;
import androidx.recyclerview.widget.LinearLayoutManager;

import com.dolphin.core.base.BaseFragment;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.constant.CommonConstant;
import com.kanglai.push.databinding.FragmentLinkBinding;
import com.kanglai.push.entity.ConcernFan;
import com.kanglai.push.entity.MenuEntity;
import com.kanglai.push.ui.activity.AuditListActivity;
import com.kanglai.push.ui.activity.BlackListActivity;
import com.kanglai.push.ui.activity.FindUsersActivity;
import com.kanglai.push.ui.activity.FocusListActivity;
import com.kanglai.push.ui.activity.LinkDetailActivity;
import com.kanglai.push.ui.adapter.LinkAdapter;
import com.kanglai.push.ui.adapter.MenuClassifyAdapter;
import com.kanglai.push.ui.vm.LinkViewModel;

import java.util.ArrayList;
import java.util.List;

import me.yokeyword.indexablerv.IndexableAdapter;
import me.yokeyword.indexablerv.IndexableHeaderAdapter;
import me.yokeyword.indexablerv.IndexableLayout;


/**
 * 联系人碎片页
 * @Author: liusixiang007
 * @since: 2023/2/15
 */
public class LinkFragment extends BaseFragment<FragmentLinkBinding, LinkViewModel> {

    public List<ConcernFan> mFans;

    /** 搜索框 */
    private SearchView mSearch;
    /** 联系人适配器 */
    public LinkAdapter mAdapter;
    /** 分类菜单 */
    private MenuClassifyAdapter mMenuClassifyAdapter;

    public static String USER_LINK_ID = "USER_LINK_ID";

    public Boolean isSkip = true;  // 防止重复点击

    @Override
    public int setContentView(LayoutInflater inflater, @Nullable ViewGroup parentContainer, @Nullable Bundle savedInstanceState) {
        return R.layout.fragment_link;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        super.mViewModel.mActivity = this;
        /** 隐藏顶部栏返回按钮 */
        ImageView toolbarBack = getView().findViewById(R.id.iv_back);
        toolbarBack.setVisibility(View.INVISIBLE);
        /** 修改顶部新增联系人按钮图标 */
        ImageView rightLogo =  getView().findViewById(R.id.iv_right_icon);
        rightLogo.setImageDrawable(getResources().getDrawable(R.drawable.icon_add_links));
        mViewModel.loadFan();   // 加载粉丝数据
        initIndexable();
    }

    public void initIndexable() {
        IndexableLayout indexableLayout = (IndexableLayout) getView().findViewById(R.id.indexableLayout);
        indexableLayout.setLayoutManager(new LinearLayoutManager(getActivity()));   // 注入试图管理模型
        mAdapter = new LinkAdapter(getActivity());// 设置适配器
        indexableLayout.setAdapter(mAdapter);
        mAdapter.setDatas(mFans);
        indexableLayout.setOverlayStyle_Center();   // 索引悬浮提示框 IOS风格气泡
        indexableLayout.setOverlayStyle_MaterialDesign(Color.parseColor("#409EFF"));  // 触发效果 - 字母检索栏 选中样式
        indexableLayout.setCompareMode(IndexableLayout.MODE_FAST);  // 快速排序。  排序规则设置为：只按首字母

        // 构造函数里3个参数,分别对应 (IndexBar的字母索引, IndexTitle, 数据源), 不想显示哪个就传null, 数据源传null时,代表add一个普通的View
        mMenuClassifyAdapter = new MenuClassifyAdapter("*", null, initMenuData());
        indexableLayout.addHeaderAdapter(mMenuClassifyAdapter);
        mMenuClassifyAdapter.setOnItemHeaderClickListener(new IndexableHeaderAdapter.OnItemHeaderClickListener<MenuEntity>() {
            @Override
            public void onItemClick(View v, int currentPosition, MenuEntity entity) {
                if (!isSkip) return;
                isSkip = false;
                /** 头部菜单点击事件 */
                switch (currentPosition) {
                    case 0:
                        startActivity(FindUsersActivity.class); // 关注对象
                    break;
                    case 1:
                        startActivity(AuditListActivity.class); // 审核列表
                    break;
                    case 2:
                        startActivity(FocusListActivity.class); // 关注-个人
                    break;
//                    case 3:
//                        startActivity(EnterpriseListActivityl.class); // 关注-企业
//                    break;
                    case 3:
                        startActivity(BlackListActivity.class);     // 黑名单
                    break;
                    default:
                        ToastUtil.showBottom(entity.getMenuTitle());
                }
            }
        });
        mAdapter.setOnItemContentClickListener(new IndexableAdapter.OnItemContentClickListener<ConcernFan>() {
            @Override
            public void onItemClick(View v, int originalPosition, int currentPosition, ConcernFan entity) {
                if (!isSkip) return;
                isSkip = false;
                /** 点击详情 */
                Intent intent = new Intent(getActivity(), LinkDetailActivity.class);
                intent.putExtra(CommonConstant.USER_LINK_ID, entity.getId());
                intent.putExtra(CommonConstant.USER_LINK_TYPE, 1);
                startActivity(intent);
            }
        });
        mAdapter.setOnItemTitleClickListener(new IndexableAdapter.OnItemTitleClickListener() {
            @Override
            public void onItemClick(View v, int currentPosition, String indexTitle) {
                /** A~Z 标题字母点击事件*/
            }
        });

        /** 初始化搜索框 */   // todo 去空格以及换行符 用editView实现
        mSearch = (SearchView) getView().findViewById(R.id.link_searchview);
        mSearch.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String query) {
                return false;
            }

            @Override
            public boolean onQueryTextChange(String newText) {
                if (!TextUtils.isEmpty(newText)) dataFilter.filter(newText.toLowerCase());
                else {
                    mAdapter.setDatas(mFans);
                    mAdapter.notifyDataSetChanged();
                }
                return false;
            }
        });

    }

    /** 加载菜单数据 */
    private List<MenuEntity> initMenuData() {
        List<MenuEntity> list = new ArrayList<>();
        list.add(new MenuEntity("添加关注", R.drawable.icon_menus_search));
        list.add(new MenuEntity("好友申请", R.drawable.icon_menus_audit));
        list.add(new MenuEntity("我的关注", R.drawable.icon_menus_focus_user));
        list.add(new MenuEntity("黑名单", R.drawable.icon_menus_focus));
        return list;
    }
    /** 通讯录过滤器 */
    Filter dataFilter = new Filter() {
        @Override
        protected FilterResults performFiltering(CharSequence charSequence) {   // 查找数据
            ArrayList<ConcernFan> list = new ArrayList<>();
            for (ConcernFan item : mFans) {
                if (item.getPhone().contains(charSequence) || item.getUserName().contains(charSequence)) list.add(item);
            }
            FilterResults results = new FilterResults();
            results.count = list.size();
            results.values = list;
            return results;
        }

        @Override
        protected void publishResults(CharSequence charSequence, FilterResults results) {
            ArrayList<ConcernFan> filterList = (ArrayList<ConcernFan>) results.values;
            mAdapter.setDatas(filterList);
            mAdapter.notifyDataSetChanged();
        }
    };

    @Override
    public void onResume() {
        super.onResume();
        mViewModel.loadFan();
    }

    @Override
    public void onStart() {
        super.onStart();
        isSkip = true;
    }
}
