package com.kanglai.push.ui.fragment;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContract;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.CollectionUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.StringUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.base.BaseFragment;
import com.dolphin.core.bus.RxBus;
import com.dolphin.core.bus.RxSubscriptions;
import com.dolphin.core.util.RxUtil;
import com.dolphin.umeng.entity.PushChatMessage;
import com.h6ah4i.android.widget.advrecyclerview.animator.GeneralItemAnimator;
import com.h6ah4i.android.widget.advrecyclerview.animator.SwipeDismissItemAnimator;
import com.h6ah4i.android.widget.advrecyclerview.swipeable.RecyclerViewSwipeManager;
import com.h6ah4i.android.widget.advrecyclerview.touchguard.RecyclerViewTouchActionGuardManager;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.constant.CommonConstant;
import com.kanglai.push.databinding.FragmentMessageBinding;
import com.kanglai.push.entity.LocalPushChatMsg;
import com.kanglai.push.entity.User;
import com.kanglai.push.ui.activity.InstantActivity;
import com.kanglai.push.ui.adapter.MessageSwipeableRecyclerAdapter;
import com.kanglai.push.ui.vm.MessageViewModel;
import com.kanglai.push.util.HistoryMsgUtil;
import com.kanglai.push.util.LocationService;
import com.scwang.smart.refresh.layout.api.RefreshLayout;
import com.umeng.message.PushAgent;

import ezy.ui.layout.LoadingLayout;
import io.reactivex.disposables.Disposable;

/**
 *<p>
 * 消息
 *</p>
 *
 * @Author: entfrm开发团队-王翔
 * @Date: 2022/7/15
 */
public class MessageFragment extends BaseFragment<FragmentMessageBinding, MessageViewModel> implements MessageSwipeableRecyclerAdapter.EventListener, View.OnTouchListener {

    public RecyclerView mRecyclerView;
    public RecyclerViewTouchActionGuardManager mRecyclerViewTouchActionGuardManager;
    public RecyclerViewSwipeManager mRecyclerViewSwipeManager;
    public RecyclerView.Adapter mWrappedAdapter;

    public MessageSwipeableRecyclerAdapter mAdapter;
    public RefreshLayout mRefreshLayout;
    public LoadingLayout mLoadingLayout;
    private Disposable mSubscription;
    private ActivityResultLauncher<LocalPushChatMsg> instantLauncherResult;
    private User user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, User.CREATOR, new User());

    @Override
    public int setContentView(LayoutInflater inflater, @Nullable ViewGroup parentContainer, @Nullable Bundle savedInstanceState) {
        return R.layout.fragment_message;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        /** 接收友盟的消息 */
        mSubscription = RxBus.getInstance().toObservableSticky(PushChatMessage.class)
        .compose(RxUtil.schedulersTransformer())
        .compose(RxUtil.exceptionTransformer())
        .subscribe(msg -> {
            PushChatMessage pushChatMessage = (PushChatMessage) msg;
            if(!TextUtils.isEmpty(pushChatMessage.getFanUserId()) && !StringUtils.equals(pushChatMessage.getFanUserId(), user.getId())) {
                mViewModel.addLocalHistoryMsg(pushChatMessage.getFanUserId(), pushChatMessage.getText());
            }
        });
        RxSubscriptions.add(mSubscription);

        /** 聊天室的回传 */
        instantLauncherResult = registerForActivityResult(new ActivityResultContract<LocalPushChatMsg, LocalPushChatMsg>() {
            @Override
            public LocalPushChatMsg parseResult(int resultCode, @Nullable Intent intent) {
                if (intent == null) {
                    return null;
                }
                return intent.getParcelableExtra(CommonConstant.INSTANT_LAUNCHER_RESULT_KEY);
            }
            @NonNull
            @Override
            public Intent createIntent(@NonNull Context context, LocalPushChatMsg localPushChatMsg) {
                Intent intent = new Intent(getActivity(), InstantActivity.class);
                intent.putExtra(CommonConstant.INSTANT_LAUNCHER_RESULT_KEY, localPushChatMsg);
                return intent;
            }
        }, result -> {
            if (result !=null) {
                mViewModel.addLocalHistoryMsg(result.getId(), result.getLastContent());
            }
        });

    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        super.mViewModel.mActivity = this;
        ImageView toolbarBack = getView().findViewById(R.id.iv_back);
        toolbarBack.setVisibility(View.INVISIBLE);

        HistoryMsgUtil.loadUser();    // 初始化时 更新一次聊天页对应的用户数据

        mRefreshLayout = getView().findViewById(R.id.refreshLayout);
        mRecyclerView = getView().findViewById(R.id.recycler_view);
        mLoadingLayout = getView().findViewById(R.id.loading);

        mRecyclerView.setOnTouchListener(this);
        FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(mRecyclerView.getLayoutParams());
        params.setMargins(0,0,0,0);
        mRecyclerView.setLayoutParams(params);
        mRecyclerViewTouchActionGuardManager = new RecyclerViewTouchActionGuardManager();
        mRecyclerViewTouchActionGuardManager.setInterceptVerticalScrollingWhileAnimationRunning(true);  // 设置在 RecyclerView 动画运行时是否拦截垂直滚动操作
        mRecyclerViewTouchActionGuardManager.setEnabled(true);
        mRecyclerViewSwipeManager = new RecyclerViewSwipeManager();
        final MessageSwipeableRecyclerAdapter adapter = new MessageSwipeableRecyclerAdapter(CollectionUtils.newArrayList());
        adapter.setEventListener(this);
        mAdapter = adapter;
        mWrappedAdapter = mRecyclerViewSwipeManager.createWrappedAdapter(adapter);
        final GeneralItemAnimator animator = new SwipeDismissItemAnimator();
        animator.setSupportsChangeAnimations(false);
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(getActivity());
        mRecyclerView.setLayoutManager(mLayoutManager);
        mRecyclerView.setAdapter(mWrappedAdapter);
        mRecyclerView.setItemAnimator(animator);
        mRecyclerViewTouchActionGuardManager.attachRecyclerView(mRecyclerView);
        mRecyclerViewSwipeManager.attachRecyclerView(mRecyclerView);
        mRefreshLayout.setEnableLoadMore(false);
        mLoadingLayout.showContent();
        mRefreshLayout.autoRefresh();
        mRefreshLayout.setOnRefreshListener(mViewModel::refresh);
    }

    @Override
    public void onStart() {
        super.onStart();
        PushAgent mPushAgent = PushAgent.getInstance(Utils.getApp());
        mPushAgent.setAlias(user.getId(), CommonConstant.UMENG_PUSH_USER_ALIAS_TYPE, (s, m)->{
            if (s) {
                LogUtils.d("alias bind success! type:" + CommonConstant.UMENG_PUSH_USER_ALIAS_TYPE + " alias:" + user.getId());
            } else {
                LogUtils.d("alias bind failure! msg:" + m);
            }
        });
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        RxSubscriptions.remove(mSubscription);
    }

    @Override
    public boolean onTouch(View view, MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            cleanSwipeable();
        }
        return false;
    }

    public void cleanSwipeable() {
        mAdapter.getData().forEach(item -> { if (item.getPinned()) item.setPinned(false); });
        mAdapter.notifyDataSetChanged();
    }

    @Override
    public void onItemPinned(int position) {
        LogUtils.d(" onItemPinned-- " + position);
    }

    @Override
    public void onItemViewTouch(View view, MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            cleanSwipeable();
        }
    }

    private long lastClickTime = 0;

    @Override
    public void onItemViewClicked(View v, LocalPushChatMsg item) {
        long currentTime = System.currentTimeMillis();
        // 设置两次点击的最小时间间隔，例如500毫秒
        long minClickInterval = 500;
        if (currentTime - lastClickTime > minClickInterval) {
            instantLauncherResult.launch(item);
            lastClickTime = currentTime;
        }
    }

    @Override
    public void setToRemove(LocalPushChatMsg item) {
        HistoryMsgUtil.del(item.getId());
        mViewModel.refresh(mRefreshLayout);
    }

    @Override
    public void setToRead(int position) {}

    /**
     * 开启定位服务
     * 如果已开启则不做变化 未开启则开启服务 保证登录之后服务仅启动一次
     */
    public void openLocation() {
        if (!HistoryMsgUtil.isServiceRunning(getActivity(), LocationService.class)) {
            getActivity().startService(new Intent(getActivity(),LocationService.class));
        }
    }
}
