package com.kanglai.push.ui.fragment;

import static com.blankj.utilcode.util.ServiceUtils.stopService;

import android.app.ActivityManager;
import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Patterns;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;

import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.CollectionUtils;
import com.blankj.utilcode.util.LogUtils;
import com.bumptech.glide.Glide;
import com.dolphin.core.BuildConfig;
import com.dolphin.core.base.BaseFragment;
import com.dolphin.core.widget.DefaultItemDecoration;
import com.kanglai.push.BR;
import com.kanglai.push.R;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.databinding.FragmentUserBinding;
import com.kanglai.push.entity.User;
import com.kanglai.push.ui.activity.AboutActivity;
import com.kanglai.push.ui.adapter.PanelRecyclerAdapter;
import com.kanglai.push.ui.vm.UserViewModel;
import com.kanglai.push.util.LocationService;
import com.kongzue.dialogx.DialogX;
import com.kongzue.dialogx.dialogs.MessageDialog;
import com.kongzue.dialogx.interfaces.OnDialogButtonClickListener;

import java.util.List;

/**
 *<p>
 * “我的”碎片页
 *</p>
 *
 * @Author: entfrm开发团队-王翔
 * @Date: 2022/7/15
 */
public class UserFragment extends BaseFragment<FragmentUserBinding, UserViewModel> implements PanelRecyclerAdapter.EventListener {

    private RecyclerView mRecyclerView;
    private PanelRecyclerAdapter mAdapter;

    public ImageView avatar;

    public User user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, User.CREATOR, new User());

    @Override
    public int setContentView(LayoutInflater inflater, @Nullable ViewGroup parentContainer, @Nullable Bundle savedInstanceState) {
        return R.layout.fragment_user;
    }

    @Override
    public int setVariableId() {
        return BR.viewModel;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        super.mViewModel.mActivity = this;

        ImageView toolbarBack = getView().findViewById(R.id.iv_back);
        toolbarBack.setVisibility(View.INVISIBLE);
        mRecyclerView = getView().findViewById(R.id.recycler_view);
        // 设置列表
        List<PanelRecyclerAdapter.PanelEntity> list = CollectionUtils.newArrayList(
                new PanelRecyclerAdapter.PanelEntity().setCode("1").setTitle("关于我们").setBadge(0).setImage(R.drawable.icon_about).setHidesLeftImage(true),
                new PanelRecyclerAdapter.PanelEntity().setCode("2").setTitle("退出登录").setBadge(0).setImage(R.drawable.icon_exit).setHidesLeftImage(true),
                new PanelRecyclerAdapter.PanelEntity().setCode("3").setTitle("测试按钮").setBadge(0).setImage(R.drawable.icon_about).setHidesLeftImage(true)
        );
        final PanelRecyclerAdapter userRecyclerAdapter = new PanelRecyclerAdapter(list);
        userRecyclerAdapter.setEventListener(this);
        mAdapter = userRecyclerAdapter;
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(getActivity());
        mRecyclerView.setLayoutManager(mLayoutManager);
        mRecyclerView.addItemDecoration(new DefaultItemDecoration(ContextCompat.getColor(requireContext(), R.color.common_divider_color)));
        mRecyclerView.setAdapter(mAdapter);

        avatar = getView().findViewById(R.id.avatar);
        String avater = user.getAvatar();
        if (!TextUtils.isEmpty(user.getAvatar()) && !Patterns.WEB_URL.matcher(user.getAvatar()).matches()) avater = BuildConfig.HOST_URL + avater.substring(1);
        Glide.with(getActivity())
                .load(avater)
                .placeholder(R.drawable.icon_img_loading)    // 加载图片
                .error(R.drawable.icon_links_avatar)    // 加载错误
                .into(mView.avatar);

    }

    @Override
    public void onItemViewClicked(PanelRecyclerAdapter.PanelEntity userEntity) {
        /** 列表点击事件 */
        switch (userEntity.code) {
            case "1":     // 关于我们
                startActivity(AboutActivity.class);
            break;
            case "2":   // 退出登录
                MessageDialog.build()
                        .setTheme(DialogX.THEME.LIGHT)
                        .setTitle("您确定要退出登录吗?")
                        .setMessage(" ")
                        .setOkButton("确 认", new OnDialogButtonClickListener<MessageDialog>() {
                            @Override
                            public boolean onClick(MessageDialog dialog, View v) {
                                mViewModel.loginOut();
                                dialog.dismiss();
                                stopService(LocationService.class);     // 停止定位服务
                                return false;
                            }
                        })
                        .setCancelButton("取 消")
                        .show();
            break;
            case "3":   // todo：测试按钮 上线记得清除
//                检测服务是否在线
                ActivityManager manager = (ActivityManager) getActivity().getSystemService(Context.ACTIVITY_SERVICE);
                for (ActivityManager.RunningServiceInfo service : manager.getRunningServices(Integer.MAX_VALUE)){
                    LogUtils.d(service.service.getClassName());
                    if (LocationService.class.getName().equals(service.service.getClassName())){
                        LogUtils.d("在线");
                    }
                }

                break;
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        // 退出时将冗余数据销毁
        if (user != null) user = null;

    }
}
