package com.kanglai.push.ui.vm;

import android.app.Application;
import android.text.Editable;
import android.util.Pair;
import android.view.View;
import android.widget.Filter;

import androidx.annotation.NonNull;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.ConcernFan;
import com.kanglai.push.entity.LinkEntity;
import com.kanglai.push.entity.User;
import com.kanglai.push.service.LinkService;
import com.kanglai.push.ui.activity.AuditListActivity;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 * 好友审核列表
 * @Author: liusixiang007
 * @since: 2023/4/12
 */
public class AuditListViewModel extends ToolbarViewModel<AuditListActivity>{

    @Inject
    LinkService linkService;

    public List<ConcernFan> concernFanList;

    private User user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, User.CREATOR, new User());

    public AuditListViewModel(@NonNull Application application) {
        super(application);
        super.setTitleText("审核列表");
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
        super.setTitleText("审 核 目 录");
    }

    @Override
    public void onResume(@NonNull LifecycleOwner owner) {
        super.onResume(owner);
        loadHttp();
    }

    /** 加载请求数据 */
    public void loadHttp() {
        linkService.getFanList(MapUtils.newHashMap(
                        Pair.create("status", "1"),
                        Pair.create("dynamicJoinType", "1"),
                        Pair.create("fanUserId", user.getId())
                ))
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse<List<ConcernFan>>>() {
                    @Override
                    public void onNext(ResultResponse<List<ConcernFan>> R) {
                        if (R.getCode() == R.SUCCESS) {
                            concernFanList = R.getData();
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {
                        loadAuditData();
                    }
                });
    }

    /** 加载所有的审核数据 */
    public void loadAuditData() {
        List<LinkEntity> list = new ArrayList<>();
        for(ConcernFan item: concernFanList) list.add(new LinkEntity(item.getId(), item.getUserName(), item.getPhone(), item.getNickName(), item.getAvatar(), "", item.getDeptName(), false));
        mActivity.mAdapter.refresh(list);
        if (list.size() == 0) {
            mActivity.empty_img.setVisibility(View.VISIBLE);
            mActivity.mRecyclerView.setVisibility(View.GONE);
        } else {
            mActivity.empty_img.setVisibility(View.GONE);
            mActivity.mRecyclerView.setVisibility(View.VISIBLE);
        }
    }

    /** 筛选 */
    public void filtrateList(Editable editable) {
        filterList.filter(editable);
    }

    Filter filterList = new Filter() {
        @Override
        protected FilterResults performFiltering(CharSequence charSequence) {
            List<LinkEntity> list = new ArrayList<>();
            for(ConcernFan item:concernFanList){
                if (item.getUserName().contains(charSequence) || item.getNickName().contains(charSequence) || item.getPhone().contains(charSequence)) {
                    list.add(new LinkEntity(item.getId(), item.getUserName(), item.getPhone(), item.getNickName(), item.getAvatar(), "", item.getDeptName(), false));
                }
            }
            FilterResults results = new FilterResults();
            results.count = list.size();
            results.values = list;
            return results;
        }

        @Override
        protected void publishResults(CharSequence charSequence, FilterResults filterResults) {
            ArrayList<LinkEntity> linkArrayList = (ArrayList<LinkEntity>) filterResults.values;
            mActivity.mAdapter.refresh(linkArrayList);
            if (linkArrayList.size() == 0) {
                mActivity.empty_img.setVisibility(View.VISIBLE);
                mActivity.mRecyclerView.setVisibility(View.GONE);
            } else {
                mActivity.empty_img.setVisibility(View.GONE);
                mActivity.mRecyclerView.setVisibility(View.VISIBLE);
            }
        }
    };

    /**
     * 通过(2)/驳回(-1)审核申请
     * @param id
     * @param r
     */
    public void auditRequest(String id, String r) {
        linkService.updateAuditStatus(id, r)
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse>() {
                    @Override
                    public void onNext(ResultResponse R) {
                        if (R.getCode() == R.SUCCESS) {
                            ToastUtil.showBottomSuccess("操作成功");
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {
                        loadHttp();
                    }
                });
    }
}
