package com.kanglai.push.ui.vm;

import android.app.Application;
import android.telecom.TelecomManager;
import android.util.Pair;

import androidx.annotation.NonNull;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.CollectionUtils;
import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.PushBlacklist;
import com.kanglai.push.service.LinkService;
import com.kanglai.push.ui.activity.BlackListActivity;
import com.kongzue.dialogx.dialogs.FullScreenDialog;
import com.scwang.smart.refresh.layout.api.RefreshLayout;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 * 黑名单列表vm
 * @Author: liusixiang007
 * @since: 2023/6/28
 */
public class BlackListViewModel extends ToolbarViewModel<BlackListActivity> {

    @Inject
    LinkService linkService;
    public Integer current = 1;
    public ArrayList<PushBlacklist> mList = CollectionUtils.newArrayList();

    public BlackListViewModel(@NonNull Application application) {
        super(application);
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
        super.setTitleText("黑 名 单");
    }

    /**
     * 下拉刷新
     * @param refresh
     */
    public void refresh(RefreshLayout refresh) {
        linkService.getPushBlacklist(
                        MapUtils.newHashMap(
                                Pair.create("size", 10),
                                Pair.create("current", current)
                        )).compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse<List<PushBlacklist>>>() {
                    @Override
                    public void onNext(ResultResponse<List<PushBlacklist>> R) {
                        if (R.getCode() == R.SUCCESS) {
                            if (!CollectionUtils.isEmpty(R.getData())){
                                mActivity.mAdapter.refresh(R.getData());
                                if (R.getTotal() == current){
                                    mActivity.mRefreshLayout.setEnableLoadMore(false);
                                }
                            }else {
                                mActivity.mAdapter.empty();
                                ToastUtil.showBottom("暂无数据");
                                refresh.finishLoadMoreWithNoMoreData();
                                mActivity.mRefreshLayout.setEnableLoadMore(false);
                            }
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {
                        refresh.finishRefresh();
                    }
                });
    }

    /**
     * 加载更多
     * @param refresh
     */
    public void loadMore(RefreshLayout refresh) {
        current = current + 1;
        linkService.getPushBlacklist(
                        MapUtils.newHashMap(
                                Pair.create("size", 10),
                                Pair.create("current", current)
                        )).compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse<List<PushBlacklist>>>() {
                    @Override
                    public void onNext(ResultResponse<List<PushBlacklist>> R) {
                        if (R.getCode() == R.SUCCESS) {
                            if (CollectionUtils.isEmpty(R.getData())){
                                mActivity.mAdapter.insert(R.getData());
                            }else {
                                mActivity.mRefreshLayout.setEnableLoadMore(false);
                                refresh.finishRefreshWithNoMoreData();
                            }
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {
                        refresh.finishRefresh();
                    }
                });
    }

    /**
     * 移除指定用户
     * @param item
     * @param dialog
     */
    public void removePushBlack(PushBlacklist item, FullScreenDialog dialog) {
        linkService.removePushBlack(item.getId())
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse>() {
                    @Override
                    public void onNext(ResultResponse R) {
                        if (R.getCode() == R.SUCCESS) {
                            ToastUtil.showBottomSuccess("操作成功");
                            dialog.dismiss();
                            mActivity.mRefreshLayout.autoRefresh();
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {

                    }
                });
    }
}
