package com.kanglai.push.ui.vm;

import android.app.Application;
import android.text.Editable;
import android.view.View;
import android.widget.Filter;

import androidx.annotation.NonNull;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.LinkEntity;
import com.kanglai.push.entity.PushEnterprise;
import com.kanglai.push.service.LinkService;
import com.kanglai.push.ui.activity.EnterpriseListActivity;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 * 企业列表
 * @Author: liusixiang007
 * @since: 2023/2/24
 */
public class EnterpriseListViewModel extends ToolbarViewModel<EnterpriseListActivity>{

    @Inject
    LinkService linkService;

    public List<PushEnterprise> enterpriseList;

    public EnterpriseListViewModel(@NonNull Application application) {
        super(application);
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
        super.setTitleText("企 业 列 表");
    }

    public void loadEnterpriseData() {
        linkService.getEnterpriseList(MapUtils.newHashMap())
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse<List<PushEnterprise>>>() {
                    @Override
                    public void onNext(ResultResponse<List<PushEnterprise>> R) {
                        if (R.getCode() == R.SUCCESS) {
                            enterpriseList = R.getData();
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                        ToastUtil.showBottomWarn("加载异常");
                    }

                    @Override
                    public void onComplete() {
                        loadAllData();
                    }
                });
    }

    public void loadAllData(){
        List<LinkEntity> list = new ArrayList<>();
        for (PushEnterprise item : enterpriseList){
            list.add(new LinkEntity(item.getId(), item.getName(), "", "", item.getLicense(), item.getDetailedAddress(), "", true));
        }
        mActivity.mAdapter.refresh(list);
        if (list.size() == 0) {
            mActivity.empty_img.setVisibility(View.VISIBLE);
            mActivity.mRecyclerView.setVisibility(View.GONE);
        } else {
            mActivity.empty_img.setVisibility(View.GONE);
            mActivity.mRecyclerView.setVisibility(View.VISIBLE);
        }
    }

    /** 筛选列表 */
    public void filtrateList(Editable editable) {
        filterList.filter(editable);
    }

    Filter filterList = new Filter() {
        @Override
        protected FilterResults performFiltering(CharSequence charSequence) {
            List<LinkEntity> list = new ArrayList<>();
            for (PushEnterprise item : enterpriseList ){
                if (item.getName().contains(charSequence)) {
                    list.add(new LinkEntity(item.getId(), item.getName(), "", "", item.getLicense(), item.getDetailedAddress(),"", true));
                }
            }
            FilterResults results = new FilterResults();
            results.count = list.size();
            results.values = list;
            return results;
        }

        @Override
        protected void publishResults(CharSequence charSequence, FilterResults filterResults) {
            ArrayList<LinkEntity> linkArrayList = (ArrayList<LinkEntity>) filterResults.values;
            mActivity.mAdapter.refresh(linkArrayList);
            if (linkArrayList.size() == 0) {
                mActivity.empty_img.setVisibility(View.VISIBLE);
                mActivity.mRecyclerView.setVisibility(View.GONE);
            } else {
                mActivity.empty_img.setVisibility(View.GONE);
                mActivity.mRecyclerView.setVisibility(View.VISIBLE);
            }
        }
    };
}
