package com.kanglai.push.ui.vm;

import android.app.Application;
import android.os.Handler;
import android.text.Editable;
import android.util.Pair;
import android.view.View;
import android.widget.Filter;

import androidx.annotation.NonNull;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.ConcernFan;
import com.kanglai.push.entity.LinkEntity;
import com.kanglai.push.entity.PushEnterprise;
import com.kanglai.push.entity.User;
import com.kanglai.push.service.LinkService;
import com.kanglai.push.ui.activity.FindUsersActivity;
import com.kongzue.dialogx.dialogs.FullScreenDialog;
import com.kongzue.dialogx.dialogs.WaitDialog;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 * 查找各类用户
 * @Author: liusixiang007
 * @since: 2023/2/20
 */
public class FindUsersViewModel extends ToolbarViewModel<FindUsersActivity> {

    @Inject
    LinkService linkService;
    public List<ConcernFan> fansList;
    public List<PushEnterprise> enterpriseList;

    private User user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, User.CREATOR, new User());

    public FindUsersViewModel(@NonNull Application application) {
        super(application);
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
        super.setTitleText("查找用户");
    }

    /** 加载用户列表数据 */
    public void loadUserList(){
        linkService.getFriendList(
                MapUtils.newHashMap(Pair.create("size","999")))
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse<List<ConcernFan>>>() {
                    @Override
                    public void onNext(ResultResponse<List<ConcernFan>> R) {
                        if (R.getCode() == R.SUCCESS) {
                            fansList = R.getData();
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                        ToastUtil.showBottomWarn("加载异常");
                    }

                    @Override
                    public void onComplete() {
                    }
                });
    }
    /** 加载公司列表 */
    public void loadFirmList(){
        linkService.getEnterpriseList(MapUtils.newHashMap(Pair.create("size","999")))
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse<List<PushEnterprise>>>() {
                    @Override
                    public void onNext(ResultResponse<List<PushEnterprise>> R) {
                        if (R.getCode() == R.SUCCESS) {
                            enterpriseList = R.getData();
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                        ToastUtil.showBottomWarn("加载异常");
                    }

                    @Override
                    public void onComplete() {

                    }
                });
    }

    /** 筛选搜索的数据 */
    public void filterUser(Editable val) {  // 筛选用户
        filterUsers.filter(val);
    }

    public void filterEnterprise(Editable val) {    // 筛选企业
        filterEnterprise.filter(val);
    }

    /** 检索用户列表 */
    Filter filterUsers = new Filter() {
        @Override
        protected FilterResults performFiltering(CharSequence charSequence) {
            List<LinkEntity> list = new ArrayList<>();
            for (ConcernFan item : fansList){
                if (item.getPhone().contains(charSequence) || item.getNickName().contains(charSequence) || item.getUserName().contains(charSequence)) {
                    list.add(new LinkEntity(item.getId(), item.getUserName(), item.getPhone(), item.getNickName(), item.getAvatar(), "", item.getDeptName(), false));
                }
            }
            FilterResults results = new FilterResults();
            results.count = list.size();
            results.values = list;
            return results;
        }

        @Override
        protected void publishResults(CharSequence charSequence, FilterResults results) {
            ArrayList<LinkEntity> linkArrayList = (ArrayList<LinkEntity>) results.values;
            if (linkArrayList.size() != 0) {
                mActivity.hideKeyboard();
                WaitDialog.show(" ");
                new Handler().postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        WaitDialog.dismiss();
                        mActivity.mAdapter.refresh(linkArrayList);
                    }
                },800);
            }else ToastUtil.showCenter("未找到与[ " + charSequence + " ]相类似的用户");
        }
    };

    /** 检索企业列表 */
    Filter filterEnterprise = new Filter() {
        @Override
        protected FilterResults performFiltering(CharSequence charSequence) {
            List<LinkEntity> list = new ArrayList<>();
            for (PushEnterprise item : enterpriseList){
                if (item.getName().contains(charSequence)) {    // 企业暂时根据企业名搜索
                    list.add(new LinkEntity(item.getId(), item.getName(), "", "", item.getLicense(), item.getDetailedAddress(), "", true));
                }
            }
            FilterResults results = new FilterResults();
            results.count = list.size();
            results.values = list;
            return results;
        }

        @Override
        protected void publishResults(CharSequence charSequence, FilterResults filterResults) {
            ArrayList<LinkEntity> linkArrayList = (ArrayList<LinkEntity>) filterResults.values;
            if (linkArrayList.size() != 0) {
                mActivity.hideKeyboard();
                WaitDialog.show(" ");
                new Handler().postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        WaitDialog.dismiss();
                        mActivity.mAdapter.refresh(linkArrayList);
                    }
                },800);
            }else ToastUtil.showCenter("未找到与[ " + charSequence + " ]相类似的企业");
        }
    };


    /** 关注好友 */
    public void attentionUser(String id, FullScreenDialog dialog) {
        linkService.attentionUserSave(
                MapUtils.newHashMap(
                        Pair.create("status", "2"),
                        Pair.create("concernUserId", id),
                        Pair.create("fanUserId",user.getId()))
                )
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse>() {
                    @Override
                    public void onNext(ResultResponse R) {
                        if (R.getCode() == R.SUCCESS) {
                            ToastUtil.showBottomSuccess("关注成功");
                            dialog.dismiss();
                            mActivity.tv_cancel.setVisibility(View.GONE);
                            mActivity.ed_search.setText("");
                            loadUserList();
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {
                        loadUserList(); // 加载用户列表数据
                    }
                });

    }
}
