package com.kanglai.push.ui.vm;

import android.app.Application;
import android.os.Bundle;
import android.util.Pair;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.CollectionUtils;
import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.binding.command.BindingAction;
import com.dolphin.core.binding.command.BindingCommand;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.constant.CommonConstant;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.ConcernFans;
import com.kanglai.push.entity.LocalPushChatMsg;
import com.kanglai.push.entity.PushType;
import com.kanglai.push.service.LinkService;
import com.kanglai.push.ui.activity.FocusDetailActivity;
import com.kanglai.push.ui.activity.InstantActivity;
import com.kanglai.push.util.HistoryMsgUtil;
import com.kongzue.dialogx.DialogX;
import com.kongzue.dialogx.dialogs.BottomMenu;
import com.kongzue.dialogx.dialogs.MessageDialog;
import com.kongzue.dialogx.interfaces.OnDialogButtonClickListener;
import com.kongzue.dialogx.interfaces.OnMenuItemClickListener;

import java.util.List;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 * 关注用户详情
 *
 * 含用户信息消息记录、取关、推送列表
 *
 * @Author: liusixiang007
 * @since: 2023/6/29
 */
public class FocusDetailViewModel extends ToolbarViewModel<FocusDetailActivity> {

    @Inject
    LinkService linkService;
    public ConcernFans followFans;   // 关注用户信息

    public FocusDetailViewModel(@NonNull Application application) {
        super(application);
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
        super.setTitleText("用户详情");
        if (!"5".equals(followFans.getUserType())) super.setRightIconVisible(View.VISIBLE);
    }

    @Override
    protected void rightIconOnClick() {
        super.rightIconOnClick();
        BottomMenu.show(new String[]{"取 关","拉 黑"})
                .setMessage("更多")
                .setOnMenuItemClickListener(new OnMenuItemClickListener<BottomMenu>() {
                    @Override
                    public boolean onClick(BottomMenu menu, CharSequence text, int index) {
                        switch (index) {
                            case 0:
                                MessageDialog.build()
                                        .setTheme(DialogX.THEME.LIGHT)
                                        .setTitle("警 告")
                                        .setMessage("确认取消对用户”"+followFans.getUserName()+"“的关注？")
                                        .setOkButton("确 定", new OnDialogButtonClickListener<MessageDialog>() {
                                            @Override
                                            public boolean onClick(MessageDialog dialog, View v) {
                                                if ("0".equals(followFans.getUserType())) ToastUtil.show("无法取关系统");
                                                else cancelFocus(dialog);
                                                return false;
                                            }
                                        })
                                        .setCancelButton("取 消")
                                        .show();
                            case 1:     // 拉 黑
                                MessageDialog.build()
                                        .setTheme(DialogX.THEME.LIGHT)
                                        .setTitle("警 告")
                                        .setMessage("确认将用户”" + followFans.getUserName()+"“拉入到黑名单中？")
                                        .setOkButton("确 定", new OnDialogButtonClickListener<MessageDialog>() {
                                            @Override
                                            public boolean onClick(MessageDialog dialog, View v) {
                                                shieldRequest(dialog);
                                                return false;
                                            }
                                        })
                                        .setCancelButton("取 消")
                                        .show();
                        }
                        return true;
                    }
                });
    }

    /**
     * 取关当前用户 并清空缓存记录 后返回
     */
    public void cancelFocus(MessageDialog dialog) {     // 取关
        linkService.removeFans(followFans.getId())
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse>() {
                    @Override
                    public void onNext(ResultResponse R) {
                        if (R.getCode() == R.SUCCESS) {
                            HistoryMsgUtil.del(followFans.getConcernUserId());
                            finish();
                            ToastUtil.showBottomSuccess("操作成功");
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                        dialog.dismiss();
                    }

                    @Override
                    public void onComplete() {
                        dialog.dismiss();
                    }
                });
    }

    /**
     * 拉黑 然后清理对应缓存 返回上一页
     */
    public void shieldRequest(MessageDialog dialog) {
        linkService.addPushBlack(
            MapUtils.newHashMap(
                    Pair.create("concernUserId", followFans.getConcernUserId()),
                    Pair.create("fanUserId", followFans.getFanUserId())
            ))
            .compose(RxUtil.schedulersTransformer())
            .compose(RxUtil.exceptionTransformer())
            .doOnSubscribe(this)
            .subscribe(new DisposableObserver<ResultResponse>(){
                @Override
                public void onNext(ResultResponse R) {
                    if (R.getCode() == R.SUCCESS) {
                        HistoryMsgUtil.del(followFans.getConcernUserId());
                        finish();
                        ToastUtil.showBottomSuccess("操作成功");
                    }else ToastUtil.showBottomWarn(R.getMsg());
                }

                @Override
                public void onError(Throwable e) {
                    ExceptionHandle.baseExceptionMsg(e);
                    dialog.dismiss();
                }

                @Override
                public void onComplete() {
                    dialog.dismiss();
                }
            });
    }

    /**
     * 加载用户类型列表数据
     * 默认加载10条 不能够删除以及新增 仅支持修改
     */
    public void loadPushType() {
        linkService.getFocusPushTypeList(
            MapUtils.newHashMap(
                Pair.create("current", 1),
                Pair.create("size", 10),
                Pair.create("concernFanId", followFans.getId())
            )
            ).compose(RxUtil.schedulersTransformer())
            .compose(RxUtil.exceptionTransformer())
            .doOnSubscribe(this)
            .subscribe(new DisposableObserver<ResultResponse<List<PushType>>>() {
                @Override
                public void onNext(ResultResponse<List<PushType>> R) {
                    if (R.getCode() == R.SUCCESS){
                        mActivity.mAdapter.refresh(R.getData());
                        if(CollectionUtils.isNotEmpty(R.getData())) mActivity.listTit.setVisibility(View.VISIBLE);
                    }else ToastUtil.showBottomWarn(R.getMsg());
                }

                @Override
                public void onError(Throwable e) {
                    ExceptionHandle.baseExceptionMsg(e);
                }

                @Override
                public void onComplete() {

                }
            });
    }

    /**
     * 设置指定用户默认推送类型
     * @param pushType
     * @param v
     */
    public void setDefaultType(PushType pushType, MessageDialog v) {
        linkService.setConcernFanDefaultType(
            MapUtils.newHashMap(
                Pair.create("concernFanId", followFans.getId()),
                Pair.create("id", pushType.getId()),
                Pair.create("defaultType", "1")
            )).compose(RxUtil.schedulersTransformer())
            .compose(RxUtil.exceptionTransformer())
            .doOnSubscribe(this)
            .subscribe(new DisposableObserver<ResultResponse>() {
                @Override
                public void onNext(ResultResponse R) {
                    if (R.getCode() == R.SUCCESS) {
                        loadPushType();
                        ToastUtil.showBottomSuccess("操作成功");
                    }else ToastUtil.showBottomWarn(R.getMsg());
                }

                @Override
                public void onError(Throwable e) {
                    ExceptionHandle.baseExceptionMsg(e);
                    v.dismiss();
                }

                @Override
                public void onComplete() {
                    v.dismiss();
                }
            });
    }

    public BindingCommand SendMsgCommand = new BindingCommand(new BindingAction() {     /**进入聊天室*/
        @Override
        public void call() {
            Bundle bundle = new Bundle();
            LocalPushChatMsg localPushChatMsg = new LocalPushChatMsg(followFans.getConcernUserId(), followFans.getAvatar(), followFans.getUserName(), followFans.getUserType(), "0");
            localPushChatMsg.setFocusType("1");
            bundle.putParcelable(CommonConstant.INSTANT_LAUNCHER_RESULT_KEY, localPushChatMsg);
            startActivity(InstantActivity.class, bundle);
        }
    });
}
