package com.kanglai.push.ui.vm;

import android.app.Application;
import android.text.Editable;
import android.util.Pair;
import android.view.View;
import android.widget.Filter;

import androidx.annotation.NonNull;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.ConcernFan;
import com.kanglai.push.entity.User;
import com.kanglai.push.service.LinkService;
import com.kanglai.push.ui.activity.FocusListActivity;
import com.kanglai.push.util.HistoryMsgUtil;
import com.kongzue.dialogx.dialogs.FullScreenDialog;

import java.util.List;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 * 已关注的好友列表
 *
 * @Author: liusixiang007
 * @since: 2023/4/11
 */
public class FocusListViewModel extends ToolbarViewModel<FocusListActivity> {

    @Inject
    LinkService linkService;

    public List<ConcernFan> concernFanList;

    private User user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, User.CREATOR, new User());

    public FocusListViewModel(@NonNull Application application) {
        super(application);
        super.setTitleText("我的关注");
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }


    /** 加载粉丝列表数据 */
    public void loadFocusList() {
        // todo 数据多一点之后就要做分页
        linkService.getFanList(MapUtils.newHashMap(
                   Pair.create("fanUserId", user.getId()),
                   Pair.create("dynamicJoinType", "1"),
                   Pair.create("status", "2")
                ))
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse<List<ConcernFan>>>() {
                    @Override
                    public void onNext(ResultResponse<List<ConcernFan>> R) {
                        if (R.getCode() == R.SUCCESS) {
                            concernFanList = R.getData();
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                        ToastUtil.showBottomWarn("加载异常");
                    }

                    @Override
                    public void onComplete() {
                        refreshView();
                    }
                });
    }

    /*刷新界面视图元素*/
    public void refreshView() {
        mActivity.mAdapter.refresh(concernFanList);
        if (concernFanList.size() == 0) {
            mActivity.empty_img.setVisibility(View.VISIBLE);
            mActivity.mRecyclerView.setVisibility(View.GONE);
        } else {
            mActivity.empty_img.setVisibility(View.GONE);
            mActivity.mRecyclerView.setVisibility(View.VISIBLE);
        }
    }

    /**
     * 取关http请求
     * @param item
     * @param dialog
     */
    public void cancelFocus(ConcernFan item, FullScreenDialog dialog){
        linkService.removeFans(item.getId())
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse>() {
                    @Override
                    public void onNext(ResultResponse R) {
                        if (R.getCode() == R.SUCCESS) {
                            ToastUtil.showBottomSuccess("操作成功");
                            loadFocusList();    // 刷新数据
                            HistoryMsgUtil.del(item.getFanUserId());
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {
                        dialog.dismiss();
                    }
                });
    }

    public void filtrateList(Editable editable) {
        filterList.filter(editable);
    }

    Filter filterList = new Filter() {
        @Override
        protected FilterResults performFiltering(CharSequence charSequence) {
            FilterResults results = new FilterResults();
            results.count = concernFanList.size();
            results.values = concernFanList;
            return results;
        }

        @Override
        protected void publishResults(CharSequence charSequence, FilterResults filterResults) {
            mActivity.mAdapter.refresh(concernFanList);
            if (concernFanList.size() == 0) {
                mActivity.empty_img.setVisibility(View.VISIBLE);
                mActivity.mRecyclerView.setVisibility(View.GONE);
            } else {
                mActivity.empty_img.setVisibility(View.GONE);
                mActivity.mRecyclerView.setVisibility(View.VISIBLE);
            }
        }
    };
}
