package com.kanglai.push.ui.vm;

import android.app.Application;
import android.icu.text.SimpleDateFormat;
import android.icu.util.Calendar;
import android.util.Pair;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.CollectionUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.StringUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.binding.command.BindingCommand;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.LocalPushChatMsg;
import com.kanglai.push.entity.MsgVo;
import com.kanglai.push.entity.PushChatMessage;
import com.kanglai.push.entity.User;
import com.kanglai.push.service.PushService;
import com.kanglai.push.ui.activity.InstantActivity;
import com.scwang.smart.refresh.layout.api.RefreshLayout;

import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 * 即时通讯活动
 * @Author: liusixiang007
 * @since: 2023/5/17
 */
public class InstantViewModel extends ToolbarViewModel<InstantActivity>{
    @Inject
    PushService pushService;

    public User user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, User.CREATOR, new User());

    public LocalPushChatMsg localPushChatMsg;

    public String cursor = "";

    public InstantViewModel(@NonNull Application application) {
        super(application);
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
        super.setRightIconVisible(View.VISIBLE);
    }
    @Override
    protected void rightIconOnClick() { // todo: 后续聊天界面详情从这里触发
        super.rightIconOnClick();
    }

    public BindingCommand uploadImageCommand = new BindingCommand(() -> {
        ToastUtil.showBottom("图片上传");
    });
    public BindingCommand photographCommand = new BindingCommand(() -> {
        ToastUtil.showBottom("拍照上传");
    });
    public BindingCommand uploadVoiceCommand = new BindingCommand(() -> {
        ToastUtil.showBottom("语音");
    });
    public BindingCommand uploadFileCommand = new BindingCommand(() -> {
        ToastUtil.showBottom("文件上传");
    });
    public BindingCommand specifyTaskCommand = new BindingCommand(() -> {
        ToastUtil.showBottom("制定任务");
    });
    public BindingCommand regularTimeCommand = new BindingCommand(() -> {
        // todo 定时消息
        ToastUtil.showBottom("定时任务");
    });

    public BindingCommand sendMsgClickCommand = new BindingCommand(() ->{
        String msg = String.valueOf(mActivity.input_box.getText());
        if (StringUtils.isTrimEmpty(msg)) {
            ToastUtil.show("不能发送空白信息");
            return;
        }
        PushChatMessage chatMessage = new PushChatMessage();
            if ("1".equals(localPushChatMsg.getFocusType())) chatMessage.setConcernFanStatus("1");   // 从关注列表中进入聊天页界面则需要做特殊处理
            chatMessage.setText(msg);
            chatMessage.setRemarks("普通消息");
            chatMessage.setTitle("APP端消息发送-" + user.getNickName() + "到" + localPushChatMsg.getUserName());
            chatMessage.setAlias(localPushChatMsg.getId());
            chatMessage.setUserId(user.getId());
        pushService.concernFanSend(chatMessage)
            .compose(RxUtil.schedulersTransformer())
            .compose(RxUtil.exceptionTransformer())
            .doOnSubscribe(this)
            .subscribe(new DisposableObserver<ResultResponse>() {
                @Override
                public void onNext(ResultResponse R) {
                    if (R.getCode() == R.SUCCESS) {
                        mActivity.input_box.setText("");
                        PushChatMessage chatMessage = new PushChatMessage();
                        chatMessage.setUserId(user.getId());
                        chatMessage.setText(msg);
                        Calendar calendar = Calendar.getInstance();
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault());
                        String currentTime = sdf.format(calendar.getTime());
                        chatMessage.setCreateTime(currentTime);
                        mActivity.mAdapter.append(chatMessage);
                        mActivity.setActivitieResult();

                    } else ToastUtil.showTop(R.getMsg());
                }

                @Override
                public void onError(Throwable e) {
                    if ("1".equals(localPushChatMsg.getFocusType())) {
                        ToastUtil.show("对方还未关注你，你只能发送一条消息，发送失败");
                    }else ExceptionHandle.baseExceptionMsg(e);
                }

                @Override
                public void onComplete() {
                    mActivity.hideKeyboard();
                    mActivity.mRecyclerView.scrollToPosition(mActivity.mAdapter.getItemCount()-1);
                }
            });
    });

    /** 聊天记录初始化 */
    public void initData(RefreshLayout refresh)  {
        pushService.listFriendHistoryMessage(
            MapUtils.newHashMap(
                Pair.create("size", pageSize),
                Pair.create("alias", localPushChatMsg.getId()),
                Pair.create("userId", user.getId())
            ))
            .compose(RxUtil.schedulersTransformer())
            .compose(RxUtil.exceptionTransformer())
            .doOnSubscribe(this)
            .subscribe(new DisposableObserver<ResultResponse<Map<String, Object>>>() {
                @Override
                public void onNext(ResultResponse<Map<String, Object>> R) {
                    if (R.getCode() == R.SUCCESS) {
                        MsgVo msgVo = (MsgVo) R.getData();
                        cursor = msgVo.getCursor();
                        List<PushChatMessage> data = msgVo.getData();
                        if (!CollectionUtils.isEmpty(data)) {
                            Collections.reverse(data);
                            mActivity.mAdapter.refresh(data);
                            mActivity.mRecyclerView.scrollToPosition(mActivity.mAdapter.getItemCount() - 1);
                        }
                    } else ToastUtil.showBottomWarn(R.getMsg());
                }
                @Override
                public void onError(Throwable e) {
                    ExceptionHandle.baseExceptionMsg(e);
                }
                @Override
                public void onComplete() {}
            });
    }

    /** 加载更多历史记录 */
    public void loadMore(RefreshLayout layout) {
        pushService.listFriendHistoryMessage(
            MapUtils.newHashMap(
            Pair.create("size", pageSize),
            Pair.create("current", pageCurrent += 1),
            Pair.create("alias", localPushChatMsg.getId()),
            Pair.create("userId", user.getId())
            ))
            .compose(RxUtil.schedulersTransformer())
            .compose(RxUtil.exceptionTransformer())
            .doOnSubscribe(this)
            .subscribe(new DisposableObserver<ResultResponse<Map<String, Object>>>() {
                @Override
                public void onNext(ResultResponse<Map<String, Object>> R) {
                    if (R.getCode() == R.SUCCESS) {
                        MsgVo msgVo = (MsgVo) R.getData();
                        cursor = msgVo.getCursor();
                        List<PushChatMessage> pushChatMessages = msgVo.getData();
                        if (CollectionUtils.isEmpty(pushChatMessages)) {
                            ToastUtil.showBottomSuccess("数据加载完毕");
                            layout.finishLoadMore();
                            return;
                        }
                        Collections.reverse(pushChatMessages);
                        mActivity.mAdapter.insert(msgVo.getData());
                        layout.finishLoadMore();
                    } else ToastUtil.show(R.getMsg());
                }

                @Override
                public void onError(Throwable e) {
                    layout.finishLoadMore(false);
                    ExceptionHandle.baseExceptionMsg(e);
                }

                @Override
                public void onComplete() {}
            });
    }
}
