package com.kanglai.push.ui.vm;

import android.app.Application;
import android.os.Bundle;
import android.util.Pair;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.binding.command.BindingAction;
import com.dolphin.core.binding.command.BindingCommand;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.constant.CommonConstant;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.ConcernFan;
import com.kanglai.push.entity.LocalPushChatMsg;
import com.kanglai.push.entity.User;
import com.kanglai.push.service.LinkService;
import com.kanglai.push.ui.activity.InstantActivity;
import com.kanglai.push.ui.activity.LinkDetailActivity;
import com.kanglai.push.util.HistoryMsgUtil;
import com.kongzue.dialogx.DialogX;
import com.kongzue.dialogx.dialogs.BottomMenu;
import com.kongzue.dialogx.dialogs.MessageDialog;
import com.kongzue.dialogx.interfaces.OnDialogButtonClickListener;
import com.kongzue.dialogx.interfaces.OnMenuItemClickListener;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 * 联系人详情
 * @Author: liusixiang007
 * @since: 2023/2/15
 */
public class LinkDetailViewModel extends ToolbarViewModel<LinkDetailActivity> {

    @Inject
    LinkService linkService;
    public ConcernFan concernFan;
    private User user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, User.CREATOR, new User());

    public LinkDetailViewModel(@NonNull Application application) {
        super(application);
        // 组件注册
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
        super.setRightIconVisible(View.VISIBLE);
    }

    /**
     * 右上角更多按钮
     * 根据不同类型用户展示数据
     *
     */
    @Override
    protected void rightIconOnClick() {
        super.rightIconOnClick();
        BottomMenu.show(new String[]{"删 除","拉 黑"})
                .setMessage("更多")
                .setOnMenuItemClickListener(new OnMenuItemClickListener<BottomMenu>() {
                    @Override
                    public boolean onClick(BottomMenu menu, CharSequence text, int index) {
                        switch (index) {
                            case 0:
                                MessageDialog.build()
                                        .setTheme(DialogX.THEME.LIGHT)
                                        .setTitle("警告")
                                        .setMessage("将粉丝”"+concernFan.getUserName()+"“删除，同时移除与该粉丝的聊天记录")
                                        .setOkButton("确 认", new OnDialogButtonClickListener<MessageDialog>() {
                                            @Override
                                            public boolean onClick(MessageDialog dialog, View v) {
                                                removeRequest(dialog);
                                                dialog.dismiss();
                                                return false;
                                            }
                                        })
                                        .setCancelButton("取 消")
                                        .show();
                                return true;
                            case 1:     // 拉黑
                                MessageDialog.build()
                                        .setTheme(DialogX.THEME.LIGHT)
                                        .setTitle("警告")
                                        .setMessage("将用户”"+concernFan.getUserName()+"“拉黑？")
                                        .setOkButton("确 定", new OnDialogButtonClickListener<MessageDialog>() {
                                            @Override
                                            public boolean onClick(MessageDialog dialog, View v) {
                                                shieldRequest(dialog);
                                                return false;
                                            }
                                        })
                                        .setCancelButton("取 消")
                                        .show();
                                return true;
                            default:
                                return false;
                        }
                    }
                });
    }

    /**
     * 拉黑http请求
     * @param dialog
     */
    private void shieldRequest(MessageDialog dialog) {
        linkService.addPushBlack(
            MapUtils.newHashMap(
                Pair.create("concernUserId", concernFan.getConcernUserId()),
                Pair.create("fanUserId", concernFan.getFanUserId())
            )).compose(RxUtil.schedulersTransformer())
            .compose(RxUtil.exceptionTransformer())
            .doOnSubscribe(this)
            .subscribe(new DisposableObserver<ResultResponse>(){

                @Override
                public void onNext(ResultResponse R) {
                    if (R.getCode() == R.SUCCESS) {
                        HistoryMsgUtil.del(concernFan.getFanUserId());
                        finish();
                        ToastUtil.showBottomSuccess("操作成功");
                    }else ToastUtil.showBottomWarn(R.getMsg());
                }

                @Override
                public void onError(Throwable e) {
                    dialog.dismiss();
                    ExceptionHandle.baseExceptionMsg(e);
                }

                @Override
                public void onComplete() {
                    dialog.dismiss();
                }
            });
    }

    /** 移除粉丝http请求 */
    private void removeRequest(MessageDialog menu) {
        linkService.removeFans(concernFan.getId())
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse>() {
                    @Override
                    public void onNext(ResultResponse R) {
                        if (R.getCode() == R.SUCCESS) {
                            ToastUtil.showBottomSuccess("操作成功");
                            /*清除缓存的聊天记录*/
                            HistoryMsgUtil.del(concernFan.getFanUserId());
                            finish();
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                        menu.dismiss();
                    }

                    @Override
                    public void onComplete() {
                        menu.dismiss();
                    }
                });
    }

    /** 发送消息 */
    public BindingCommand SendMsgCommand = new BindingCommand(new BindingAction() {
        @Override
        public void call() {
            Bundle bundle = new Bundle();
            bundle.putParcelable(CommonConstant.INSTANT_LAUNCHER_RESULT_KEY, new LocalPushChatMsg(concernFan.getFanUserId(), concernFan.getAvatar(), concernFan.getUserName(), concernFan.getUserType(), "0"));
            startActivity(InstantActivity.class, bundle);
        }
    });

    /** 加载联系人详情数据 */
    public void getUserMsgById(String id) {
        linkService.getFansById(id)
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse<ConcernFan>>() {
                    @Override
                    public void onNext(ResultResponse<ConcernFan> R) {
                        if (R.getCode() == R.SUCCESS) {
                            mActivity.loadUserMsg(R.getData());
                            concernFan = R.getData();
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {
                    }
                });
    }
}
