package com.kanglai.push.ui.vm;

import android.app.Application;
import android.util.Pair;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.ConcernFan;
import com.kanglai.push.entity.User;
import com.kanglai.push.service.LinkService;
import com.kanglai.push.ui.activity.FindUsersActivity;
import com.kanglai.push.ui.fragment.LinkFragment;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 * 联系人
 * @Author: liusixiang007
 * @since: 2023/2/15
 */
public class LinkViewModel extends ToolbarViewModel<LinkFragment>{

    public Integer pageCurrent = 1;
    public Integer pageSize = 9999;
    private List<ConcernFan> mData;

    @Inject
    LinkService linkService;

    private User user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, User.CREATOR, new User());

    public LinkViewModel(@NonNull Application application) {
        super(application);
        super.setTitleText("粉 丝");
        // 组件注册
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
        super.setRightIconVisible(View.VISIBLE);
    }

    @Override
    protected void rightIconOnClick() {
        super.rightIconOnClick();
        startActivity(FindUsersActivity.class);
    }

    public void loadFan(){
        // todo: 暂时 加载全部数据 数据量多起来就 再做分页
        linkService.getFanList(MapUtils.newHashMap(
                Pair.create("current", pageCurrent),
               Pair.create("size", pageSize),
                Pair.create("status", "2"),
                Pair.create("dynamicJoinType", "0"),
                Pair.create("concernUserId", user.getId())
                ))
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse<List<ConcernFan>>>() {
                    @Override
                    public void onNext(ResultResponse<List<ConcernFan>> R) {
                        if (R.getCode() == R.SUCCESS) {
//                            List<ConcernFan> list = new ArrayList<>(R.getData());
                            mActivity.mFans = R.getData();
                            mActivity.mAdapter.setDatas(R.getData());
                            mActivity.mAdapter.notifyDataSetChanged();
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {
                        LogUtils.d("好友加载成功");
                    }
                });
    }
}
