package com.kanglai.push.ui.vm;

import android.app.Application;

import androidx.annotation.NonNull;
import androidx.databinding.ObservableField;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.StringUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.base.BaseViewModel;
import com.dolphin.core.binding.command.BindingCommand;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.SsoUser;
import com.kanglai.push.entity.User;
import com.kanglai.push.service.LoginService;
import com.kanglai.push.ui.activity.LoginSurveyActivity;
import com.kanglai.push.ui.activity.TabBarActivity;
import com.kanglai.push.util.RandomUtil;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 * 登录问卷 - 用户第一次登录时必须填写的用户信息
 *
 * @Author: liusixiang007
 * @since: 2024/1/3
 */
public class LoginSurveyViewModel extends BaseViewModel<LoginSurveyActivity> {

    @Inject
    LoginService loginService;

    /** 女生单选框 */
    public ObservableField<Boolean> womanCheckedStatus = new ObservableField<>(false);

    /** 男生单选框 默认值 */
    public ObservableField<Boolean> manCheckedStatus = new ObservableField<>(true);

    /** 名字输入框值 */
    public ObservableField<String> inputName = new ObservableField<>("");

    /**  女生单选框点击事件 */
    public BindingCommand womanClickCommand = new BindingCommand(() ->{
        // 选中女
        womanCheckedStatus.set(true);
        manCheckedStatus.set(false);
    });

    /** 男生单选框点击事件 */
    public BindingCommand manClickCommand = new BindingCommand(() ->{
        // 选中男
        manCheckedStatus.set(true);
        womanCheckedStatus.set(false);
    });

    /** 昵称输入框修改 */
    public BindingCommand inputNameValueChangeCommand = new BindingCommand<String>(newName -> {
        if (!StringUtils.isTrimEmpty(newName)) {
            inputName.set(newName);
        }
    });

    /** 获取随机姓名 */
    public BindingCommand randomNameClickCommand = new BindingCommand(() -> {
        inputName.set(RandomUtil.getRandomName(true, 3));
    });

    /** 跳过按钮 */
    public BindingCommand skipBtnClickCommand = new BindingCommand(() ->{
        if (StringUtils.isTrimEmpty(inputName.get())) inputName.set(RandomUtil.getRandomName(true, 3)); // 如果没输入名字 则随机一个
        submitFormMsg(inputName.get(), womanCheckedStatus.get());
    });
    public BindingCommand submitBtnClickCommand = new BindingCommand(() ->{
        if (StringUtils.isTrimEmpty(inputName.get())) {     // 如果没输入名字 则随机一个
            ToastUtil.show("请输入姓名");
            return;
        }
        submitFormMsg(inputName.get(), womanCheckedStatus.get());
    });

    public LoginSurveyViewModel(@NonNull Application application) {
        super(application);
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
        inputName.set(RandomUtil.getRandomName(true, 3));
    }

    /**
     * 表单提交
     * @param name
     * @param isMan
     */
    private void submitFormMsg(String name, Boolean isMan) {
        String sex = isMan ? "0" : "1";
        User user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, User.CREATOR, new User());
        SsoUser ssoUser = new SsoUser(user.getId(), sex, name);

        loginService.updateSsoUser(ssoUser)
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse>() {
                    @Override
                    public void onNext(ResultResponse R) {
                        if (R.getCode() == R.SUCCESS) {
                            user.setSex(sex);
                            user.setNickName(name);
                            CacheDiskUtils.getInstance().put(CacheConstant.USER_INFO, user);
                        }
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {
                        startActivity(TabBarActivity.class);
                    }
                });
     }

}
