package com.kanglai.push.ui.vm;

import android.app.Application;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Pair;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.databinding.ObservableField;
import androidx.databinding.ObservableInt;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.StringUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.base.BaseViewModel;
import com.dolphin.core.binding.command.BindingCommand;
import com.dolphin.core.constant.AppConstant;
import com.dolphin.core.entity.KiccUser;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.R;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.constant.CommonConstant;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.TokenEnhancer;
import com.kanglai.push.entity.User;
import com.kanglai.push.service.LoginService;
import com.kanglai.push.ui.activity.DocumentActivity;
import com.kanglai.push.ui.activity.LoginActivity;
import com.kanglai.push.ui.activity.LoginSurveyActivity;
import com.kanglai.push.ui.activity.TabBarActivity;
import com.kanglai.push.util.CountDownTimerUtils;
import com.kongzue.dialogx.dialogs.WaitDialog;
import com.tencent.mmkv.MMKV;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 *<p>
 * 登录视图模型层
 * 提供模板规范代码参考,请尽量保证编写代码风格跟模板规范代码一致
 * 采用 rxJava 可观测编写
 * Copyright © 2020-2022 <a href="http://www.entfrm.com/">entfrm</a> All rights reserved.
 *</p>
 *
 * @Author: entfrm开发团队-王翔
 * @since: 2022/10/14
 */
public class LoginViewModel extends BaseViewModel<LoginActivity> {

    @Inject
    LoginService loginService;

    /** 用户名称 */
    public ObservableField<String> username = new ObservableField("");

    /** 用户密码 */
//    public ObservableField<String> password = new ObservableField("123456");

    /** 电话号码 */
    public ObservableField<String> phones = new ObservableField("");

    /** 验证码 */
    public ObservableField<String> captcha = new ObservableField("");

    /** 隐私协议确认按钮 */
    public ObservableField<Boolean> agreement = new ObservableField<>(true);

    /** 清空电话号码操作图标可见 */
    public ObservableInt phoneCleanVisible = new ObservableInt();

    /** 用户是否为第一次注册 */
    public Boolean isRegister = false;

    /** 手机号清除焦点修改事件 */
    public BindingCommand phoneFocusChangeCommand = new BindingCommand<Boolean>(hasFocus -> {
        if(hasFocus && !StringUtils.isTrimEmpty(phones.get())) {
            phoneCleanVisible.set(View.VISIBLE);
        } else phoneCleanVisible.set(View.INVISIBLE);
    });

    /** 手机号清除点击事件 */
    public BindingCommand phoneCleanClickCommand = new BindingCommand(() -> {
        phones.set("");
        loginState02();
        if (!StringUtils.isEmpty(captcha.get())) captcha.set("");
    });

    /** 手机号输入框值更改事件 */
    public BindingCommand phoneValueChangeCommand = new BindingCommand<String>(value -> {
        if(!StringUtils.isTrimEmpty(value)) {
            phoneCleanVisible.set(View.VISIBLE);
            if (!StringUtils.isTrimEmpty(captcha.get()) && agreement.get()) loginState01();
            else loginState02();
        } else {
            phoneCleanVisible.set(View.INVISIBLE);
            loginState02();
        }
    });

    /** 验证码输入框值修改事件 */
    public BindingCommand captchaValueChangeCommand = new BindingCommand<String>(value -> {
        if (!StringUtils.isTrimEmpty(value)){   // 同意内容之后 手机号以及电话不能为空
            if (!StringUtils.isTrimEmpty(phones.get()) && agreement.get()){
                loginState01();
            }else loginState02();
        }else loginState02();
    });

    /** 隐私协议checkBox修改事件 */
    public BindingCommand agreementClickCommand = new BindingCommand(() -> {
        agreement.set(!agreement.get());
        if (agreement.get()){   // 同意内容之后 手机号以及电话不能为空
            if (!StringUtils.isTrimEmpty(phones.get()) && !StringUtils.isTrimEmpty(captcha.get())){
                loginState01();
            }else loginState02();
        }else loginState02();
    });

    /** 用户协议 */
    public BindingCommand documentClickCommand = new BindingCommand(() ->{
        Bundle bundle = new Bundle();
        bundle.putString("DOCUMENT_TYPE", "0");
        startActivity(DocumentActivity.class,  bundle);
    });

    /** 隐私政策 */
    public BindingCommand documentsClickCommand = new BindingCommand(() ->{
        Bundle bundle = new Bundle();
        bundle.putString("DOCUMENT_TYPE", "1");
        startActivity(DocumentActivity.class,  bundle);
    });

    public void loginState01() {  // 可登录状态
        mActivity.loginBtn.setBackgroundResource(R.drawable.icon_btn_green_bg);
        mActivity.loginBtn.setTextColor(Color.parseColor("#F2FFFFFF"));
    }

    public void loginState02() {    // 默认-表单待完善状态
        mActivity.loginBtn.setBackgroundResource(R.drawable.icon_btn_gray_bg);
        mActivity.loginBtn.setTextColor(Color.parseColor("#40000000"));
    }

    public LoginViewModel(@NonNull Application application) {
        super(application);
        // 注入服务组件
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication)Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    /** 获取验证码点击事件 */
    public BindingCommand captchaClickCommand = new BindingCommand(() -> {
        if (StringUtils.isTrimEmpty(phones.get())) {
            ToastUtil.showBottomWarn("请输入电话号码!");
            return;
        }
        loginService.sendSmsCode(phones.get())
            .compose(RxUtil.schedulersTransformer())
            .compose(RxUtil.exceptionTransformer())
            // 生命周期同步,ViewModel销毁时会清除异步观测
            .doOnSubscribe(this)
            .doOnSubscribe(disposable -> WaitDialog.show(""))
            .subscribe(new DisposableObserver<ResultResponse>() {

                @Override
                public void onNext(ResultResponse R) {
                    if (Boolean.parseBoolean(String.valueOf(R.getData()))) {
                        ToastUtil.showBottomSuccess("验证码已发送，注意查收");
                        CountDownTimerUtils countDownTimerUtils = new CountDownTimerUtils(mActivity.mTextView, 60000, 1000);
                        countDownTimerUtils.start();
                    } else ToastUtil.showBottomWarn(R.getMsg());
                }

                @Override
                public void onError(Throwable e) {
                    WaitDialog.dismiss();
                    ExceptionHandle.baseExceptionMsg(e);
                }

                @Override
                public void onComplete() {
                    WaitDialog.dismiss();
                }
            });
    });

    /** 通过 手机号、验证码、用户类型、租户id 查询用户用户信息 */
    public BindingCommand loginClickCommand = new BindingCommand(() -> {
        if (StringUtils.isTrimEmpty(phones.get())) {
            ToastUtil.showBottomWarn("请输入手机号");
            return;
        }
        if (StringUtils.isTrimEmpty(captcha.get())) {
            ToastUtil.showBottomWarn("请输入验证码");
            return;
        }
        if (!agreement.get()) {
            ToastUtil.showBottomWarn("请确认用户协议以及营私政策");
            return;
        }
        loginService.getPhoneRegister(MapUtils.newHashMap(
                    Pair.create("identityProvider", "app"), // 用户类型
                    Pair.create("tenantId", CommonConstant.MULTI_TENANT_ID), // 租户类型
                    Pair.create("phone", phones.get()),
                    Pair.create("captcha", captcha.get())))
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                // 生命周期同步,ViewModel销毁时会清除异步观测
                .doOnSubscribe(this)
                .doOnSubscribe(disposable -> WaitDialog.show(" "))      // 若无异常状态则一直处于加载状态(禁止用户进行其他操作)，直至登录成功以及界面跳转前
                .subscribe(new DisposableObserver<ResultResponse<User>>() {
                    @Override
                    public void onNext(ResultResponse<User> R) {
                        if (R.getCode() == R.SUCCESS) {
                            User user = R.getData();
                            login(user);
                            if (user.getRegister() != null && user.getRegister()) isRegister = true;
                        }else {
                            WaitDialog.dismiss();
                            ToastUtil.showBottomWarn(R.getMsg());
                        }
                    }

                    @Override
                    public void onError(Throwable e) {
                        WaitDialog.dismiss();
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {
                        captcha.set("");   // 验证码+电话 验证之后就清空一次验证码
                    }
                });
    });

    private void login(User user) {
        // 请求采用 RxJava2CallAdapterFactory 订阅监听处理
        loginService.login(new KiccUser().setUsername(
                    user.getUserName()).setPassword(user.getPassword()),    // 用户名 + 密码
                    CommonConstant.ID_ENTITY_PROVIDER, // 用户类型
                    CommonConstant.MULTI_TENANT_ID)    //租户id
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                // 生命周期同步,ViewModel销毁时会清除异步观测
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<TokenEnhancer>() {
                    @Override
                    public void onNext(TokenEnhancer tokenEnhancer) {
                        MMKV.defaultMMKV().putString(AppConstant.ACCESS_TOKEN_NAME, tokenEnhancer.getAccess_token());
                        MMKV.defaultMMKV().putString(AppConstant.REFRESH_TOKEN_NAME, tokenEnhancer.getRefresh_token());
                        // 请求最好不要使用同步的会阻塞ui主线程,否则超过5秒卡顿会报ANR错误,如果非要创建同步请求,尽力控制在5秒
                        // 实现方法重写com.dolphin.core.http.HttpRequest的retrofit去掉addCallAdapterFactory,极力不推荐
                        // https://developer.android.com/topic/performance/vitals/anr?hl=zh-cn
                        loginService.getUserInfo()
                                .compose(RxUtil.schedulersTransformer())
                                .compose(RxUtil.exceptionTransformer())
                                .doOnSubscribe(LoginViewModel.this)
                                .subscribe(new DisposableObserver<ResultResponse<User>>() {
                                    @Override
                                    public void onNext(ResultResponse<User> R) {
                                        if (R.getCode() == R.SUCCESS) {
                                            if ("2".equals(R.getData().getUserType()) || "5".equals(R.getData().getUserType())) { // 用户类型不能够是企业用户
                                                loginService.logout().compose(RxUtil.schedulersTransformer()).doOnSubscribe(LoginViewModel.this);
                                                ToastUtil.showBottomWarn("app仅支持普通用户登录");
                                            }else {
                                                // 采用磁盘缓存存储数据,内部会根据Lru淘汰策略进行淘汰如果缓存满了会丢掉最后一个缓存对象
                                                User user = R.getData();
                                                if (StringUtils.isTrimEmpty(user.getAvatar())) user.setAvatar("/default.png");
                                                CacheDiskUtils.getInstance().put(CacheConstant.USER_INFO, user);
                                                if (isRegister) {   // 判断是否第一次登录 isRegister
                                                    startActivity(LoginSurveyActivity.class);
                                                    return;
                                                }
                                                startActivity(TabBarActivity.class);
                                            }
                                        } else ToastUtil.showBottomWarn(R.getMsg());
                                    }
                                    @Override
                                    public void onError(Throwable e) {
                                        ExceptionHandle.baseExceptionMsg(e);
                                    }
                                    @Override
                                    public void onComplete() {

                                    }
                                });
                    }
                    @Override
                    public void onError(Throwable e) {
                        WaitDialog.dismiss();
                        ExceptionHandle.baseExceptionMsg(e);
                    }
                    @Override
                    public void onComplete() {
                        WaitDialog.dismiss();
                    }
                });
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
    }

    @Override
    public void onStart(@NonNull LifecycleOwner owner) {
        super.onStart(owner);
        // 设置图标初始化可见默认值
        phoneCleanVisible.set(View.INVISIBLE);
    }

}
