package com.kanglai.push.ui.vm;

import android.app.Application;
import android.content.Intent;

import androidx.annotation.NonNull;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.ArrayUtils;
import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.http.HttpRequest;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.PermissionUtil;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.LocalPushChatMsg;
import com.kanglai.push.entity.ResultVo;
import com.kanglai.push.entity.SystemUser;
import com.kanglai.push.entity.User;
import com.kanglai.push.service.PushService;
import com.kanglai.push.ui.fragment.MessageFragment;
import com.kanglai.push.util.HistoryMsgUtil;
import com.kanglai.push.util.LocationService;
import com.kanglai.push.util.TimeFormatUtil;
import com.scwang.smart.refresh.layout.api.RefreshLayout;

import java.util.Map;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

/**
 *<p>
 * 消息视图模型层
 *</p>
 *
 * @Author: entfrm开发团队-王翔
 * @since: 2022/11/1
 */
public class MessageViewModel extends ToolbarViewModel<MessageFragment> {

    @Inject
    PushService pushService;

    public MessageViewModel(@NonNull Application application) {
        super(application);
        super.setTitleText("友 信");
        // 注入服务组件
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
    }

    /** 核心 - 刷新数据 */
    public void refresh(RefreshLayout refresh) {
        if (HistoryMsgUtil.noneId()) {
            getUserInfo(refresh);
            return;
        }
        Map<String, LocalPushChatMsg> localHistoryMsg = HistoryMsgUtil.select();
        if (localHistoryMsg != null && !localHistoryMsg.isEmpty()) {
            mActivity.mAdapter.refresh(localHistoryMsg.values());
            mActivity.mLoadingLayout.showContent();
        } else mActivity.mLoadingLayout.showEmpty();
        refresh.finishRefresh();
        mActivity.openLocation();   // 开启定位服务
    }

    /** 加载一次个人信息后 重新加载界面 */
    private void getUserInfo(RefreshLayout refresh) {
        pushService.getUserInfo()
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse<User>>() {
                    @Override
                    public void onNext(ResultResponse<User> R) {
                        /** 请求成功之后 直接判断 */
                        if (R.getCode() == R.SUCCESS) {
                            User user = R.getData();
                            CacheDiskUtils.getInstance().put(CacheConstant.USER_INFO, user);
                            HistoryMsgUtil.loadUser();
                            refresh(refresh);
                        }else {
                            ToastUtil.showBottomWarn("用户信息加载失败，请重新登录");
                            HttpRequest.getInstance().retrofit.create(ExceptionHandle.LoginMapper.class)
                                    .logout().compose(RxUtil.schedulersTransformer());
                            PermissionUtil.logout();
                        }
                    }

                    @Override
                    public void onError(Throwable e) {
                        ToastUtil.showBottomWarn("用户信息加载失败，请重新登录");
                        ExceptionHandle.baseExceptionMsg(e);
                        HttpRequest.getInstance().retrofit.create(ExceptionHandle.LoginMapper.class)
                                .logout().compose(RxUtil.schedulersTransformer());
                        PermissionUtil.logout();
                    }

                    @Override
                    public void onComplete() {
                    }
                });
    }

    /** 加载新的缓存消息记录 */
    public void addLocalHistoryMsg(String userId, String content) {
        pushService.getUserById(userId)
            .compose(RxUtil.schedulersTransformer())
            .compose(RxUtil.exceptionTransformer())
            .doOnSubscribe(this)
            .subscribe(new DisposableObserver<ResultResponse<ResultVo<SystemUser>>>() {
                @Override
                public void onNext(ResultResponse<ResultVo<SystemUser>> R) {
                    if (R.getCode() == R.SUCCESS) {
                        if (R.getData().getResult() != null) {
                            SystemUser systemUser = R.getData().getResult();
                            LocalPushChatMsg newMsg = new LocalPushChatMsg(userId, systemUser.getAvatar(), systemUser.getNickName(), systemUser.getUserType(), "0");
                            newMsg.setLastContent(content);
                            newMsg.setSendTime(TimeFormatUtil.getCurrentTime());
                            HistoryMsgUtil.put(userId, newMsg);
                            refresh(mActivity.mRefreshLayout);
                        }else ToastUtil.showBottom("用户不存在");
                    }else ToastUtil.showBottom(R.getMsg());
                }

                @Override
                public void onError(Throwable e) {
                    ExceptionHandle.baseExceptionMsg(e);
                }

                @Override
                public void onComplete() {
                    if (mActivity.mAdapter.getItemCount() > 0) {
                        mActivity.mLoadingLayout.showContent();
                    } else mActivity.mLoadingLayout.showEmpty();
                }
            });
    }

}
