package com.kanglai.push.ui.vm;

import android.app.Application;
import android.text.Editable;
import android.text.InputFilter;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.Pair;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.lifecycle.LifecycleOwner;

import com.blankj.utilcode.util.CollectionUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.Utils;
import com.dolphin.core.http.api.ResultResponse;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.kanglai.push.R;
import com.kanglai.push.app.AppApplication;
import com.kanglai.push.di.component.DaggerServiceComponent;
import com.kanglai.push.entity.PushType;
import com.kanglai.push.service.PushService;
import com.kanglai.push.ui.activity.PushTypeActivity;
import com.kanglai.push.util.ViewFilterUtil;
import com.kongzue.dialogx.dialogs.FullScreenDialog;
import com.kongzue.dialogx.dialogs.MessageDialog;
import com.kongzue.dialogx.interfaces.OnBindView;
import com.suke.widget.SwitchButton;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import io.reactivex.observers.DisposableObserver;

public class PushTypeViewModel extends ToolbarViewModel<PushTypeActivity>{

    @Inject
    PushService pushService;

    public ArrayList<PushType> mList = new ArrayList<>();;
    public PushTypeViewModel(@NonNull Application application) {
        super(application);
        DaggerServiceComponent
                .builder()
                .appComponent(((AppApplication) Utils.getApp().getApplicationContext()).appComponent)
                .build().inject(this);
    }

    @Override
    public void onCreate(@NonNull LifecycleOwner owner) {
        super.onCreate(owner);
        super.setTitleText("推送管理");
        super.setRightIconVisible(View.VISIBLE);
    }

    /** 右侧按钮点击事件*/
    @Override
    protected void rightIconOnClick() {
        super.rightIconOnClick();
        FullScreenDialog.show(new OnBindView<FullScreenDialog>(R.layout.item_full_dialog_push) {
            @Override
            public void onBind(FullScreenDialog dialog, View v) {
                PushType pushType = new PushType();
                TextView tit = v.findViewById(R.id.dialog_tit);     // 标题
                tit.setText("新增推送类型");
                EditText input = v.findViewById(R.id.edit_type_name);   //  类型名称
                input.setFilters(new InputFilter[]{ViewFilterUtil.getNoSpecialCharactersFilter()});  // 去空格、换行符等特殊符号
                input.addTextChangedListener(new TextWatcher() {
                    @Override
                    public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

                    }

                    @Override
                    public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

                    }

                    @Override
                    public void afterTextChanged(Editable editable) {
                        pushType.setName(String.valueOf(editable).trim());
                    }
                });
                SwitchButton switch01 = v.findViewById(R.id.switch_button_01);  // 声音
                SwitchButton switch02 = v.findViewById(R.id.switch_button_02);  // 震动
                SwitchButton switch03 = v.findViewById(R.id.switch_button_03);  // 闪光
                SwitchButton switch04 = v.findViewById(R.id.switch_button_04);  // 语音转文字
                switch01.setOnCheckedChangeListener(new SwitchButton.OnCheckedChangeListener() {
                    @Override
                    public void onCheckedChanged(SwitchButton view, boolean isChecked) {
                        pushType.setPlaySound((isChecked ? "1" : "0"));
                    }
                });
                switch02.setOnCheckedChangeListener(new SwitchButton.OnCheckedChangeListener() {
                    @Override
                    public void onCheckedChanged(SwitchButton view, boolean isChecked) {
                        pushType.setPlayVibrate((isChecked ? "1" : "0"));
                    }
                });
                switch03.setOnCheckedChangeListener(new SwitchButton.OnCheckedChangeListener() {
                    @Override
                    public void onCheckedChanged(SwitchButton view, boolean isChecked) {
                        pushType.setPlayLights((isChecked ? "1" : "0"));
                    }
                });
                switch04.setOnCheckedChangeListener(new SwitchButton.OnCheckedChangeListener() {
                    @Override
                    public void onCheckedChanged(SwitchButton view, boolean isChecked) {
                        pushType.setPlayToText((isChecked ? "1" : "0"));
                    }
                });
                Button submit = v.findViewById(R.id.dialog_type_button);
                submit.setOnClickListener(view -> submit(pushType, dialog));
            }
        });
    }


    /**
     * 新增数据
     * @param pushType
     * @param dialog
     */
    private void submit(PushType pushType, FullScreenDialog dialog) {
        if (TextUtils.isEmpty(pushType.getName())) ToastUtil.showBottomWarn("推送类型名称不能为空");
        else {
            pushService.savePushType(pushType)
                .compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse>() {
                    @Override
                    public void onNext(ResultResponse R) {
                        if (R.getCode() == R.SUCCESS) {
                            ToastUtil.showBottomSuccess("操作成功");
                            loadPushTypeList(); // 加载列表数据
                            dialog.dismiss();   // 隐藏弹出框
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {

                    }
                });
        }
    }

    /** 获取推送类型列表数据*/
    public void loadPushTypeList() {
        pushService.getPushTypeList(MapUtils.newHashMap(
                Pair.create("current", pageCurrent),
                Pair.create("size", "10")
                )).compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse<List<PushType>>>() {
                    @Override
                    public void onNext(ResultResponse<List<PushType>> R) {
                        if (R.getCode() == R.SUCCESS) {
                            mList.clear();
                            mList.addAll(R.getData());
                            mActivity.mAdapter.refresh(mList);
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {

                    }
                });
    }

    /** 设置默认类型*/
    public void setDefaultType(PushType item, MessageDialog v) {
        pushService.setDefaultType(
                MapUtils.newHashMap(
                        Pair.create("defaultType", "1"),
                        Pair.create("id", item.getId())
                    )).compose(RxUtil.schedulersTransformer())
                .compose(RxUtil.exceptionTransformer())
                .doOnSubscribe(this)
                .subscribe(new DisposableObserver<ResultResponse>() {
                    @Override
                    public void onNext(ResultResponse R) {
                        if (R.getCode() == R.SUCCESS) {
                            ToastUtil.showBottomSuccess("操作成功");
                            v.dismiss();
                            loadPushTypeList();
                        }else ToastUtil.showBottomWarn(R.getMsg());
                    }

                    @Override
                    public void onError(Throwable e) {
                        ExceptionHandle.baseExceptionMsg(e);
                    }

                    @Override
                    public void onComplete() {

                    }
                });
    }

    /** 更新指定一项数据*/
    public void updateType(PushType pushType) {
    }
}
