package com.kanglai.push.util;

import android.graphics.Color;
import android.os.CountDownTimer;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.style.ForegroundColorSpan;
import android.widget.TextView;

/**
 * 验证码60秒 工具类
 * @Author: liusixiang007
 * @since: 2023/12/26
 */
public class CountDownTimerUtils extends CountDownTimer {

    private TextView mTextView;

    /**
     *
     * @param millisInFuture    从调用{@link #start()}到倒计时完成并调用{@link #onFinish()}的未来millis数
     * @param countDownInterval 接收{@link #onTick(long)}回调的间隔。
     */
    public CountDownTimerUtils(TextView textView ,long millisInFuture, long countDownInterval) {
        super(millisInFuture, countDownInterval);
        this.mTextView = textView;  // 界面显示数据
    }

    @Override
    public void onTick(long millisUntilFinished) {
        mTextView.setClickable(false);  // 设置为不可点击状态
        mTextView.setText(millisUntilFinished / 1000 + "s可重新发送");   // 设置倒计时
        int length = mTextView.getText().length();
        SpannableString spannableString = new SpannableString(mTextView.getText().toString());  // SpannableString - 显示文本
        ForegroundColorSpan span = new ForegroundColorSpan(Color.GRAY);  // 为文本段落添加颜色
        spannableString.setSpan(span, 0, length, Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
        mTextView.setText(spannableString);
    }

    @Override
    public void onFinish() {
        mTextView.setTextColor(Color.BLACK); // 设置文字颜色
        mTextView.setText("获取验证码");
        mTextView.setClickable(true);//重新获得点击
    }
}
