package com.kanglai.push.util;

import com.google.gson.JsonArray;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonNull;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonPrimitive;
import com.google.gson.internal.LinkedTreeMap;
import com.kanglai.push.entity.PushChatMessage;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Gson 反序列化器
 * @Author: liusixiang007
 * @since: 2024/1/6
 */
public class CustomDeserializer implements JsonDeserializer<Map<String, Object>> {
    @Override
    public Map<String, Object> deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context) throws JsonParseException {
        Map<String, Object> treeMap = new LinkedTreeMap<>();
        JsonObject jsonObject = json.getAsJsonObject();
        Set<Map.Entry<String, JsonElement>> entrySet = jsonObject.entrySet();
        for (Map.Entry<String, JsonElement> entry : entrySet) {
            JsonElement ot = entry.getValue();
            if (ot == JsonNull.INSTANCE) { // null字符串转换成真实的null
                treeMap.put(entry.getKey(), null);
            } else if (ot instanceof JsonPrimitive) { // 取内部值，数字类型的会在内部被变成字符串
                treeMap.put(entry.getKey(), ot.getAsString());
            } else if (ot instanceof JsonObject) { // 对map对象再次做转换，把 JsonPrimitive 转成普通的数据
                treeMap.put(entry.getKey(), deserialize(ot, typeOfT, context));
            } else if (ot instanceof JsonArray) { // 对map list 对象再次做转换，把 JsonPrimitive 转成普通的数据
                List<Map<String, Object>> treeMapList = new ArrayList<>();
                for (JsonElement jsonElement : ot.getAsJsonArray()) {
                    treeMapList.add((Map<String, Object>) deserialize(jsonElement, typeOfT, context));
                }
                treeMap.put(entry.getKey(), treeMapList);
            } else {
                treeMap.put(entry.getKey(), ot);
            }
        }
        return treeMap;
    }
}
