package com.kanglai.push.util;

import android.app.ActivityManager;
import android.content.Context;

import com.blankj.utilcode.util.CacheDiskUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.MapUtils;
import com.blankj.utilcode.util.StringUtils;
import com.dolphin.core.http.HttpRequest;
import com.dolphin.core.http.exception.ExceptionHandle;
import com.dolphin.core.util.PermissionUtil;
import com.dolphin.core.util.RxUtil;
import com.dolphin.core.util.ToastUtil;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.kanglai.push.constant.CacheConstant;
import com.kanglai.push.entity.LocalPushChatMsg;
import com.kanglai.push.entity.User;
import com.kanglai.push.ui.vm.LoginViewModel;

import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.Objects;

import lombok.experimental.UtilityClass;

/**
 * 聊天页 本地缓存 记录工具类
 *
 * 查询缓存的键名： userid + type
 *
 *
 * @Author: liusixiang007
 * @since: 2024/1/11
 */
@UtilityClass
public class HistoryMsgUtil {

    private User user;

    /** 加载用户缓存 */
    public void loadUser(){   // 初始化用户信息
        user = CacheDiskUtils.getInstance().getParcelable(CacheConstant.USER_INFO, User.CREATOR, new User());
    }

    /** 检查id是否消失 */
    public Boolean noneId(){
        if (StringUtils.isEmpty(user.getId())) return true;
        else return false;
    }

    /** 获取缓存的列表数据 */
    public HashMap<String, LocalPushChatMsg> select() {
        String localHistoryMsgJson = CacheDiskUtils.getInstance().getString(user.getId().concat(user.getUserType())  ,"");
        Type type = new TypeToken<HashMap<String, LocalPushChatMsg>>(){}.getType();
        return new Gson().fromJson(localHistoryMsgJson, type);
    }

    /**
     * 更新缓存数据
     *
     * @param key 对应用户id
     * @param value 需要更新的值
     */
    public void put(String key,LocalPushChatMsg value) {
        Objects.requireNonNull(key);
        String localHistoryMsgJson = CacheDiskUtils.getInstance().getString(user.getId().concat(user.getUserType())  ,"");
        Type type = new TypeToken<HashMap<String, LocalPushChatMsg>>(){}.getType();
        HashMap<String, LocalPushChatMsg> localHistoryMsg = Objects.requireNonNullElse(new Gson().fromJson(localHistoryMsgJson, type), MapUtils.newHashMap());
        localHistoryMsg.put(key, value);
        String newLocalHistoryMsgJson = new Gson().toJson(localHistoryMsg);
        CacheDiskUtils.getInstance().put(user.getId().concat(user.getUserType()), newLocalHistoryMsgJson);
    }

    /**
     * 更具id删除指定的缓存数据
     *
     * @param key
     */
    public void del(String key){
        Objects.requireNonNull(key);
        String localHistoryMsgJson = CacheDiskUtils.getInstance().getString(user.getId().concat(user.getUserType())  ,"");
        Type type = new TypeToken<HashMap<String, LocalPushChatMsg>>(){}.getType();
        HashMap<String, LocalPushChatMsg> localHistoryMsg = Objects.requireNonNullElse(new Gson().fromJson(localHistoryMsgJson, type), MapUtils.newHashMap());
        localHistoryMsg.remove(key);
        String newLocalHistoryMsgJson = new Gson().toJson(localHistoryMsg);
        CacheDiskUtils.getInstance().put(user.getId().concat(user.getUserType()), newLocalHistoryMsgJson);
    }

    /** 清空聊天记录 */
    public void clean() {
        CacheDiskUtils.getInstance().remove(user.getId().concat(user.getUserType()));
    }

    /** 检测服务是否进行 */
    public boolean isServiceRunning(Context context, Class<?> serviceClass){
        ActivityManager manager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        for (ActivityManager.RunningServiceInfo service : manager.getRunningServices(Integer.MAX_VALUE)){
            LogUtils.d(service.service.getClassName());
            if (serviceClass.getName().equals(service.service.getClassName())){
                return true;
            }
        }
        return false;
    }

}
