package com.kanglai.push.util;

import android.app.Activity;
import android.content.Context;
import android.content.ContextWrapper;

import org.intellij.lang.annotations.RegExp;

import java.util.regex.Pattern;

/**
 *<p>
 * 图片选择器加载工具类
 *</p>
 *
 * @Author: entfrm开发团队-王翔
 * @since: 2022/11/23
 */
public class ImageLoaderUtil {

    public static boolean assertValidRequest(Context context) {
        if (context instanceof Activity) {
            Activity activity = (Activity) context;
            return !isDestroy(activity);
        } else if (context instanceof ContextWrapper) {
            ContextWrapper contextWrapper = (ContextWrapper) context;
            if (contextWrapper.getBaseContext() instanceof Activity) {
                Activity activity = (Activity) contextWrapper.getBaseContext();
                return !isDestroy(activity);
            }
        }
        return true;
    }

    private static boolean isDestroy(Activity activity) {
        if (activity == null) {
            return true;
        }
        return activity.isFinishing() || activity.isDestroyed();
    }

    /**
     * 判断字符串是否为url
     *
     * @Author: liusixiang007
     * @since: 2023/4/20
     */
    private static boolean isURL(String val) {
        String regex = "^((https|http|ftp|rtsp|mms)?://)" // 协议
                + "?(([0-9]{1,3}\\.){3}[0-9]{1,3}" // IP地址
                + "|" // 或者域名
                + "([\\w-]+\\.)+" // 子域名
                + "[a-zA-Z]{2,15}" // 顶级域名
                + "(:[0-9]{1,5})?" // 端口号
                + "(/[\\w-./?%&=]*)?$"; // 路径以及后面的查询参数和锚点
        Pattern pattern = Pattern.compile(regex);

        return pattern.matcher(val).matches();
    }
}
