package com.kanglai.push.util;

import android.app.AlarmManager;
import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.os.IBinder;
import android.os.SystemClock;

import androidx.annotation.Nullable;
import androidx.core.app.NotificationCompat;

import com.blankj.utilcode.util.LogUtils;
import com.kanglai.push.R;
import com.kanglai.push.ui.activity.TabBarActivity;


/**
 * 拉起任务栏提示 以及 后台定位广播
 * @Author: liusixiang007
 * @since: 2024/2/28
 */
public class LocationService extends Service{

    /** 在特定时间点或经过一定时间后执行某个操作 */
    private AlarmManager mAlarmManager = null;

    /** 用于表示即将发生的操作的意图 */
    private PendingIntent pendingIntent;

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        NotificationChannel notificationChannel = null;
        Notification.Builder builder;
        NotificationManager manager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);

//        原本是需要判断安装Android的版本号 当前app最低安装版本 就是Android 10
//        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            notificationChannel = new NotificationChannel("CHANNEL_ONE_ID",
                    "CHANNEL_ONE_ID", NotificationManager.IMPORTANCE_HIGH);
            notificationChannel.enableLights(true);
            notificationChannel.setLightColor(Color.RED);
            notificationChannel.setShowBadge(true);
            notificationChannel.setLockscreenVisibility(Notification.VISIBILITY_PUBLIC);
            manager.createNotificationChannel(notificationChannel);
            builder = new Notification.Builder(this, "CHANNEL_ONE_ID");
//        }
        /** 设置任务栏提示 */
//        NotificationCompat.Builder builder = new NotificationCompat.Builder(this);
//        Notification notification = new Notification.Builder(this).setChannelId("CHANNEL_ONE_ID")
//                .setTicker("Nature")
//                .setSmallIcon(R.drawable.umeng_push_notification_default_small_icon)
//                .setContentTitle(getString(R.string.app_name))
//                .setContentText("后台运行中")
//                .build();
//        notification.flags |= Notification.FLAG_NO_CLEAR;
//        startForeground(1, notification);
//        builder.setTicker("友信第一次");
        builder.setSmallIcon(R.drawable.umeng_push_notification_default_small_icon);    // 设置最小图标
        builder.setLargeIcon(BitmapFactory.decodeResource(getResources(), R.drawable.icon_app));    // 设置大图标
        builder.setPriority(Notification.PRIORITY_MAX);   // 设置优先级 优先级低则会被隐藏
        //设置通知时间，默认为系统发出通知的时间，通常不用设置
        builder.setWhen(System.currentTimeMillis());
        builder.setOnlyAlertOnce(true);     // 状态栏是否能被清除
        builder.setContentTitle(getString(R.string.app_name));      // 任务栏标题
        builder.setContentText("后台定位运行中...");   // 友信正在为您安全守护中...
        builder.setContentIntent(PendingIntent.getActivity(this, 0, new Intent(this, TabBarActivity.class),0 )); // 任务栏运行意图 点击跳转
        //设置消息的提醒方式，震动提醒：DEFAULT_VIBRATE     声音提醒：NotificationCompat.DEFAULT_SOUND
        //三色灯提醒NotificationCompat.DEFAULT_LIGHTS     以上三种方式一起：DEFAULT_ALL
        //.setLights(Color.YELLOW, 300, 0)//单独设置呼吸灯，一般三种颜色：红，绿，蓝，经测试，小米支持黄色
        //.setSound(url)//单独设置声音
        //.setVibrate(new long[] { 100, 250, 100, 250, 100, 250 })//单独设置震动
        builder.setDefaults(NotificationCompat.DEFAULT_ALL);    // 通知默认行为 震动、声音 等等
        LogUtils.d("LocationService-----onCreate");
        /**
         * android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP
         * 安卓版本大于5.0时 Notification 的显示等级
         *
         * VISIBILITY_PUBLIC只有在没有锁屏时会显示通知
         * VISIBILITY_PRIVATE任何情况都会显示通知
         * VISIBILITY_SECRET在安全锁和没有锁屏的情况下显示通知
         *
         * */
        builder.setVisibility(Notification.VISIBILITY_PRIVATE);
        startForeground(1, builder.build());    // 开始运行
        mAlarmManager = (AlarmManager) getSystemService(ALARM_SERVICE);  // 设置唤醒周期
    }

    /**
     * 通过startService()的方式启动Service的时候被调用的生命周期
     * @param intent startService时传入的inten
     * @param flags lags有三个可以传入的值：0，START_FLAG_REDELIVERY和START_FLAG_RETRY
     * @param startId 唯一的启动请求
     * @return
     */
    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        int anhour = 10 * 1000;
        /** SystemClock.elapsedRealtime() -获取系统自设备启动以来的时间 */
        long triggerAtMillis = SystemClock.elapsedRealtime() + anhour;
        Intent alarmIntent = new Intent(LocationService.this, AlarmReceiver.class);
        alarmIntent.setAction("youxing_location");
        /** PendingIntent.getBroadcast 广播这个意图 且如果已经存在相同 Intent 的 PendingIntent，则更新当前的 PendingIntent */
        pendingIntent = PendingIntent.getBroadcast(LocationService.this, 0, alarmIntent, PendingIntent.FLAG_UPDATE_CURRENT);
        mAlarmManager.cancel(pendingIntent);
        //  系统版本大于等于8时
        mAlarmManager.setExactAndAllowWhileIdle(AlarmManager.ELAPSED_REALTIME_WAKEUP, triggerAtMillis, pendingIntent);
        // Build.VERSION_CODES.M
        // 当系统版本小于6 - mAlarmManager.setRepeating(AlarmManager.ELAPSED_REALTIME_WAKEUP, triggerAtMillis, TIME_INTERVAL, pendingIntent);
        // 当系统版本小于4 - mAlarmManager.setExact(AlarmManager.ELAPSED_REALTIME_WAKEUP, triggerAtMillis, pendingIntent);
        flags = Service.START_FLAG_REDELIVERY;
//        LogUtils.d("LocationService-----onStartCommand");
        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public void onDestroy() {
        // 指定广播路径
        Intent intent = new Intent("com.kanglai.push.util.AlarmReceiver");
        sendBroadcast(intent);
        super.onDestroy();
    }

}
