package com.kanglai.push.util;

import android.text.InputFilter;
import android.text.Spanned;

import androidx.appcompat.widget.SearchView;

/**
 * 界面自定义控件
 * @Author: liusixiang007
 * @since: 2023/6/27
 */
public class ViewFilterUtil {

    /**
     * 输入框特殊字符限制
     * @return
     */
    public static InputFilter getNoSpecialCharactersFilter(){
        return new InputFilter() {
            @Override
            public CharSequence filter(CharSequence source, int start, int end, Spanned dest, int dstart, int dend) {
                StringBuilder builder = new StringBuilder();
                for (int i = start; i < end ; i++) {
                    char c = source.charAt(i);  // charAt() 获取字符串中指定位置字符
                    /** 过滤空格、换行符和特殊字符 */
                    if (!Character.isWhitespace(c) && !Character.isISOControl(c)) builder.append(c);
                }
                return builder.toString();
            }
        };
    }

    public static void setFilterQueryTextListener(SearchView v){
        v.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String query) {
                return false;
            }

            @Override
            public boolean onQueryTextChange(String newText) {
                return false;
            }
        });
    }

    /** 去空格、换行符 简化版 */
    public static String filterSpecialCharacters(String text) {
        StringBuilder builder = new StringBuilder();

        for (int i = 0; i < text.length(); i++) {
            char c = text.charAt(i);

            // 过滤空格、换行符和特殊字符
            if (!Character.isWhitespace(c) && !Character.isISOControl(c)) {
                builder.append(c);
            }
        }

        return builder.toString();
    }
}
