package com.dolphin.umeng.util;

import android.content.Context;

import com.baidu.tts.client.SpeechError;
import com.baidu.tts.client.SpeechSynthesizer;
import com.baidu.tts.client.SpeechSynthesizerListener;
import com.baidu.tts.client.TtsMode;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.Utils;

import lombok.experimental.UtilityClass;

@UtilityClass
public class BaiduTtsUtil {
    private volatile boolean isInitTts;
    private SpeechSynthesizer mSpeechSynthesizer;
    private static final String BD_APP_ID = "37122313";
    private static final String BD_APP_KEY = "BNrZhvnx5DgX74j847KKrjzn";
    private static final String BD_APP_SECRET_KEY = "bF33QKfD1grBbO1R0Ur1L0A65wGkj3GR";

    static {
        mSpeechSynthesizer = SpeechSynthesizer.getInstance();
        mSpeechSynthesizer.setContext(Utils.getApp().getApplicationContext());
        mSpeechSynthesizer.setSpeechSynthesizerListener(new SpeechSynthesizerListener() {
            @Override
            public void onSynthesizeStart(String s) {
                LogUtils.d("准备开始合成,序列号:" + s);
            }

            @Override
            public void onSynthesizeDataArrived(String s, byte[] bytes, int i, int i1) {}

            @Override
            public void onSynthesizeFinish(String s) {
                LogUtils.d("合成结束回调, 序列号:" + s);
            }

            @Override
            public void onSpeechStart(String s) {   // 开始播放 暂时提高媒体音量
                LogUtils.d("播放开始回调, 序列号:" + s);
            }

            @Override
            public void onSpeechProgressChanged(String s, int i) {
                LogUtils.d("播放进度回调, progress：" + s + ";序列号:" + i);
            }

            @Override
            public void onSpeechFinish(String s) {  // 播放结束之后恢复媒体音量
                LogUtils.d("SpeechSynthesizerListener-onSpeechFinish 播报结束");
                release();  // 播放结束之后就清除占用
            }

            @Override
            public void onError(String s, SpeechError speechError) {
                mSpeechSynthesizer.pause();
                LogUtils.d("语音合成报错 错误id"+s+" 报错原因 "+speechError);
            }
        });
        mSpeechSynthesizer.setAppId(BD_APP_ID);
        mSpeechSynthesizer.setApiKey(BD_APP_KEY , BD_APP_SECRET_KEY);
        // 设置在线发声音人： 0 普通女声（默认） 1 普通男声 2 特别男声 3 情感男声<度逍遥> 4 情感儿童声<度丫丫>
        mSpeechSynthesizer.setParam(SpeechSynthesizer.PARAM_SPEAKER, "0");
        // 设置合成的音量，0-15 ，默认 5
        mSpeechSynthesizer.setParam(SpeechSynthesizer.PARAM_VOLUME, "15");
        // 设置合成的语速，0-15 ，默认 5
        mSpeechSynthesizer.setParam(SpeechSynthesizer.PARAM_SPEED, "5");
        // 设置合成的语调，0-15 ，默认 5
        mSpeechSynthesizer.setParam(SpeechSynthesizer.PARAM_PITCH, "5");
        // 仅需要在线合成并不使用压缩传输
        mSpeechSynthesizer.setParam(SpeechSynthesizer.PARAM_AUDIO_RATE, SpeechSynthesizer.AUDIO_BITRATE_PCM);
        mSpeechSynthesizer.setParam(SpeechSynthesizer.PARAM_AUDIO_ENCODE, SpeechSynthesizer.AUDIO_ENCODE_PCM);
        int result = mSpeechSynthesizer.initTts(TtsMode.ONLINE);
        isInitTts = result == 0;
    }

    public int ttsSpeak(String text) {
        if (mSpeechSynthesizer != null && isInitTts){
            return mSpeechSynthesizer.speak(text);
        } else return 1;
    }

    public void release() {
        if (mSpeechSynthesizer != null){
            mSpeechSynthesizer.stop();
            mSpeechSynthesizer.release();
            isInitTts = false;
            mSpeechSynthesizer = null;
        }
    }
}
