package com.dolphin.umeng.util;

import android.media.AudioManager;
import android.media.MediaPlayer;
import android.net.Uri;
import android.os.PowerManager;
import android.text.TextUtils;

import com.blankj.utilcode.util.Utils;
import com.blankj.utilcode.util.VolumeUtils;
import com.dolphin.umeng.R;

import lombok.experimental.UtilityClass;

/**
 * 媒体声音播放工具类
 * @Author: liusixiang007
 * @since: 2024/1/27
 */
@UtilityClass
public class MultimediaUtil {

    private MediaPlayer mediaPlayer;

    public void playSound(String onlineRingtone) {
        if (TextUtils.isEmpty(onlineRingtone)) {
            mediaPlayer = MediaPlayer.create(Utils.getApp().getApplicationContext(), R.raw.audio_system_hint);
        } else mediaPlayer = MediaPlayer.create(Utils.getApp().getApplicationContext(), Uri.parse(onlineRingtone));
        mediaPlayer.setWakeMode(Utils.getApp().getApplicationContext(), PowerManager.PARTIAL_WAKE_LOCK);
        if (VolumeUtils.getMinVolume(AudioManager.STREAM_MUSIC) == 0) {
            VolumeUtils.setVolume(AudioManager.STREAM_MUSIC, VolumeUtils.getMaxVolume(AudioManager.STREAM_MUSIC)/3, AudioManager.FLAG_PLAY_SOUND);
        }
        mediaPlayer.start();
    }

    public void ttsSpeak(String ttsText) {
        BaiduTtsUtil.ttsSpeak(ttsText);
    }

    public void playSoundAndTtsSpeak(String onlineRingtone, String ttsText) {
        playSound(onlineRingtone);
        mediaPlayer.setOnCompletionListener(mp -> {
            ttsSpeak(ttsText);
        });
    }

}
